@@ -199,7 +199,8 @@ latex_parser::parse (string s, int& i, string stop, int change) {
 	  (s[i] != '$' && s[i] != '}' &&
 	   (i+2>n || s(i,i+2) != "\\]") &&
 	   (i+2>n || s(i,i+2) != "\\)") &&
-	   (i+4>n || s(i,i+4) != "\\end")))) {
+	   (i+4>n || s(i,i+4) != "\\end"))) &&
+	 (stop != "\\egroup" || i+7>n || s(i,i+7) != "\\egroup")) {
     if (lf == 'N' && s[i] != '\n') lf= 'M';
     switch (s[i]) {
     case '~':
@@ -460,6 +461,14 @@ latex_parser::parse_backslash (string s, int& i, int change) {
     i+=6;
     return parse_verbatim (s, i, s(i-1,i), "\\verbatim");
   }
+  if (((i+6)<n) && (s(i,i+4)=="\\url") && s[i+4] != '{' && s[i+4] != ' ') {
+    i+=5;
+    return parse_verbatim (s, i, s(i-1,i), "\\url");
+  }
+  if (((i+7)<n) && (s(i,i+5)=="\\path") && s[i+5] != '{' && s[i+5] != ' ') {
+    i+=6;
+    return parse_verbatim (s, i, s(i-1,i), "\\verbatim");
+  }
   if (((i+29)<n) && (s(i,i+16)=="\\begin{verbatim}")) {
     i+=16;
     return parse_verbatim (s, i, "\\end{verbatim}", "verbatim");
@@ -504,6 +513,21 @@ latex_parser::parse_backslash (string s, int& i, int change) {
     if (i<n && s[i] == '{') { i++; u= parse (s, i, "}"); i++; }
     return tree (TUPLE, "\\href", ss, u);
   }
+  if (((i+8)<n) && (s(i,i+7)=="\\bgroup")) {
+    i+=7;
+    tree t (CONCAT);
+    t << tree (TUPLE, "\\begingroup");
+    t << parse (s, i, "\\egroup", change);
+    t << tree (TUPLE, "\\endgroup");
+    if (((i+8)<n) && (s(i,i+7)=="\\egroup")) i+=7;
+    if ((i<n) && (!is_space (s[i]))) return t;
+    int ln=0;
+    while ((i<n) && is_space (s[i]))
+      if (s[i++]=='\n') ln++;
+    if (ln >= 2) t << "\n";
+    else if (i<n) t << " ";
+    return t;
+  }
 
   /************************ special commands *********************************/
   i++;
@@ -1458,7 +1482,7 @@ latex_parser::parse_alltt (string s, int& i, string end, string env, tree opt)
 * This routine may be used to transform accented characters to the Cork format
 ******************************************************************************/
 
-static char Cork_unaccented[128]= {
+char Cork_unaccented[128]= {
   'A', 'A', 'C', 'C', 'D', 'E', 'E', 'G',
   'L', 'L', ' ', 'N', 'N', ' ', 'O', 'R',
   'R', 'S', 'S', 'S', 'T', 'T', 'U', 'U',
@@ -1477,7 +1501,7 @@ static char Cork_unaccented[128]= {
   ' ', 'u', 'u', 'u', 'u', 'y', ' ', ' '
 };
 
-static char Cork_accent[128]= {
+char Cork_accent[128]= {
   'u' , 'k' , '\'', 'v' , 'v' , 'v' , 'k' , 'u' ,
   '\'', 'v' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
   'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , 'r' ,
@@ -1712,9 +1736,9 @@ get_latex_language (string s) {
   tree langs = concat(); 
   langs << "british" << "bulgarian" << "chinese" << "croatian" << "czech"
         << "danish" << "dutch" << "finnish" << "french" << "german"
-        << "hungarian" << "italian" << "japanese" << "korean" << "polish"
-        << "portuguese" << "romanian" << "russian" << "slovene" << "spanish"
-        << "swedish" << "taiwanese" << "ukrainian";
+        << "greek" << "hungarian" << "italian" << "japanese" << "korean"
+        << "polish" << "portuguese" << "romanian" << "russian"
+        << "slovene" << "spanish" << "swedish" << "taiwanese" << "ukrainian";
 
   for (int i = 0 ; i < N(langs) ; i++)
     if (test(r, 0 , as_string(langs[i])))
