@@ -28,6 +28,12 @@ string format_to_suffix (string format);
 string get_format (string s, string suffix);
 tree   generic_to_tree (string s, string format);
 string tree_to_generic (tree doc, string format);
+array<string> compute_keys (string s, string fm);
+array<string> compute_keys (tree t, string fm);
+array<string> compute_keys (url u);
+scheme_tree compute_index (string s, string fm);
+scheme_tree compute_index (tree t, string fm);
+scheme_tree compute_index (url u);
 
 /*** Texmacs ***/
 tree   texmacs_to_tree (string s);
@@ -79,8 +85,12 @@ tree   tracked_latex_to_texmacs (string s, bool as_pic);
 string conservative_texmacs_to_latex (tree doc, object opts);
 string tracked_texmacs_to_latex (tree doc, object opts);
 tree   conservative_latex_to_texmacs (string s, bool as_pic);
-int    number_latex_errors (url aux);
-int    number_latex_pages (url aux);
+int    get_line_number (string s, int pos);
+int    get_column_number (string s, int pos);
+tree   try_latex_export (tree doc, object opts, url src, url dest);
+int    number_latex_errors (url log);
+tree   get_latex_errors (url log);
+int    number_latex_pages (url log);
 
 /*** Xml / Html / Mathml ***/
 tree   parse_xml (string s);
@@ -90,6 +100,8 @@ tree   upgrade_mathml (tree t);
 
 /*** BibTeX ***/
 tree   parse_bib (string s);
+tree   conservative_bib_import (string olds, tree oldt, string news);
+string conservative_bib_export (tree oldt, string olds, tree newt);
 
 /*** Post corrections ***/
 bool   seems_buggy_html_paste (string s);
