@@ -275,17 +275,21 @@ init_std_drd () {
 	binding (0) -> accessible (1));
   init (PROVIDES, "provides",
 	fixed (1) -> returns_boolean () -> string_type (0));
-  init (VALUE, "value", fixed (1) -> variable (0));
+  init (VALUE, "value", fixed (1) -> variable (0) ->
+        locals (0, "mode", "src"));
   init (QUOTE_VALUE, "quote-value",
-	fixed (1) -> variable (0) -> name ("quoted value"));
+	fixed (1) -> variable (0) -> name ("quoted value") ->
+        locals (0, "mode", "src"));
   init (MACRO, "macro",
 	var_repeat (1, 1, BIFORM) -> argument (0) -> regular (1));
   init (DRD_PROPS, "drd-props",
 	repeat (3, 2) -> name ("drd properties"));
   init (QUOTE_ARG, "quote-arg",
-	repeat (1, 1, BIFORM) -> argument (0) -> name ("quoted argument"));
+	repeat (1, 1, BIFORM) -> argument (0) -> name ("quoted argument") ->
+        locals (0, "mode", "src"));
   init (ARG, "arg",
-	repeat (1, 1, BIFORM) -> argument (0) -> name ("argument"));
+	repeat (1, 1, BIFORM) -> argument (0) -> name ("argument") ->
+        locals (0, "mode", "src"));
   init (COMPOUND, "compound",
 	repeat (1, 1, BIFORM) -> variable (0) -> accessible (1));
   // FIXME: should be refined. The current setting is f.i. needed for "theorem"
@@ -328,6 +332,7 @@ init_std_drd () {
 	fixed (1, 1, BIFORM) -> variable (0) -> regular (1));
   init (EXTERN, "extern",
 	repeat (1, 1, BIFORM) -> code (0) -> regular (1)); // func and args
+  init (VAR_INCLUDE, "include*", fixed (1) -> url_type (0));
   init (INCLUDE, "include", fixed (1) -> url_type (0));
   init (USE_PACKAGE, "use-package", repeat (1, 1) -> string_type (0));
   init (USE_MODULE, "use-module", repeat (1, 1) -> code (0));
@@ -371,6 +376,8 @@ init_std_drd () {
 	fixed (1, 1, BIFORM) -> accessible (0) -> string_type (1));
   init (FIND_FILE, "find-file",
 	var_repeat (1, 1) -> returns_url () -> url_type (0)); // dirs and file
+  init (FIND_FILE_UPWARDS, "find-file-upwards",
+	repeat (1, 1) -> returns_url () -> string_type (0));
   init (IS_TUPLE, "is-tuple",
 	fixed (1) -> returns_boolean () -> regular (0) -> name ("tuple?"));
   init (LOOK_UP, "look-up",
@@ -390,6 +397,7 @@ init_std_drd () {
   init (GREATEREQ, "greatereq",
 	fixed (2) -> returns_boolean () -> regular (0) ->
 	name ("greater or equal"));
+  init (BLEND, "blend", fixed (2) -> returns_color () -> color_type (0));
 
   init (CM_LENGTH, "cm-length", fixed (0) -> returns_length ());
   init (MM_LENGTH, "mm-length", fixed (0) -> returns_length ());
@@ -484,6 +492,10 @@ init_std_drd () {
 	repeat (1, 1, BIFORM) -> returns_adhoc () ->
 	accessible (0) -> code (0) -> name (0, "function") ->
 	accessible (1) -> regular (1) -> name (0, "arguments"));
+  init (OBSERVER, "observer",
+	fixed (1, 1, BIFORM) -> returns_adhoc () ->
+	accessible (0) -> identifier (0) -> name (0, "identifier") ->
+	accessible (1) -> code (1) -> name (1, "call back"));
   init (FIND_ACCESSIBLE, "find-accessible",
 	fixed (1) -> regular (0));
   init (HLINK, "hlink",
@@ -511,6 +523,9 @@ init_std_drd () {
   init (PAGEREF, "pageref",
 	fixed (1) -> name ("page reference") ->
         identifier (0) -> name (0, "id") -> long_name (0, "identifier"));
+  init (GET_ATTACHMENT, "get-attachment",
+	fixed (1) -> name ("get attachment") ->
+        identifier (0) -> name (0, "name"));
   init (WRITE, "write",
 	fixed (1, 1, BIFORM) ->
 	string_type (0) -> name (0, "channel") ->
@@ -824,6 +839,7 @@ init_std_drd () {
   init_var (THE_LABEL, TYPE_REGULAR);
   init_var (THE_TAGS, TYPE_ADHOC);
   init_var (THE_MODULES, TYPE_ADHOC);
+  init_var (WARN_MISSING, TYPE_BOOLEAN);
   init_var (GLOBAL_TITLE, TYPE_STRING);
   init_var (GLOBAL_AUTHOR, TYPE_STRING);
   init_var (GLOBAL_SUBJECT, TYPE_STRING);
@@ -897,6 +913,7 @@ init_std_drd () {
   init_var (PAGE_SCREEN_MARGIN, TYPE_BOOLEAN);
   init_var (PAGE_BREAKING, TYPE_STRING);
   init_var (PAGE_FLEXIBILITY, TYPE_NUMERIC);
+  init_var (PAGE_FIRST, TYPE_NUMERIC);
   init_var (PAGE_NR, TYPE_INTEGER);
   init_var (PAGE_THE_PAGE, TYPE_ADHOC);
   init_var (PAGE_WIDTH, TYPE_LENGTH);
