@@ -28,6 +28,7 @@ string TABULAR ("tabular");
 string THE_LABEL ("the-label");
 string THE_TAGS ("the-tags");
 string THE_MODULES ("the-modules");
+string WARN_MISSING ("warn-missing");
 string GLOBAL_TITLE ("global-title");
 string GLOBAL_AUTHOR ("global-author");
 string GLOBAL_SUBJECT ("global-subject");
@@ -113,6 +114,7 @@ string PAGE_HEIGHT_MARGIN ("page-height-margin");
 string PAGE_SCREEN_MARGIN ("page-screen-margin");
 string PAGE_BREAKING ("page-breaking");
 string PAGE_FLEXIBILITY ("page-flexibility");
+string PAGE_FIRST ("page-first");
 string PAGE_NR ("page-nr");
 string PAGE_THE_PAGE ("page-the-page");
 string PAGE_WIDTH ("page-width");
@@ -169,6 +171,7 @@ string TABLE_RBORDER ("table-rborder");
 string TABLE_BBORDER ("table-bborder");
 string TABLE_TBORDER ("table-tborder");
 string TABLE_HYPHEN ("table-hyphen");
+string TABLE_BLOCK ("table-block");
 string TABLE_MIN_ROWS ("table-min-rows");
 string TABLE_MIN_COLS ("table-min-cols");
 string TABLE_MAX_ROWS ("table-max-rows");
