@@ -171,7 +171,7 @@ edit_main_rep::get_metadata (string kind) {
       !is_none (resolve_in_path ("finger")) &&
       !is_none (resolve_in_path ("sed"))) {
     string val= var_eval_system ("finger `whoami` | sed -e '/Name/!d' -e 's/.*Name: //'");
-    if (N(val) > 1) return val;
+    if (N(val) > 1) return utf8_to_cork (val);
   }
 #endif
   return "";
@@ -204,7 +204,7 @@ use_ps () {
 }
 
 void
-edit_main_rep::print_bis (url name, bool conform, int first, int last) {
+edit_main_rep::print (url name, bool conform, int first, int last) {
   bool ps  = (suffix (name) == "ps");
   bool pdf = (suffix (name) == "pdf");
   url  orig= resolve (name, "");
@@ -267,21 +267,23 @@ edit_main_rep::print_bis (url name, bool conform, int first, int last) {
   ren= printer (name, dpi, pages, page_type, landsc, w/cm, h/cm);
 #endif
   
-  int i;
-  ren->set_metadata ("title", get_metadata ("title"));
-  ren->set_metadata ("author", get_metadata ("author"));
-  ren->set_metadata ("subject", get_metadata ("subject"));
-  for (i=start; i<end; i++) {
-    tree bg= env->read (BG_COLOR);
-    ren->set_background (bg);
-    if (bg != "white")
-      ren->clear_pattern (0, (SI) -h, (SI) w, 0);
+  if (ren->is_started ()) {
+    int i;
+    ren->set_metadata ("title", get_metadata ("title"));
+    ren->set_metadata ("author", get_metadata ("author"));
+    ren->set_metadata ("subject", get_metadata ("subject"));
+    for (i=start; i<end; i++) {
+      tree bg= env->read (BG_COLOR);
+      ren->set_background (bg);
+      if (bg != "white")
+        ren->clear_pattern (0, (SI) -h, (SI) w, 0);
 
-    rectangles rs;
-    the_box[0]->sx(i)= 0;
-    the_box[0]->sy(i)= 0;
-    the_box[0][i]->redraw (ren, path (0), rs);
-    if (i<end-1) ren->next_page ();
+      rectangles rs;
+      the_box[0]->sx(i)= 0;
+      the_box[0]->sy(i)= 0;
+      the_box[0][i]->redraw (ren, path (0), rs);
+      if (i<end-1) ren->next_page ();
+    }
   }
   tm_delete (ren);
 
@@ -294,29 +296,18 @@ edit_main_rep::print_bis (url name, bool conform, int first, int last) {
     gs_to_ps (name, orig, landsc, h/cm, w/cm);
     ::remove (name);
   }
+  if (ps || pdf)
+    if (get_preference ("texmacs->pdf:check", "off") == "on") {
+      system_wait ("Checking exported file for correctness", "please wait");
+      gs_check (orig);
+    }
 #endif
 }
 
-void
-edit_main_rep::print (url name, bool conform, int first, int last) {
-  if (inside ("screens")) {
-    tree style= copy (get_style ());
-    tree init = copy (get_init ());
-    tree t    = copy (subtree (et, rp));
-    call ("dynamic-make-slides");
-    apply_changes ();
-    print_bis (name, conform, first, last);
-    set_style (style);
-    set_init (init);
-    assign (rp, t);
-    apply_changes ();
-  }
-  else print_bis (name, conform, first, last);
-}
-
 void
 edit_main_rep::print_to_file (url name, string first, string last) {
   print (name, false, as_int (first), as_int (last));
+  set_message ("Done printing", "print to file");
 }
 
 void
@@ -332,6 +323,7 @@ edit_main_rep::print_buffer (string first, string last) {
   target= url_temp (".ps"); 
   print (target, false, as_int (first), as_int (last));
   system (printing_cmd, target);  // Send the document to the printer
+  set_message ("Done printing", "print buffer");
  #endif
   ::remove (target);
 }
