@@ -9,6 +9,7 @@
 * in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
+#include <climits>
 #include "edit_typeset.hpp"
 #include "tm_buffer.hpp"
 #include "convert.hpp"
@@ -36,7 +37,8 @@ edit_typeset_rep::edit_typeset_rep ():
   stydef (UNINIT), pre (UNINIT), init (UNINIT), fin (UNINIT),
   env (drd, buf->buf->master,
        buf->data->ref, (buf->prj==NULL? buf->data->ref: buf->prj->data->ref),
-       buf->data->aux, (buf->prj==NULL? buf->data->aux: buf->prj->data->aux)),
+       buf->data->aux, (buf->prj==NULL? buf->data->aux: buf->prj->data->aux),
+       buf->data->att, (buf->prj==NULL? buf->data->att: buf->prj->data->att)),
   ttt (new_typesetter (env, subtree (et, rp), reverse (rp))) {}
 edit_typeset_rep::~edit_typeset_rep () { delete_typesetter (ttt); }
 
@@ -50,6 +52,12 @@ edit_typeset_rep::set_data (new_data data) {
   add_init (data->init);
   notify_change (THE_DECORATIONS);
   typeset_invalidate_env ();
+  iterator<string> it = iterate (data->att);
+  while (it->busy()) {
+    string key= it->next ();
+    (void) call (string ("notify-set-attachment"),
+                 buf->buf->name, key, data->att [key]);
+  }
 }
 
 void
@@ -69,6 +77,32 @@ hashmap<string,tree> edit_typeset_rep::get_att () { return buf->data->att; }
 void edit_typeset_rep::set_fin (hashmap<string,tree> H) { fin= H; }
 void edit_typeset_rep::set_att (hashmap<string,tree> H) { buf->data->att= H; }
 
+tree
+edit_typeset_rep::get_att (string key) {
+  return buf->data->att[key];
+}
+
+void
+edit_typeset_rep::set_att (string key, tree im) {
+  buf->data->att (key)= im;
+}
+
+void
+edit_typeset_rep::reset_att (string key) {
+  buf->data->att->reset (key);
+}
+
+array<string>
+edit_typeset_rep::list_atts () {
+  tree a= (tree) buf->data->att;
+  array<string> v;
+  int i, n= N(a);
+  for (i=0; i<n; i++)
+    v << a[i][0]->label;
+  merge_sort (v);
+  return v;
+}
+
 void
 edit_typeset_rep::set_init (hashmap<string,tree> H) {
   init= hashmap<string,tree> (UNINIT);
@@ -86,7 +120,6 @@ void
 edit_typeset_rep::clear_local_info () {
   buf->data->ref= hashmap<string,tree> ();
   buf->data->aux= hashmap<string,tree> ();
-  buf->data->att= hashmap<string,tree> ();
 }
 
 /******************************************************************************
@@ -214,6 +247,45 @@ restricted_exec (edit_env env, tree t, int end) {
     env->exec (t[0]);
 }
 
+static tree
+filter_format (tree fm, int i, int n) {
+  array<tree> r;
+  for (int k=0; k<N(fm); k++)
+    if (is_func (fm[k], CWITH) && N(fm[k]) >= 4 &&
+        is_int (fm[k][0]) && is_int (fm[k][1])) {
+      int j1= as_int (fm[k][0]->label);
+      int j2= as_int (fm[k][1]->label);
+      if (j1 > 0) j1--; else j1 += n;
+      if (j2 > 0) j2--; else j2 += n;
+      if (i >= j1 && i <= j2) r << fm[k] (2, N(fm[k]));
+    }
+  return tree (TFORMAT, r);
+}
+
+static void
+table_descend (tree& t, path& p, tree& fm) {
+  while (!is_nil (p)) {
+    if (L(t) == TFORMAT && p->item == N(t) - 1) {
+      array<tree> r;
+      for (int k=0; k<N(t)-1; k++)
+        if (is_func (t[k], CWITH, 6) &&
+            is_atomic (t[k][4]) &&
+            !starts (t[k][4]->label, "cell-"))
+          r << t[k];
+      fm= tree (TFORMAT, r);
+      t= t[N(t)-1];
+      p= p->next;
+    }
+    else if ((L(t) == TABLE || L(t) == ROW) &&
+             p->item >= 0 && p->item < N(t)) {
+      fm= filter_format (fm, p->item, N(t));
+      t= t[p->item];
+      p= p->next;
+    }
+    else break;
+  }
+}
+
 void
 edit_typeset_rep::typeset_exec_until (path p) {
   //time_t t1= texmacs_time ();
@@ -243,17 +315,27 @@ edit_typeset_rep::typeset_exec_until (path p) {
     while (!is_nil (q)) {
       int i= q->item;
       restricted_exec (env, t, i);
-      tree w= drd->get_env_child (t, i, tree (ATTR));
-      if (w == "") break;
-      //cout << "t= " << t << "\n";
-      //cout << "i= " << i << "\n";
-      //cout << "w= " << w << "\n";
-      for (int j=0; j<N(w); j+=2) {
-	//cout << w[j] << " := " << env->exec (w[j+1]) << "\n";
-	env->write (w[j]->label, env->exec (w[j+1]));
+      if (L(t) == TFORMAT && i == N(t) - 1) {
+        tree fm= tree (TFORMAT);
+        table_descend (t, q, fm);
+        if (!is_nil (q))
+          for (int k=0; k<N(fm); k++)
+            if (is_func (fm[k], CWITH, 2))
+              env->write (fm[k][0]->label, fm[k][1]);
+      }
+      else {
+        tree w= drd->get_env_child (t, i, tree (ATTR));
+        if (w == "") break;
+        //cout << "t= " << t << "\n";
+        //cout << "i= " << i << "\n";
+        //cout << "w= " << w << "\n";
+        for (int j=0; j<N(w); j+=2) {
+          //cout << w[j] << " := " << env->exec (w[j+1]) << "\n";
+          env->write (w[j]->label, env->exec (w[j+1]));
+        }
+        t= t[i];
+        q= q->next;
       }
-      t= t[i];
-      q= q->next;
     }
     if (env->read (PREAMBLE) == "true")
       env->write (MODE, "src");
@@ -503,8 +585,12 @@ value_to_compound (tree t, hashmap<string,tree> h) {
 tree
 edit_typeset_rep::exec_latex (tree t, path p) {
   t= convert_OTS1_symbols_to_universal_encoding (t);
-  string pref= "texmacs->latex:expand-macros";
-  if (as_string (call ("get-preference", pref)) != "on") return t;
+  bool expand_unknown_macros= "on" == as_string (
+      call ("get-preference", "texmacs->latex:expand-macros"));
+  bool expand_user_macro= "on" == as_string (
+      call ("get-preference", "texmacs->latex:expand-user-macros"));
+  if (!expand_unknown_macros && !expand_user_macro)
+    return t;
   if (p == (rp * 0)) typeset_preamble ();
   typeset_exec_until (p);
   hashmap<string,tree> H= copy (cur[p]);
@@ -514,14 +600,19 @@ edit_typeset_rep::exec_latex (tree t, path p) {
   tree patch= as_tree (call ("stree->tree", call ("tmtex-env-patch", t, l)));
   hashmap<string,tree> P (UNINIT, patch);
   H->join (P);
-  if (is_document (t) && is_compound (t[0], "hide-preamble")) {
+
+  if (!expand_user_macro &&
+      is_document (t) && is_compound (t[0], "hide-preamble")) {
     tree r= copy (t);
     r[0]= "";
     r= exec (value_to_compound (r, P), H, false);
     r[0]= exec (t[0], H, false);
     return r;
   }
-  else return exec (value_to_compound (t, P), H, false);
+  else {
+    tree r= exec (value_to_compound (t, P), H, false);
+    return r;
+  }
 }
 
 tree
@@ -643,8 +734,8 @@ report_redefined (array<tree> redefined) {
 
 void
 edit_typeset_rep::typeset (SI& x1, SI& y1, SI& x2, SI& y2) {
-  int missing_nr= 1000000;
-  int redefined_nr= 1000000;
+  int missing_nr= INT_MAX;
+  int redefined_nr= INT_MAX;
   while (true) {
     typeset_sub (x1, y1, x2, y2);
     if (!env->complete) break;
