@@ -383,6 +383,7 @@ edit_interface_rep::compute_env_rects (path p, rectangles& rs, bool recurse) {
            is_graphical (st) ||
            (is_func (st, WITH) && is_graphical (st[N(st)-1])) ||
            (is_func (st, WITH) && is_graphical_text (st[N(st)-1])) ||
+           is_compound (st, "shared", 3) ||
            (is_compound (st, "math", 1) &&
             is_compound (subtree (et, path_up (p)), "input")))
     compute_env_rects (path_up (p), rs, recurse);
@@ -465,7 +466,15 @@ edit_interface_rep::apply_changes () {
         { SERVER (side_tools (0, "(vertical (link texmacs-side-tools))")); }
       SERVER (bottom_tools (0, "(vertical (link texmacs-bottom-tools))"));
       set_footer ();
-      if (has_current_window ()) concrete_window()->set_modified (need_save());
+      if (has_current_window ()) {
+        array<url> ws= buffer_to_windows (
+                         window_to_buffer (
+                           abstract_window (concrete_window ())));
+        int n= N(ws);
+        bool ns= need_save ();
+        for (int i=0; i<n; i++)
+          concrete_window (ws[i])->set_modified (ns);
+      }
       if (!gui_interrupted ()) drd_update ();
       cache_memorize ();
       last_update= last_change;
@@ -543,7 +552,8 @@ edit_interface_rep::apply_changes () {
       rectangles visible (rectangle (vx1, vy1, vx2, vy2));
       for (int i=0; i<N(alt_selection_rects); i++)
         invalidate (alt_selection_rects[i] & visible);
-      range_set alt_sel= get_alt_selection ("alternate");
+      range_set alt_sel= append (get_alt_selection ("alternate"),
+                                 get_alt_selection ("brackets"));
       if (is_empty (alt_sel))
         alt_selection_rects= array<rectangles> ();
     }
@@ -718,7 +728,8 @@ edit_interface_rep::apply_changes () {
       selection_rects= rs;
       invalidate (selection_rects);
     }
-    range_set alt_sel= get_alt_selection ("alternate");
+    range_set alt_sel= append (get_alt_selection ("alternate"),
+                               get_alt_selection ("brackets"));
     if (!is_empty (alt_sel)) {
       alt_selection_rects= array<rectangles> ();
       for (int i=0; i+1<N(alt_sel); i+=2) {
