@@ -93,6 +93,29 @@ edit_select_rep::semantic_active (path q) {
   return false;
 }
 
+static bool
+right_most_inside (path p, tree t) {
+  if (is_atomic (t)) return p == path (N(t->label));
+  else if (p == path (1)) return true;
+  if (is_nil (p) || is_nil (p->next)) return false;
+  if (N(t) == 1 && p->item == 0)
+    return right_most_inside (p->next, t[0]);
+  if (is_func (t, WIDE, 2))
+    return p->item == 0 && right_most_inside (p->next, t[0]);
+  if (is_func (t, CONCAT) || is_func (t, DOCUMENT))
+    return p->item == N(t) - 1 && right_most_inside (p->next, t[N(t) - 1]);
+  return false;
+}
+
+static path
+correct_right_most_inside (path p, tree t) {
+  if (is_nil (p) || is_nil (p->next) || is_atomic (t)) return p;
+  if (0 > p->item || p->item >= N(t)) return p;
+  p= path (p->item, correct_right_most_inside (p->next, t[p->item]));
+  if (right_most_inside (p, t)) return path (1);
+  else return p;
+}
+
 bool
 edit_select_rep::semantic_select (path p, path& q1, path& q2, int mode) {
   if (!semantic_active (p)) return false;
@@ -107,6 +130,7 @@ edit_select_rep::semantic_select (path p, path& q1, path& q2, int mode) {
   path cp= (p <= tp? tp / p: path ());
   tree st= subtree (et, p);
   bool ret= packrat_select (lan, "Main", st, cp, p1, p2, mode);
+  p1= correct_right_most_inside (p1, st);
   if (ret) {
     q1= p * p1;
     q2= p * p2;
@@ -767,11 +791,12 @@ edit_select_rep::raw_cut (path p1, path p2) {
   if (is_func (t, TFORMAT) || is_func (t, TABLE) || is_func (t, ROW)) {
     path fp= ::table_search_format (et, p);
     tree st= subtree (et, fp);
-    int row1, col1, row2, col2;
+    int row1, col1, row2, col2, nr_rows, nr_cols;
     table_search_coordinates (st, tail (p1, N(fp)), row1, col1);
     table_search_coordinates (st, tail (p2, N(fp)), row2, col2);
     if (row1>row2) { int tmp= row1; row1= row2; row2= tmp; }
     if (col1>col2) { int tmp= col1; col1= col2; col2= tmp; }
+    table_get_extents (fp, nr_rows, nr_cols);
 
     int i, j;
     for (i=row1; i<=row2; i++)
@@ -787,6 +812,10 @@ edit_select_rep::raw_cut (path p1, path p2) {
       table_del_format (fp, row1+1, col1+1, row2+1, col2+1, "");
 
     if (fp == search_format ()) {
+      if (col1 == 0 && col2 == nr_cols-1 && row2 > row1)
+        table_remove (fp, row1 + 1, col1, row2 - row1, 0);      
+      else if (row1 == 0 && row2 == nr_rows-1 && col2 > col1)
+        table_remove (fp, row1, col1 + 1, 0, col2 - col1);      
       table_correct_block_content ();
       table_resize_notify ();
     }
@@ -916,6 +945,7 @@ edit_select_rep::focus_search (path p, bool skip_flag, bool up_flag) {
       is_func (st, CELL) ||
       is_compound (st, "shown") ||
       is_func (st, HIDDEN) ||
+      is_compound (st, "shared") ||
       up_flag)
     return focus_search (path_up (p), skip_flag, false);
   return p;
