@@ -122,11 +122,16 @@ protected:
   virtual void animate () = 0;
   virtual path search_format () = 0;
   virtual path search_format (int& row, int& col) = 0;
+  virtual void table_get_extents (path fp, int& nr_rows, int& nr_cols) = 0;
   virtual void table_bound (path fp, int& i1, int& j1, int& i2, int& j2) = 0;
   virtual tree table_get_subtable (path p, int i1, int j1, int i2, int j2) = 0;
   virtual void table_write_subtable (path fp, int row, int col, tree subt) = 0;
   virtual void table_del_format (path fp, int I1, int J1,
 				 int I2, int J2, string var) = 0;
+  virtual void table_insert (path fp, int row, int col,
+                             int nr_rows, int nr_cols) = 0;
+  virtual void table_remove (path fp, int row, int col,
+                             int nr_rows, int nr_cols) = 0;
 
 public:
   editor_rep ();
@@ -190,6 +195,7 @@ public:
 
   /* public routines from edit_cursor */
   virtual path make_cursor_accessible (path p, bool forwards) = 0;
+  virtual bool cursor_is_accessible () = 0;
   virtual void show_cursor_if_hidden () = 0;
   virtual void go_to (SI x, SI y, bool absolute= true) = 0;
   virtual void go_left_physical () = 0;
@@ -287,6 +293,10 @@ public:
   virtual tree     get_init_all () = 0;
   virtual void     init_env (string var, tree by) = 0;
   virtual void     init_default (string var) = 0;
+  virtual tree     get_att (string key) = 0;
+  virtual void     set_att (string key, tree im) = 0;
+  virtual void     reset_att (string key) = 0;
+  virtual array<string> list_atts () = 0;
   virtual void     typeset_invalidate (path p) = 0;
   virtual void     typeset_invalidate_all () = 0;
 
@@ -378,12 +388,15 @@ public:
   virtual void   table_remove_column (bool forward, bool flag= false) = 0;
   virtual int    table_nr_rows () = 0;
   virtual int    table_nr_columns () = 0;
+  virtual array<int> table_get_extents () = 0;
   virtual void   table_set_extents (int rows, int cols) = 0;
   virtual int    table_which_row () = 0;
   virtual int    table_which_column () = 0;
+  virtual array<int> table_which_cells () = 0;
   virtual path   table_search_cell (int row, int col) = 0;
   virtual void   table_go_to (int row, int col) = 0;
   virtual void   table_set_format (string var, tree val) = 0;
+  virtual tree   table_get_format () = 0;
   virtual string table_get_format (string var) = 0;
   virtual void   table_del_format (string var= "") = 0;
   virtual void   table_format_center () = 0;
