@@ -369,6 +369,27 @@ qt_font (string family, int size, int dpi) {
 * Miscellaneous
 ******************************************************************************/
 
+static hashmap<string,font> larger_font_table;
+
+static font
+make_rubber_font (font fn) {
+  if (starts (fn->res_name, "stix-"))
+    return rubber_stix_font (fn);
+  else if (fn->type == FONT_TYPE_UNICODE)
+    return rubber_unicode_font (fn);
+  else
+    return fn;
+}
+
+font
+rubber_font (font base) {
+  if (larger_font_table->contains (base->res_name))
+    return larger_font_table (base->res_name);
+  font larger= make_rubber_font (base);
+  larger_font_table (base->res_name)= larger;
+  return larger;
+}
+
 int
 script (int sz, int level) {
   int i;
