@@ -118,12 +118,14 @@ tree::operator () (int begin, int end) {
 
 bool
 operator == (tree t, tree u) {
+  if (strong_equal (t, u)) return true;
   return (L(t)==L(u)) &&
     (L(t)==STRING? (t->label==u->label): (A(t)==A(u)));
 }
 
 bool
 operator != (tree t, tree u) {
+  if (strong_equal (t, u)) return false;
   return (L(t)!=L(u)) ||
     (L(t)==STRING? (t->label!=u->label): (A(t)!=A(u)));
 }
@@ -307,7 +309,7 @@ is_multi_paragraph (tree t) {
   case ACTIVE:
   case VAR_ACTIVE:
     return is_multi_paragraph (t[N(t)-1]);
-  case INCLUDE:
+  case VAR_INCLUDE:
     return true;
   case LOCUS:
   case CANVAS:
@@ -390,7 +392,7 @@ is_empty (tree t) {
       if (!is_empty (t[i])) return false;
     return is_concat (t) || (n<=1);
   }
-  return false;
+  return is_compound (t, "suppressed");
 }
 
 /******************************************************************************
