@@ -123,7 +123,11 @@ public:
 
 // generic_rep in generic_tree.hpp
 
-template<> struct type_helper<tree> { static int id; static tree init; };
+template<> struct type_helper<tree> {
+  static int  id;
+  static tree init;
+  static inline tree init_val () { return tree (); }
+};
 
 typedef tree scheme_tree;
 
