@@ -55,15 +55,17 @@ enum tree_label {
   MARK, EXPAND_AS,
   EVAL, QUOTE, QUASI, QUASIQUOTE, UNQUOTE, VAR_UNQUOTE, COPY,
   IF, VAR_IF, CASE, WHILE, FOR_EACH,
-  EXTERN, INCLUDE, USE_PACKAGE, USE_MODULE,
+  EXTERN, VAR_INCLUDE, INCLUDE, USE_PACKAGE, USE_MODULE,
 
   // computational markup
   OR, XOR, AND, NOT,
   PLUS, MINUS, TIMES, OVER, DIV, MOD, MINIMUM, MAXIMUM,
   MATH_SQRT, EXP, LOG, POW, COS, SIN, TAN,
-  MERGE, LENGTH, RANGE, NUMBER, _DATE, TRANSLATE, CHANGE_CASE, FIND_FILE,
+  MERGE, LENGTH, RANGE, NUMBER, _DATE, TRANSLATE, CHANGE_CASE,
+  FIND_FILE, FIND_FILE_UPWARDS,
   IS_TUPLE, LOOK_UP,
   EQUAL, UNEQUAL, LESS, LESSEQ, GREATER, GREATEREQ,
+  BLEND,
 
   // built-in length units
   CM_LENGTH, MM_LENGTH, IN_LENGTH, PT_LENGTH,
@@ -86,8 +88,10 @@ enum tree_label {
 
   // linking primitives
   LOCUS, ID, HARD_ID,
-  LINK, URL, SCRIPT, FIND_ACCESSIBLE, HLINK, ACTION,
-  SET_BINDING, GET_BINDING, LABEL, REFERENCE, PAGEREF, WRITE, TOC_NOTIFY,
+  LINK, URL, SCRIPT, OBSERVER,
+  FIND_ACCESSIBLE, HLINK, ACTION,
+  SET_BINDING, GET_BINDING, LABEL, REFERENCE, PAGEREF,
+  GET_ATTACHMENT, WRITE, TOC_NOTIFY,
 
   // various other tags
   TUPLE, ATTR, TMLEN, COLLECTION, ASSOCIATE, BACKUP, PATTERN, GRADIENT,
