@@ -58,7 +58,7 @@ class t3font;
 class pdf_hummus_renderer_rep : public renderer_rep {
   
   static const int default_dpi= 72; // PDF initial coordinate system corresponds to 72 dpi
-  
+  bool		started;  // initialisation is OK
   url       pdf_file_name;
   int       dpi;
   int       nr_pages;
@@ -167,7 +167,7 @@ class pdf_hummus_renderer_rep : public renderer_rep {
                int alpha);
   
   
-  void bezier_arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta);
+  void bezier_arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta, bool filled);
 
   // hooks for the Hummus library
 
@@ -204,6 +204,7 @@ public:
                            string ptype, bool landsc, double paper_w, double paper_h);
   ~pdf_hummus_renderer_rep ();
   bool is_printer ();
+  bool is_started ();
   void next_page ();
   
   void set_transformation (frame fr);
@@ -289,26 +290,24 @@ pdf_hummus_renderer_rep::pdf_hummus_renderer_rep (
   // setup library
 
   EStatusCode status;
-  {
-    c_string _pdf_file_name (concretize (pdf_file_name));
+	status = pdfWriter.StartPDF(as_charp(concretize (pdf_file_name)), ePDFVersion14 ); // PDF 1.4 for alpha
+	//   , LogConfiguration(true, true, "/Users/mgubi/Desktop/pdfwriter-x.log")
+	//   , PDFCreationSettings(false) ); // true = compression on
+	if (status != PDFHummus::eSuccess) {
+		convert_error << "failed to start PDF\n";
+		started=false;
+	} else {
+		started=true;
+		pdfWriter.GetDocumentContext().AddDocumentContextExtender (new DestinationsWriter(this));
 
-    status = pdfWriter.StartPDF((char*)_pdf_file_name, ePDFVersion14 ); // PDF 1.4 for alpha
-                               //   , LogConfiguration(true, true, "/Users/mgubi/Desktop/pdfwriter-x.log")
-                               //   , PDFCreationSettings(false) ); // true = compression on
-    if (status != PDFHummus::eSuccess) {
-      convert_error << "failed to start PDF\n";
-    }
-  }
-  
-  pdfWriter.GetDocumentContext().AddDocumentContextExtender (new DestinationsWriter(this));
+		// start real work
 
-  // start real work
-  
-  begin_page();
+		begin_page();
+	}
 }
 
 pdf_hummus_renderer_rep::~pdf_hummus_renderer_rep () {
-  
+  if (!started) return; // no cleanup to do
   end_page();
   
   flush_images();
@@ -358,6 +357,11 @@ pdf_hummus_renderer_rep::is_printer () {
   return true;
 }
 
+bool
+pdf_hummus_renderer_rep::is_started () {
+  return started;
+}
+
 void
 pdf_hummus_renderer_rep::next_page () {
   end_page();
@@ -367,7 +371,6 @@ pdf_hummus_renderer_rep::next_page () {
 void
 pdf_hummus_renderer_rep::begin_page() {
   //EStatusCode status;
-  
 
   page = new PDFPage();
   page->SetMediaBox(PDFRectangle(0,0,width,height));
@@ -375,25 +378,25 @@ pdf_hummus_renderer_rep::begin_page() {
   if (NULL == contentContext) {
     //status = PDFHummus::eFailure;
     convert_error << "Failed to create content context for page\n";
+  } else {
+    alpha = 255;
+    fg  = -1;
+    bg  = -1;
+    lw  = -1;
+    current_width = -1.0;
+    cfn= "";
+    cfid = NULL;
+    inText = false;
+    clip_level = 0;
+    
+      // outmost save of the graphics state
+    contentContext->q();
+      // set scaling suitable for dpi (pdf default is 72)
+    contentContext->cm((double)default_dpi / dpi, 0, 0, (double)default_dpi / dpi, 0, 0);
+    
+    set_origin (0, paper_h*dpi*pixel/2.54);
+    set_clipping (0, (int) ((-dpi*pixel*paper_h)/2.54), (int) ((dpi*pixel*paper_w)/2.54), 0);
   }
-  
-  alpha = 255;
-  fg  = -1;
-  bg  = -1;
-  lw  = -1;
-  current_width = -1.0;
-  cfn= "";
-  cfid = NULL;
-  inText = false;
-  clip_level = 0;
-
-  // outmost save of the graphics state
-  contentContext->q();
-  // set scaling suitable for dpi (pdf default is 72)
-  contentContext->cm((double)default_dpi / dpi, 0, 0, (double)default_dpi / dpi, 0, 0);
-  
-  set_origin (0, paper_h*dpi*pixel/2.54);
-  set_clipping (0, (int) ((-dpi*pixel*paper_h)/2.54), (int) ((dpi*pixel*paper_w)/2.54), 0);
 }
 
 void
@@ -419,6 +422,10 @@ pdf_hummus_renderer_rep::end_page(){
   
   EStatusCodeAndObjectIDType res = pdfWriter.GetDocumentContext().WritePageAndRelease(page);
   status = res.first;
+  if (status != PDFHummus::eSuccess) {
+    convert_error << "Failed to write page and release\n";
+  }
+
   page_id (page_num) = res.second;
   page_num++;
 }
@@ -1047,20 +1054,18 @@ pdf_hummus_renderer_rep::draw_glyphs()
   }
 }
 
-
-
-static double font_size (string name) {
-    int pos= search_forwards (".", name);
-    int szpos= pos-1;
-    while ((szpos>0) && is_numeric (name[szpos-1])) szpos--;
-    double size= as_double (name (szpos, pos));
-    if (size == 0) size= 10;
-    double dpi= as_double (name (pos+1, N(name)-2));
-    double mag= (size) * (dpi/72.0);
-    return mag;
+static double
+font_size (string name) {
+  int pos= search_backwards (".", name);
+  int szpos= pos-1;
+  while ((szpos>0) && is_numeric (name[szpos-1])) szpos--;
+  double size= as_double (name (szpos, pos));
+  if (size == 0) size= 10;
+  double dpi= as_double (name (pos+1, N(name)-2));
+  double mag= (size) * (dpi/72.0);
+  return mag;
 }
 
-
 void
 pdf_hummus_renderer_rep::draw (int ch, font_glyphs fn, SI x, SI y) {
   //debug_convert << "draw \"" << (char)ch << "\" " << ch << " " << fn->res_name << "\n";
@@ -1076,9 +1081,10 @@ pdf_hummus_renderer_rep::draw (int ch, font_glyphs fn, SI x, SI y) {
         starts (fontname, "eur") ||
         starts (fontname, "eus") ||
         starts (fontname, "msam") ||
-        starts (fontname, "msbm"))
+        starts (fontname, "msbm")) {
       draw (161, fn, x, y);
-    return;
+      return;
+    }
   }
   string char_name (fontname * "-" * as_string ((int) ch));
   pdf_raw_image glyph;
@@ -1111,7 +1117,7 @@ pdf_hummus_renderer_rep::draw (int ch, font_glyphs fn, SI x, SI y) {
   }
   if (cfid != NULL) {
     begin_text ();
-#if 0
+#if 1
     contentContext->Td(to_x(x)-prev_text_x, to_y(y)-prev_text_y);
     prev_text_x = to_x(x);
     prev_text_y = to_y(y);
@@ -1192,79 +1198,68 @@ pdf_hummus_renderer_rep::fill (SI x1, SI y1, SI x2, SI y2) {
 }
 
 void
-pdf_hummus_renderer_rep::bezier_arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta)
+pdf_hummus_renderer_rep::bezier_arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta, bool filled)
 {
   // PDF can describe only cubic bezier paths, so we have to make up the arc with them
+  // Since this is not mathematically exact, we minimize errors by drawing beziers sub-arcs of at most 90°
   
-  // Control points for an arc of radius 1, centered on the x-axis and spanning theta degrees
-  // are given by:
-  // bx0 = cos(theta/2.0); by0 = sin(theta/2.0);
-  // bx3 = bx0;            by3 = -by0;
-  // bx1 = (4.0-bx0)/3.0;  by1 = (1.0-bx0)*(3.0-bx0)/(3.0*by0);
-  // bx2 = bx1;            by2 = -by1;
-  // from: http://www.tinaja.com/glib/bezcirc2.pdf
-  
-  // we compose the arc using the above basic structure and user space transformations
-  
-
   contentContext->q(); // save graphics state
 
-  {
     double xx1 = to_x(x1), yy1 = to_y(y1), xx2 = to_x(x2), yy2 = to_y(y2);
     double cx = (xx1+xx2)/2, cy = (yy1+yy2)/2;
     double rx = (xx2-xx1)/2, ry = (yy2-yy1)/2;
-    contentContext->cm(rx, 0, 0, ry, cx, cy); // scaling and centering
-  }
-  
-  if (alpha != 0) {
-    double a = 2.0*M_PI*alpha/(360.0*64.0);
-    contentContext->cm(cos(a), sin(a), -sin(a), cos(a), 0, 0); // rotation
-  }
-  
-  if (delta == 360*64) {
-    // closed circle
-    contentContext->m(1.0,0.0);
-  } else {
-    // an open arc closed in the center
-    //FIXME: is this really what we want ?
-    contentContext->m(0.0,0.0);
-    contentContext->l(1.0,0.0);
-  }
-  
-  int prev_phi = 0;
-  while (delta > 0) {
-    int phi = min(delta,90*64); // draw arcs in portions of 90 degrees maximum
-    delta -= phi;
-    double sphi = sin(2*M_PI*(phi+prev_phi)/(2.0*360.0*64.0));
-    double cphi = cos(2*M_PI*(phi+prev_phi)/(2.0*360.0*64.0));
-    contentContext->cm(cphi, sphi, -sphi, cphi, 0, 0); // rotation of phi/2 degrees
-    sphi = sin(2*M_PI*(phi)/(2.0*360.0*64.0));
-    cphi = cos(2*M_PI*(phi)/(2.0*360.0*64.0));
-    double bx0 = cphi, by0 = sphi; // , bx3 = bx0, by3 = -by0;
-    double bx1 = (4.0-bx0)/3.0, by1 = (1.0-bx0)*(3.0-bx0)/(3.0*by0);
-    double bx2 = bx1, by2 = -by1;
-    contentContext->c(bx2, by2, bx1, by1, bx0, by0);
-    prev_phi = phi;
-  }
-  
-  contentContext->h(); // close the path
-  contentContext->Q(); // restore the graphics state
+    contentContext->cm(1, 0, 0, 1, cx, cy); // centering
+	//we can't apply scale here because in pdf the pen is scaled too
+
+	int i=1+abs(delta)/(90*64); //number of sub-arcs needed 
+	if ((abs(delta)%(90*64))==0) i-- ; //correction needed if exact multiple of 90°
+	double phi= 2.0*M_PI*(delta)/(i*360.0*64.0); //angular span of each sub-arc
+	double a = 2.0*M_PI*(alpha)/(360.0*64.0); //start angle in radians
+
+  // Control points for an arc of radius 1, centered on the x-axis and spanning phi degrees
+  // from: http://www.tinaja.com/glib/bezcirc2.pdf
+    double sphi = sin(phi/2),	cphi = cos(phi/2);
+    double bx0 = cphi,			by0 = -sphi;
+    double bx1 = (4.0-bx0)/3.0,	by1 = (1.0-bx0)*(3.0-bx0)/(3.0*by0);
+    double bx2 = bx1,			by2 = -by1;
+    double bx3 = bx0,			by3 = -by0;
+	
+	// repeatedly draw rotated and scaled sub-arc
+	// cannot use user-space transformations with cm util path is painted (otherwise path is lost)
+	// so we perform explicit rotation+scaling calculations
+	int k;
+	for (k=0; k<i;k++) {
+	sphi = sin(phi*(k+0.5)+a); 
+    cphi = cos(phi*(k+0.5)+a);
+	if (k==0) contentContext->m(rx*(bx0*cphi-by0*sphi),ry* (+bx0*sphi+by0*cphi)); //start point
+	contentContext->c(
+		rx*(bx1*cphi-by1*sphi), ry*(+bx1*sphi+by1*cphi),
+		rx*(bx2*cphi-by2*sphi), ry*(+bx2*sphi+by2*cphi),
+		rx*(bx3*cphi-by3*sphi), ry*(+bx3*sphi+by3*cphi));
+	}
+	
+	//paint
+    if (filled) {
+		// contentContext->l(0.0, 0.0); // for a filled "pie" with vertex at the center
+		contentContext->f();
+	}	
+	else  ( (abs(delta) == 360*64) ? contentContext->s() : contentContext->S()); 
+	// here we close the path if it's a full circle 
+  contentContext->Q(); // restore the graphics state (undoes centering only)
 }
 
 void
 pdf_hummus_renderer_rep::arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta) {
   // debug_convert << "arc\n";
   end_text ();
-  bezier_arc(x1, y1, x2, y2, alpha, delta);
-  contentContext->S();
+  bezier_arc(x1, y1, x2, y2, alpha, delta, false);
 }
 
 void
 pdf_hummus_renderer_rep::fill_arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta) {
   // debug_convert << "fill_arc\n";
   end_text ();
-  bezier_arc(x1, y1, x2, y2, alpha, delta);
-  contentContext->f();
+  bezier_arc(x1, y1, x2, y2, alpha, delta, true);
 }
 
 void
@@ -1315,82 +1310,76 @@ pdf_image_rep::flush (PDFWriter& pdfw)
     name= "$TEXMACS_PATH/misc/pixmaps/unknown.ps";
   
   // do not use "convert" to convert from eps to pdf since it rasterizes the picture
-  // string filename = concretize (name);
   
-  url temp= url_temp (".pdf");
-  string tempname= sys_concretize (temp);
-  c_string fname (tempname);
+  url temp;
   string s= suffix (name);
   double scale_x  = 1, scale_y  = 1;
   // debug_convert << "flushing :" << fname << LF;
-  
-  if (s == "pdf") {
-    // * PDF
-    // FIXME: better avoid copying in this case
-    string cmd= "cp";
-    system (cmd, name, temp);
-  } else if ( s != "ps" && s != "eps") {
-    // * generic image format
-    
-    // try to work out inclusion using our own tools
-    // note that we have to return since flush_raster
-    // already build the appopriate Form XObject into the PDF
-    
-    
-    if ((s == "jpg") || (s == "jpeg"))
-      if (flush_jpg(pdfw, name)) return;
-    
-    if (flush_raster (pdfw, name)) return;
+ 
+	if (s == "pdf") {
+		temp=name;
+		name=url_none();
+	} else {
+		temp= url_temp (".pdf");
 
-    // if this fails try using convert from ImageMagik
-    // to convert to pdf
-    string cmd= "convert";
-    system (cmd, name, temp);
-  } else {
-    // * ps or eps
-    // use gs to convert eps to pdf and take care of properly handling the bounding box
-    // the resulting pdf image will always start at 0,0.
-    
-    int bx1, by1, bx2, by2; // bounding box
-    ps_bounding_box(u, bx1, by1, bx2, by2);
-    
-    string cmd= gs_prefix();
-    cmd << " -dQUIET -dNOPAUSE -dBATCH -dSAFER -sDEVICE=pdfwrite ";
-    cmd << " -sOutputFile=" << tempname << " ";
-    cmd << " -c \" << /PageSize [ " << as_string(bx2-bx1) << " " << as_string(by2-by1)
-    << " ] >> setpagedevice gsave  "
-    << as_string(-bx1) << " " << as_string(-by1) << " translate \" ";
-    cmd << " -f " << sys_concretize (name);
-    cmd << " -c \" grestore \"  ";
-    // debug_convert << cmd << LF;
-    system(cmd);
-    
-    scale_x = w/((double)(bx2-bx1));
-    scale_y = h/((double)(by2-by1));
-  }
-  EStatusCode status = PDFHummus::eSuccess;
+		if ( s != "ps" && s != "eps") {
+			// * generic image format
+
+			// try to work out inclusion using our own tools
+			// note that we have to return since flush_raster
+			// already build the appopriate Form XObject into the PDF
+
+			if ((s == "jpg") || (s == "jpeg"))
+				if (flush_jpg(pdfw, name)) return;
+
+			if (!(s == "png" && exists_in_path ("convert")))
+			  if (flush_raster (pdfw, name)) return;
+
+			// if this fails try using convert from ImageMagik
+			// to convert to pdf
+			string cmd= "convert";
+			system (cmd, sys_concretize (name), sys_concretize(temp));
+		} else {
+			// * ps or eps
+			// use gs to convert eps to pdf and take care of properly handling the bounding box
+			// the resulting pdf image will always start at 0,0.
+
+			int bx1, by1, bx2, by2; // bounding box
+			ps_bounding_box(u, bx1, by1, bx2, by2);
+
+			string cmd= gs_prefix();
+			cmd << " -dQUIET -dNOPAUSE -dBATCH -dSAFER -sDEVICE=pdfwrite ";
+			cmd << " -sOutputFile=" << sys_concretize(temp) << " ";
+			cmd << " -c \" << /PageSize [ " << as_string(bx2-bx1) << " " << as_string(by2-by1)
+				<< " ] >> setpagedevice gsave  "
+				<< as_string(-bx1) << " " << as_string(-by1) << " translate \" ";
+			cmd << " -f " << sys_concretize (name);
+			cmd << " -c \" grestore \"  ";
+			// debug_convert << cmd << LF;
+			system(cmd);
+
+			scale_x = w/((double)(bx2-bx1));
+			scale_y = h/((double)(by2-by1));
+		}
+	}
+  EStatusCode status = PDFHummus::eFailure;
   DocumentContext& dc = pdfw.GetDocumentContext();
   
   PDFRectangle cropBox (0, 0, w, h);
   double tMat[6] = { scale_x, 0, 0, scale_y, 0, 0};
   
-  PDFDocumentCopyingContext *copyingContext = pdfw.CreatePDFCopyingContext((char*)fname);
-  do {
-    if(!copyingContext) break;
+  PDFDocumentCopyingContext *copyingContext = pdfw.CreatePDFCopyingContext(as_charp(concretize(temp)));
+  if(copyingContext) {
     PDFFormXObject *form = dc.StartFormXObject(cropBox, id, tMat);
     status = copyingContext->MergePDFPageToFormXObject(form,0);
-    if(status != eSuccess) break;
-    pdfw.EndFormXObjectAndRelease(form);
-  } while (false);
-  if (copyingContext) {
+    if(status == eSuccess) pdfw.EndFormXObjectAndRelease(form);
     delete copyingContext;
-    copyingContext = NULL;
   }
-  remove (temp);
+  if(!is_none(name)) remove (temp);
   
   if (status == PDFHummus::eFailure) {
     convert_error << "pdf_hummus, failed to include image file: "
-                  << fname << LF;
+                  << temp << LF;
   }
 }
 
@@ -1898,30 +1887,38 @@ pdf_hummus_renderer_rep::draw_scalable (scalable im, SI x, SI y, int alpha) {
   }
 }
 
+/******************************************************************************
+* Conversions between strings and std::string
+******************************************************************************/
+
+string
+std_string_to_string (std::string str) {
+  string r;
+  for (std::string::iterator it=str.begin(); it!=str.end(); ++it) {
+    r << *it;
+  }
+  return r;
+}
+
+static PDFTextString
+as_hummus_string (string s) {
+  s= cork_to_utf8 (s);
+  c_string u (s);
+  PDFTextString r;
+  std::string stds ((char*) u);
+  return r.FromUTF8 (stds);
+}
+
+static string
+prepare_text (string s) {
+  std::string r= as_hummus_string(s).ToString();
+  return std_string_to_string (r);
+}
 
 /******************************************************************************
  * hyperlinks
  ******************************************************************************/
 
-static string
-prepare_text (string s) {
-  int i;
-  string r;
-  for (i=0; i<N(s); i++) {
-    int c= ((unsigned char) s[i]);
-    if ((s[i]=='(') || (s[i]==')') || (s[i]=='\\'))
-      r << '\\' << s[i];
-    else if ((c <= 32) || (c >= 128)) {
-      r << '\\';
-      r << ('0' + (c >> 6));
-      r << ('0' + ((c >> 3) & 7));
-      r << ('0' + (c & 7));
-    }
-    else r << s[i];
-  }
-  return r;
-}
-
 PDFHummus::EStatusCode
 pdf_hummus_renderer_rep::on_catalog_write (CatalogInformation* inCatalogInformation,
                                          DictionaryContext* inCatalogDictionaryContext,
@@ -1976,7 +1973,7 @@ pdf_hummus_renderer_rep::href (string label, SI x1, SI y1, SI x2, SI y2)
     dict << "\t/Dest /label" << as_string(get_label_id(prepare_text (label))) << "\r\n";
   }
   else {
-    dict << "/Action << /Subtype /URI /URI (" << prepare_text (label) << ") >>\r\n";
+    dict << "/A << /S /URI /URI (" << prepare_text (label) << ") >>\r\n";
   }
   dict << ">>\r\n";
   annot_list (annotId) = dict;
@@ -2026,7 +2023,7 @@ pdf_hummus_renderer_rep::toc_entry (string kind, string title, SI x, SI y) {
   if (kind == "toc-4") ls= 6;
   if (kind == "toc-5") ls= 7;
 
-  outlines << outline_data(title, page_num, to_x(x), to_y(y), ls);
+  outlines << outline_data(title, page_num, to_x(x), to_y(y+20*pixel), ls);
 }
 
 void pdf_hummus_renderer_rep::recurse (ObjectsContext& objectsContext, list<outline_data>& it, ObjectIDType parentId,
@@ -2120,14 +2117,6 @@ pdf_hummus_renderer_rep::set_metadata (string kind, string val) {
   metadata (kind)= val;
 }
 
-static PDFTextString
-as_hummus_string (string s) {
-  c_string u (s);
-  PDFTextString r;
-  std::string stds ((char*) u);
-  return r.FromUTF8 (stds);
-}
-
 void
 pdf_hummus_renderer_rep::flush_metadata () {
   if (N(metadata) == 0) return;
@@ -2209,3 +2198,4 @@ pdf_hummus_renderer (url pdf_file_name, int dpi, int nr_pages,
 }
 
 
+
