@@ -9,14 +9,19 @@
 * in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
-#include "QTMMenuHelper.hpp"
-#include "QTMGuiHelper.hpp"
-#include "QTMStyle.hpp"
+#include "analyze.hpp"
+
 #include "qt_gui.hpp"
 #include "qt_utilities.hpp"
 #include "qt_window_widget.hpp"
-#include "qt_ui_element.hpp"  // qt_choice_command_rep
-#include "analyze.hpp"
+#include "qt_ui_element.hpp"    // qt_choice_command_rep
+#include "qt_picture.hpp"       // xpm_image
+#include "qt_tm_widget.hpp"     // tweak_iconbar_size
+#include "QTMMenuHelper.hpp"
+#include "QTMGuiHelper.hpp"
+#include "QTMStyle.hpp"
+#include "QTMApplication.hpp"
+#include "QTMTreeModel.hpp"
 
 #include <QToolTip>
 #include <QCompleter>
@@ -29,12 +34,20 @@
 /*! Queues the object's command into the main queue. */
 void 
 QTMCommand::apply()  {
-  if (DEBUG_QT)
-    cout << "QTMCommand::apply() (delayed): " << cmd << "\n";
-  if (!is_nil (cmd)) { the_gui->process_command (cmd); }
+BEGIN_SLOT
+  if (!is_nil (cmd)) {
+    the_gui->process_command (cmd);
+    if (DEBUG_QT) {
+      debug_qt << "QTMCommand::apply() (delayed)\n";
+      /* FIXME: this sometimes crashes:
+         cmd->print(debug_qt);
+         debug_qt << "\n";
+      */
+    }
+  }
+END_SLOT
 }
 
-
 /******************************************************************************
  * QTMAction
  ******************************************************************************/
@@ -44,11 +57,11 @@ QTMAction::QTMAction (QObject *parent) : QAction (parent) {
                     this,                  SLOT (doRefresh()));
   _timer = new QTimer (this);
   QObject::connect (_timer, SIGNAL (timeout()),
-                   this,     SLOT (doShowToolTip()));
+                    this,     SLOT (doShowToolTip()));
 }
 
 QTMAction::~QTMAction() { 
-  if (menu() && !(menu()->parent())) delete menu(); 
+  if (menu() && !menu()->parent()) delete menu();
 }
 
 void
@@ -64,14 +77,17 @@ QTMAction::set_text (string s) {
 
 void 
 QTMAction::doRefresh() {
+BEGIN_SLOT
   set_text (str);
+END_SLOT
 }
 
 void
-QTMAction::showToolTip()
-{
+QTMAction::showToolTip() {
+BEGIN_SLOT
   _timer->start (500);   // Restarts the timer if already running
   _pos = QCursor::pos();
+END_SLOT
 }
 
 /*
@@ -86,17 +102,19 @@ QTMAction::showToolTip()
  stops, the distance is bigger than the given constant and no tooltip is
  displayed.
  */
+
 void
 QTMAction::doShowToolTip() {
+BEGIN_SLOT
   static int step = QApplication::font().pointSize();
   _timer->stop();
   if ((QCursor::pos() - _pos).manhattanLength() < step)  // Hideous HACK
     QToolTip::showText (QCursor::pos(), toolTip());
   else
     QToolTip::hideText();
+END_SLOT
 }
 
-
 /******************************************************************************
  * QTMWidgetAction
  ******************************************************************************/
@@ -120,9 +138,8 @@ QTMWidgetAction::createWidget (QWidget * parent) {
  * QTMTileAction
  ******************************************************************************/
 
-QTMTileAction::QTMTileAction (QWidget* parent, array<widget>& arr, int _cols)
-: QWidgetAction (parent), cols (_cols)
-{
+QTMTileAction::QTMTileAction (array<widget>& arr, int _cols, QObject* parent)
+: QWidgetAction (parent), cols (_cols) {
   actions.reserve (N (arr));
   for (int i = 0; i < N (arr); i++) {
     if (is_nil (arr[i])) break;
@@ -143,7 +160,7 @@ QWidget*
 QTMTileAction::createWidget (QWidget* parent)
 {
   if (DEBUG_QT_WIDGETS)
-    cout << "QTMTileAction::createWidget\n";
+    debug_widgets << "QTMTileAction::createWidget\n";
   QWidget* wid= new QTMMenuWidget (parent);
   QGridLayout* l= new QGridLayout (wid);
     // wid->setAutoFillBackground (true);
@@ -153,9 +170,9 @@ QTMTileAction::createWidget (QWidget* parent)
   l->setHorizontalSpacing (2);
   l->setVerticalSpacing (2);
   l->setContentsMargins (4, 0, 4, 0);
-  int row= 0, col= 0;
-  for (int i=0; i < actions.count(); i++) {
-    QAction* sa= actions[i];
+  int row = 0, col = 0;
+  for (int    i = 0; i < actions.count(); i++) {
+    QAction* sa = actions[i];
     QToolButton* tb= new QTMMenuButton (wid);
     tb->setDefaultAction (sa);
     QObject::connect (tb, SIGNAL (released()), this, SLOT (trigger()));
@@ -172,13 +189,13 @@ QTMTileAction::createWidget (QWidget* parent)
  * QTMMinibarAction
  ******************************************************************************/
 
-QTMMinibarAction::QTMMinibarAction (QWidget* parent, array<widget>& arr)
+QTMMinibarAction::QTMMinibarAction (array<widget>& arr, QObject* parent)
 : QWidgetAction (parent)
 {
   actions.reserve (N (arr));
   for (int i = 0; i < N (arr); i++) {
     if (is_nil (arr[i])) break;
-    QAction *act = concrete (arr[i])->as_qaction();
+    QAction* act = concrete (arr[i])->as_qaction();
     act->setParent (this);
     actions.append (act);
   };
@@ -192,7 +209,11 @@ QTMMinibarAction::QTMMinibarAction (QWidget* parent, array<widget>& arr)
  */
 QWidget*
 QTMMinibarAction::createWidget (QWidget* parent) {
-  if (DEBUG_QT_WIDGETS) cout << "QTMMinibarAction::createWidget\n";
+  static QImage* pxm = xpm_image ("tm_add.xpm"); // See qt_tm_widget.cpp 
+  QSize sz = pxm ? pxm->size() : QSize (16, 16);
+  qt_tm_widget_rep::tweak_iconbar_size (sz);
+  
+  if (DEBUG_QT_WIDGETS) debug_widgets << "QTMMinibarAction::createWidget\n";
   QWidget* wid= new QWidget (parent);
   QBoxLayout* l= new QBoxLayout (QBoxLayout::LeftToRight, wid);
   wid->setLayout (l);
@@ -217,10 +238,10 @@ QTMMinibarAction::createWidget (QWidget* parent) {
       tb->setAutoRaise (true);
       tb->setPopupMode (QToolButton::InstantPopup);
       tb->setStyle (qtmstyle());
-        //  tb->setIconSize (QSize (12,12));
+      tb->setIconSize (sz);
       QFont f = tb->font();
       int fs = as_int (get_preference ("gui:mini-fontsize", QTM_MINI_FONTSIZE));
-      f.setPixelSize (fs > 0 ? fs : QTM_MINI_FONTSIZE);
+      f.setPointSize (fs > 0 ? fs : QTM_MINI_FONTSIZE);
       tb->setFont(f);
       l->addWidget (tb);
     }
@@ -228,66 +249,164 @@ QTMMinibarAction::createWidget (QWidget* parent) {
   return wid;
 }
 
+/******************************************************************************
+ * QTMMenuButton
+ ******************************************************************************/
+
+QTMMenuButton::QTMMenuButton (QWidget* parent) : QToolButton (parent) {
+  QTMAuxMenu m;
+  m.myInitStyleOption (&option);
+  setAttribute (Qt::WA_Hover);
+}
+
+void
+QTMMenuButton::mousePressEvent (QMouseEvent* e) {
+    // this one triggers the action and toggles the button
+  QToolButton::mousePressEvent (e);
+    // this one forwards the event to the parent
+    // (which eventually is the menu)
+  QWidget::mousePressEvent (e);
+}
+
+void
+QTMMenuButton::mouseReleaseEvent (QMouseEvent* e) {
+    // this one triggers the action and untoggles the button
+  QToolButton::mouseReleaseEvent (e);
+    // this one forwards the event to the parent
+    // (which eventually is the menu which then closes itself)
+  QWidget::mouseReleaseEvent (e);
+}
+
+void
+QTMMenuButton::paintEvent (QPaintEvent* e) {
+  (void) e;
+  QPainter p (this);
+  
+    // initialize the options
+  QStyleOptionToolButton opt;
+  initStyleOption (&opt);
+  QRect r = rect();
+  option.rect = r;
+  option.state = QStyle::State_Enabled | (opt.state & QStyle::State_MouseOver
+                                          ? QStyle::State_Selected
+                                          : QStyle::State_None);
+    // draw the control background as a menu item
+  style()->drawControl (QStyle::CE_MenuItem, &option, &p, this);
+    // draw the icon with a bit of inset.
+  r.adjust (2, 2, -2, -2);
+  defaultAction()->icon().paint (&p, r);
+}
+
+/******************************************************************************
+ * QTMMenuWidget
+ ******************************************************************************/
+
+QTMMenuWidget::QTMMenuWidget (QWidget* parent) : QWidget (parent) {
+  QTMAuxMenu m;
+  m.myInitStyleOption (&option);
+}
+
+void
+QTMMenuWidget::paintEvent(QPaintEvent* e) {
+  QPainter p (this);
+  option.rect = rect ();
+  style()->drawControl (QStyle::CE_MenuEmptyArea, &option, &p, this);
+  QWidget::paintEvent (e);
+}
 
 /******************************************************************************
  * QTMLazyMenu
  ******************************************************************************/
 
+QTMLazyMenu::QTMLazyMenu (promise<widget> _pm, QWidget* p, bool right)
+: QMenu (p), promise_widget (_pm), show_right (right) {
+  QObject::connect (this, SIGNAL (aboutToShow ()), this, SLOT (force ()));
+}
+
 void
-rerootActions (QWidget* dest, QWidget* src) {
-  if (src == NULL || dest == NULL) return;
-  QList<QAction *> list = dest->actions();
-  while (!list.isEmpty()) {
-    QAction* a= list.takeFirst();
-    dest->removeAction (a);
-    //    delete a;
-    a->deleteLater();
+QTMLazyMenu::showEvent (QShowEvent* e)
+{
+  if (show_right && parentWidget()) {
+    QPoint p = pos();
+    p.rx() += parentWidget()->width();
+    p.ry() -= parentWidget()->height();
+    move (p);
   }
-  list = src->actions();
+  QMenu::showEvent (e);
+}
+
+/*! Sets the QTMLazyMenu as the menu for the QAction and makes its destruction
+ depend on that of the latter. */
+void
+QTMLazyMenu::attachTo (QAction* a) {
+  QObject::connect (a,  SIGNAL (destroyed (QObject*)),
+                    this, SLOT (destroy (QObject*)));
+  a->setMenu (this);
+}
+
+void
+QTMLazyMenu::transferActions (QWidget* from) {
+  if (from == NULL) return;
+  QList<QAction*> list = actions();
   while (!list.isEmpty()) {
-    QAction* a= list.takeFirst();
-    dest->addAction (a);
-    a->setParent (dest);
+    QAction* a = list.takeFirst();
+    removeAction (a);
+    // For some reason, using deleteLater() leaks all the QActions
+    //a->deleteLater();
+    delete a;
+  }
+  list = from->actions();
+  while (!list.isEmpty()) {
+    QAction* a = list.takeFirst();
+    addAction (a);
+    a->setParent (this);
   }
 }
 
 void
 QTMLazyMenu::force () {
-  if (DEBUG_QT_WIDGETS) cout << "Force lazy menu" << LF;
-  widget w = pm ();
-  QMenu* menu2 = concrete(w)->get_qmenu();
-  rerootActions (this, menu2);
-    //menu2->deleteLater();  // OK? Conflicting policies in qt_menu and qt_ui_element
+BEGIN_SLOT
+  QMenu* m = concrete (promise_widget())->get_qmenu();
+  transferActions (m);
+END_SLOT
 }
 
+void
+QTMLazyMenu::destroy (QObject* obj) {
+BEGIN_SLOT
+  (void) obj;
+  deleteLater();
+END_SLOT
+}
 
 /******************************************************************************
  * QTMInputTextWidgetHelper
  ******************************************************************************/
 
-QTMInputTextWidgetHelper::QTMInputTextWidgetHelper (qt_input_text_widget_rep* _wid) 
-: QObject (NULL), p_wid (abstract (_wid)), done (false) { }
-
-/*! Destructor.
- Removes reference to the helper in the texmacs widget. If needed the texmacs
- widget is automatically deleted.
- */
-QTMInputTextWidgetHelper::~QTMInputTextWidgetHelper() {
-  wid()->helper = NULL;
+QTMInputTextWidgetHelper::QTMInputTextWidgetHelper (qt_widget _wid,
+                                                    QTMLineEdit* le)
+: QObject (le), p_wid (_wid), done (false) {
+  ASSERT (le != NULL, "QTMInputTextWidgetHelper: expecting valid QTMLineEdit");
+  QObject::connect (le, SIGNAL (returnPressed ()), this, SLOT (commit ()));
+  QObject::connect (le, SIGNAL (focusOut (Qt::FocusReason)),
+                    this, SLOT (leave (Qt::FocusReason)));
 }
 
 void
 QTMInputTextWidgetHelper::apply () {
+BEGIN_SLOT
   if (done) return;
   done = true;
   the_gui->process_command (wid()->cmd, wid()->ok
                             ? list_object (object (wid()->input))
                             : list_object (object (false)));
+END_SLOT
 }
 
 /*! Executed when the enter key is pressed. */
 void
 QTMInputTextWidgetHelper::commit () {
+BEGIN_SLOT
   QTMLineEdit* le = qobject_cast <QTMLineEdit*> (sender());
   if (!le) return;
 
@@ -301,15 +420,20 @@ QTMInputTextWidgetHelper::commit () {
     concrete (get_canvas (win))->qwid->setFocus();
 
   if (win) apply();    // This is 0 inside a dialog => no command
+END_SLOT
 }
 
 /*! Executed after commit of the input field (enter) and when losing focus */
 void
-QTMInputTextWidgetHelper::leave () {
+QTMInputTextWidgetHelper::leave (Qt::FocusReason reason) {
+BEGIN_SLOT
   QTMLineEdit* le = qobject_cast <QTMLineEdit*> (sender());
   if (!le) return;
 
-  if (get_preference ("gui:line-input:autocommit") == "#t") {
+    // Don't autocommit if the user pressed the escape key
+    // (see QTMLineEdit::focusOut()).
+  if (reason != Qt::OtherFocusReason &&
+      get_preference ("gui:line-input:autocommit") == "#t") {
     done         = false;
     wid()->ok    = true;
     wid()->input = from_qstring (le->text());
@@ -319,71 +443,45 @@ QTMInputTextWidgetHelper::leave () {
 
   widget_rep* win = qt_window_widget_rep::widget_from_qwidget (le);
   if (win) apply();    // This is 0 inside a dialog => no command
+END_SLOT
 }
 
-void
-QTMInputTextWidgetHelper::remove (QObject* obj) {
-  views.removeAll (qobject_cast<QTMLineEdit*> (obj));
-  if (views.count () == 0)
-    deleteLater();
-}
-
-void
-QTMInputTextWidgetHelper::add (QObject* obj) {
-  QTMLineEdit* le = qobject_cast<QTMLineEdit*> (obj);
-  if (le && !views.contains (le)) {
-    QObject::connect (le, SIGNAL (destroyed (QObject*)), this, SLOT (remove (QObject*)));
-    QObject::connect (le, SIGNAL (returnPressed ()),     this, SLOT (commit ()));
-    QObject::connect (le, SIGNAL (focusOut ()),          this, SLOT (leave ()));
-    views << le;
-  }
-}
-
-
 /******************************************************************************
  * QTMFieldWidgetHelper
  ******************************************************************************/
 
-QTMFieldWidgetHelper::QTMFieldWidgetHelper (qt_field_widget _wid) 
-  : QObject (NULL), wid (_wid), done (false) { }
-
-/*! Destructor.
- Removes reference to the helper in the texmacs widget. Deletion of the texmacs
- widget is automagic.
- */
-QTMFieldWidgetHelper::~QTMFieldWidgetHelper() {
-  wid->helper = NULL;
+QTMFieldWidgetHelper::QTMFieldWidgetHelper (qt_widget _wid, QComboBox* cb)
+: QObject (cb), wid (_wid), done (false) {
+  ASSERT (cb != NULL, "QTMFieldWidgetHelper: expecting valid QComboBox");
+  QObject::connect (cb, SIGNAL (editTextChanged (const QString&)),
+                    this, SLOT (commit (const QString&)));
+}
+QTMFieldWidgetHelper::QTMFieldWidgetHelper (qt_widget _wid, QLineEdit* cb)
+: QObject (cb), wid (_wid), done (false) {
+  ASSERT (cb != NULL, "QTMFieldWidgetHelper: expecting valid QLineEdit");
+  QObject::connect (cb, SIGNAL (textChanged (const QString&)),
+                    this, SLOT (commit (const QString&)));
 }
 
 void
 QTMFieldWidgetHelper::commit (const QString& qst) {
-  wid->input = scm_quote (from_qstring (qst));
+BEGIN_SLOT
+  static_cast<qt_field_widget_rep*> (wid.rep)->input =
+      scm_quote (from_qstring (qst));
+END_SLOT
 }
 
-void
-QTMFieldWidgetHelper::remove (QObject* obj) {
-  views.removeAll (qobject_cast<QTMComboBox*> (obj));
-  if (views.count () == 0)
-    deleteLater();
-}
-
-void
-QTMFieldWidgetHelper::add (QObject* obj) {
-  QComboBox* cb = qobject_cast<QComboBox*> (obj);
-  if (obj && !views.contains (cb)) {
-    QObject::connect (cb, SIGNAL (destroyed (QObject*)), this, SLOT (remove (QObject*)));
-    QObject::connect (cb, SIGNAL (editTextChanged (const QString&)), this, SLOT (commit (const QString&)));
-    views << cb;
-  }
-}
-
-
 /******************************************************************************
  * QTMLineEdit
  ******************************************************************************/
 
-QTMLineEdit::QTMLineEdit (QWidget* parent, string _ww, int style)
-: QLineEdit (parent), completing (false), ww (_ww) {
+QTMLineEdit::QTMLineEdit (QWidget* parent, string _type, string _ww,
+                          int style, command _cmd)
+  : QLineEdit (parent), completing (false),
+    type ("default"), name ("default"), serial ("default"),
+    ww (_ww), cmd (_cmd), last_key (0) {
+  set_type (_type);
+  if (type == "password") setEchoMode(QLineEdit::Password);
   if (style & WIDGET_STYLE_MINI) {
     setStyle (qtmstyle());
       // FIXME: we should remove this and let the scheme code decide.
@@ -394,11 +492,34 @@ QTMLineEdit::QTMLineEdit (QWidget* parent, string _ww, int style)
   }
   
     // just to be sure we don't capture the wrong keys in keyPressEvent
-  setCompleter(0);
+  setCompleter (0);
 
   qt_apply_tm_style (this, style);
 }
 
+void
+QTMLineEdit::set_type (string t) {
+  int i= search_forwards (":", 0, t);
+  if (i >= 0) {
+    type= t (i+1, N(t));
+    name= t (0, i);
+    int j= search_forwards ("#", 0, name);
+    if (j >= 0) {
+      serial= name (j+1, N(name));
+      name  = name (0, j);
+    }
+  }
+  else type= t;
+}
+
+bool
+QTMLineEdit::continuous () {
+  return
+    starts (type, "search") ||
+    starts (type, "replace-") ||
+    starts (serial, "form-");
+}
+
 /*
  We need to reimplement the main event handler because we need the tab key for
  completions. (See Qt docs for QWidget::event())
@@ -409,10 +530,11 @@ QTMLineEdit::event (QEvent* ev) {
     keyPressEvent (static_cast<QKeyEvent*> (ev));
   else
     return QWidget::event (ev);
-
   return true;
 }
 
+extern hashmap<int,string> qtkeymap;
+
 /*
  FIXME: This is a hideous mess...
  */
@@ -420,78 +542,159 @@ void
 QTMLineEdit::keyPressEvent (QKeyEvent* ev)
 {
   QCompleter* c = completer();
-
-  int key = (ev->key() == Qt::Key_Tab && ev->modifiers() & Qt::ShiftModifier)
+  
+  last_key = (ev->key() == Qt::Key_Tab && ev->modifiers() & Qt::ShiftModifier)
             ? Qt::Key_Backtab
             : ev->key();
-  
-  if (c) {
+
+  if (continuous ()) {
+    if ((last_key != Qt::Key_Tab || type == "replace-what") &&
+        (last_key != Qt::Key_Backtab || type == "replace-by") &&
+        last_key != Qt::Key_Down &&
+        last_key != Qt::Key_Up &&        
+        last_key != Qt::Key_Enter &&
+        last_key != Qt::Key_Return &&
+        last_key != Qt::Key_Escape &&
+        (ev->modifiers() & Qt::ControlModifier) == 0 &&
+        (ev->modifiers() & Qt::MetaModifier) == 0)
+      QLineEdit::keyPressEvent (ev);
+    string key= "none";
+    string s  = from_qstring (text());
+    if (last_key >= 32 && last_key <= 126) {
+      key= string ((char) last_key);
+      if (key[0] >= 'A' && key[0] <= 'Z')
+        if ((ev->modifiers() & Qt::ShiftModifier) == 0)
+          key[0]= (int) (key[0] + ((int) 'a') - ((int) 'A'));
+    }
+    if (qtkeymap->contains (last_key)) key= qtkeymap[last_key];
+    if ((ev->modifiers() & Qt::ShiftModifier) && N(key) > 1) key= "S-" * key;
+#ifdef Q_WS_MAC
+    if (ev->modifiers() & Qt::ControlModifier) key= "C-" * key;
+    if (ev->modifiers() & Qt::AltModifier) key= "none";
+    if (ev->modifiers() & Qt::MetaModifier) key= "M-" * key;
+#else
+    if (ev->modifiers() & Qt::ControlModifier) key= "M-" * key;
+    if (ev->modifiers() & Qt::AltModifier) key= "A-" * key;
+    if (ev->modifiers() & Qt::MetaModifier) key= "C-" * key;
+#endif
+    cmd (list_object (list_object (object (s), object (key))));
+    return;
+  }
+  else if (c) {
     int row = 0;
-    switch (key) {
+    switch (last_key) {
       case Qt::Key_Down:
         completing = true;
+        setCursorPosition (0);
         c->complete();
       case Qt::Key_Tab:
       {
+//        cout << "Completing= " << completing << LF;
+//        cout << "hasSelectedText= " << hasSelectedText() << LF;
+//        cout << "CursorPosition= " << cursorPosition() << LF;
+//        cout << "SelectionStart= " << selectionStart() << LF;
+        
         if (completing) {
+//          cout << "CompletionCount= " << c->completionCount() << LF;
           if (c->completionCount() > 1) {
             if (! c->setCurrentRow (c->currentRow() + 1))
               c->setCurrentRow (0);    // cycle
           } else {
-            setCursorPosition (text().length());
             completing = false;
+            setCursorPosition (text().length());
+            c->setCompletionPrefix ("");
               //c->popup()->hide();
           }
-        } else {
           if (hasSelectedText())
-            c->setCompletionPrefix (text().left (selectionStart()));
+            setCursorPosition (selectionStart());
+          if (c->currentCompletion() != "") {
+            int pos = cursorPosition();
+            setText (c->currentCompletion ());
+            setSelection (pos, text().length());
+          } else {
+            completing = false;
+            setSelection (0, text(). length());
+            c->setCompletionPrefix ("");
+          }
+        } else {
+          QString prefix;
+          if (hasSelectedText())
+            prefix = text().left (selectionStart());
           else
-            c->setCompletionPrefix (text().left (cursorPosition()));
-
-            // If there are no completions, send the key up for tab navigation
+            prefix = text().left (cursorPosition());
+          c->setCompletionPrefix (prefix);
+//          cout << "prefix= " << from_qstring (prefix) << LF;
+//          cout << "CompletionCount= " << c->completionCount() << LF;
+          
+            // If there are no completions, go to the end of the line or
+            // send the key up for tab navigation
           if (c->completionCount() == 0 ||
               (c->completionCount() == 1 && c->currentCompletion() == text())) {
-            QLineEdit::keyPressEvent (ev);
+            if (c->popup() && c->popup()->isVisible()) {
+              setCursorPosition (text().length());
+              c->popup()->hide();
+            } else if (cursorPosition() == text().length()) {
+              QLineEdit::keyPressEvent (ev);
+            } else {
+              setCursorPosition (text().length());
+            }
             return;
           }
-          
+
           completing = true;
+            // hack: advance one completion (needed after tab navigation)
+          if (c->currentCompletion() == text()) {
+            clear();
+            if (! c->setCurrentRow (c->currentRow() + 1))
+              c->setCurrentRow (0);    // cycle
+          }
           c->complete();
         }
-        if (hasSelectedText())
-          setCursorPosition (selectionStart());
-        if (c->currentCompletion() != "") {
-          int pos = cursorPosition();
-          setText (c->currentCompletion ());
-          setSelection (pos, text().length() - 1);
-        } else {
-          completing = false;
-          setSelection (0, text(). length() - 1);
-        }
-
         ev->accept();
       }
         return;
           // This is different on purpose: when "back-completing" suggested text
           // we want to display the previous entry to the one suggested
       case Qt::Key_Up:
+        completing = true;
+        setCursorPosition (0);
+        c->complete();
       case Qt::Key_Backtab:
       {
-        completing = true;
-        c->complete();
-        row = c->currentRow();
-        if (! c->setCurrentRow (row - 1))
-          c->setCurrentRow (c->completionCount() - 1);    // cycle
-        if (c->currentCompletion() != "") {
-          int pos;
+        if (!completing) {
           if (hasSelectedText())
-            pos = selectionStart();
+            c->setCompletionPrefix (text().left (selectionStart()));
           else
-            pos = cursorPosition();
-          setText (c->currentCompletion ());
-          setSelection (pos, text().length() - 1);
+            c->setCompletionPrefix (text().left (cursorPosition()));
+            // If there are no completions, go to the end of the line or
+            // send the key up for tab navigation
+          if (c->completionCount() == 0 ||
+              (c->completionCount() == 1 && c->currentCompletion() == text())) {
+            if (c->popup() && c->popup()->isVisible()) {
+              setCursorPosition (text().length());
+              c->popup()->hide();
+            } else if (cursorPosition() == text().length()) {
+              QLineEdit::keyPressEvent (ev);
+            } else {
+              setCursorPosition (text().length());
+            }
+            return;
+          }
+          completing = true;
+          c->complete();
         } else {
-            // TODO: blink somehow
+          row = c->currentRow();
+          if (! c->setCurrentRow (row - 1))
+            c->setCurrentRow (c->completionCount() - 1);    // cycle
+          if (c->currentCompletion() != "") {
+            int pos;
+            if (hasSelectedText())
+              pos = selectionStart();
+            else
+              pos = cursorPosition();
+            setText (c->currentCompletion ());
+            setSelection (pos, text().length());
+          }
         }
         ev->accept();
       }
@@ -502,27 +705,33 @@ QTMLineEdit::keyPressEvent (QKeyEvent* ev)
           setCursorPosition (text().length());
           c->popup()->hide();
         } else if (completing) {
+          completing = false;
           setText (c->currentCompletion());
           setCursorPosition (text().length());
+          c->setCompletionPrefix ("");
         } else {
-          emit returnPressed();
+          completing = false;
+          c->setCompletionPrefix ("");
+          QLineEdit::keyPressEvent (ev);
+          return;
         }
-        completing = false;
         ev->accept();
         return;
       case Qt::Key_Escape:
-        if (completing) {
+        if (completing && c->completionMode() == QCompleter::PopupCompletion) {
           if (c->popup()) c->popup()->hide();
           completing = false;
         } else {
           emit editingFinished();
           ev->accept();
           if (parentWidget())        // HACK to return focus to the main editor widget
-            parentWidget()->setFocus();
+            parentWidget()->setFocus ();
         }
+        c->setCompletionPrefix ("");
         return;
       default:
         completing = false;
+        c->setCompletionPrefix ("");
         QLineEdit::keyPressEvent (ev);
         return;
     }
@@ -547,12 +756,15 @@ QTMLineEdit::focusInEvent (QFocusEvent* ev)
 void
 QTMLineEdit::focusOutEvent (QFocusEvent* ev)
 {
-  emit focusOut();
+  if (!continuous ()) {
+    Qt::FocusReason reason =
+      (last_key != Qt::Key_Escape) ? ev->reason() : Qt::OtherFocusReason;
+    emit focusOut (reason);
+  }
   QLineEdit::focusOutEvent (ev);
 }
 
 
-
 /******************************************************************************
  * QTMTabWidget
  ******************************************************************************/
@@ -570,6 +782,7 @@ QTMTabWidget::QTMTabWidget (QWidget *p) : QTabWidget(p) {
  */
 void
 QTMTabWidget::resizeOthers (int current) {
+BEGIN_SLOT
   for (int i = 0; i < count(); ++i) {
     if (i != current)
       widget(i)->setSizePolicy (QSizePolicy::Ignored, QSizePolicy::Ignored);
@@ -588,15 +801,15 @@ QTMTabWidget::resizeOthers (int current) {
   if (window()->minimumSize()!=QSize (0,0) && 
       window()->maximumSize() != QSize (QWIDGETSIZE_MAX, QWIDGETSIZE_MAX))
     window()->setFixedSize (window()->sizeHint());
+END_SLOT
 }
 
-
-
 /******************************************************************************
  * QTMRefreshWidget
  ******************************************************************************/
 
 widget make_menu_widget (object wid);
+extern bool menu_caching;
 
 QTMRefreshWidget::QTMRefreshWidget (qt_widget _tmwid, string _strwid, string _kind)
 : QWidget (), strwid (_strwid), kind (_kind),
@@ -629,7 +842,7 @@ QTMRefreshWidget::recompute (string what) {
     object uwid = eval (s);
     cur = make_menu_widget (uwid);
     tmwid->add_child (cur); // FIXME?! Is this ok? what when we refresh?
-    cache (xwid) = cur;
+    if (menu_caching) cache (xwid) = cur;
     return true;
   }
 }
@@ -660,6 +873,7 @@ QTMRefreshWidget::deleteLayout (QLayout* l) {
 
 void
 QTMRefreshWidget::doRefresh (string kind) {
+BEGIN_SLOT
   if (recompute (kind)) {
     if (qwid) qwid->setParent (NULL);
     delete qwid;
@@ -677,6 +891,7 @@ QTMRefreshWidget::doRefresh (string kind) {
         window()->maximumSize() != QSize (QWIDGETSIZE_MAX, QWIDGETSIZE_MAX))
       window()->setFixedSize (window()->sizeHint());  
   }
+END_SLOT
 }
 
 
@@ -737,6 +952,7 @@ QTMRefreshableWidget::deleteLayout (QLayout* l) {
 
 void
 QTMRefreshableWidget::doRefresh (string kind) {
+BEGIN_SLOT
   if (recompute (kind)) {
     if (qwid) qwid->setParent (NULL);
     delete qwid;
@@ -754,6 +970,7 @@ QTMRefreshableWidget::doRefresh (string kind) {
         window()->maximumSize() != QSize (QWIDGETSIZE_MAX, QWIDGETSIZE_MAX))
       window()->setFixedSize (window()->sizeHint());  
   }
+END_SLOT
 }
 
 
@@ -839,8 +1056,7 @@ QTMComboBox::event (QEvent* ev) {
  It also scrolls the viewport to the position of selected items in QListWidgets.
  */
 void
-QTMScrollArea::setWidgetAndConnect (QWidget* w)
-{
+QTMScrollArea::setWidgetAndConnect (QWidget* w) {
   setWidget (w);
  
   listViews = w->findChildren<QTMListView*>();
@@ -853,8 +1069,8 @@ QTMScrollArea::setWidgetAndConnect (QWidget* w)
 
 /*! Scrolls the area to a given index in a QTMListView. */
 void
-QTMScrollArea::scrollToSelection (const QItemSelection& sel)
-{
+QTMScrollArea::scrollToSelection (const QItemSelection& sel) {
+BEGIN_SLOT
   if (sel.isEmpty())
     return;
 
@@ -868,6 +1084,7 @@ QTMScrollArea::scrollToSelection (const QItemSelection& sel)
     if (! viewport()->geometry().contains (x, y))
       ensureVisible (x, y, r.width(), r.height());
   }
+END_SLOT
 }
 
 /*! Work around a problem with scrolling before the widget is shown.
@@ -876,8 +1093,7 @@ QTMScrollArea::scrollToSelection (const QItemSelection& sel)
  insufficient amount. See the comments to QTMScrollArea.
  */
 void
-QTMScrollArea::showEvent (QShowEvent* ev)
-{
+QTMScrollArea::showEvent (QShowEvent* ev) {
   for (ListViewsIterator it = listViews.begin(); it != listViews.end(); ++it) {
     QItemSelection sel = (*it)->selectionModel()->selection();
     (*it)->selectionChanged (sel, sel);
@@ -885,7 +1101,6 @@ QTMScrollArea::showEvent (QShowEvent* ev)
   QScrollArea::showEvent (ev);
 }
 
-
 /******************************************************************************
  * QTMListView
  ******************************************************************************/
@@ -911,12 +1126,18 @@ QTMListView::QTMListView (const command& cmd,
   setSelectionMode (multiple ? ExtendedSelection : SingleSelection);
   setEditTriggers (NoEditTriggers);
 
+    // NOTE: using selectionModel()->select(item, QItemSelection::SelectCurrent)
+    // doesn't update the selection but overwrites it, so we explicitly define
+    // our QItemSelection and use merge()
+  QItemSelection sel;
   for (int i = 0; i < model()->rowCount(); ++i) {
     QModelIndex item = model()->index (i, 0);
-    if (selections.contains (model()->data (item, Qt::DisplayRole).toString(), Qt::CaseSensitive))
-      selectionModel()->select (item, QItemSelectionModel::SelectCurrent);
+    if (selections.contains (model()->data (item, Qt::DisplayRole).toString(),
+                             Qt::CaseSensitive))
+      sel.merge (QItemSelection(item, item), QItemSelectionModel::Select);
   }
-
+  selectionModel()->select (sel, QItemSelectionModel::Select);
+  
   if (!scroll) {
     setMinimumWidth (sizeHintForColumn(0));
     setMinimumHeight (sizeHintForRow(0) * model()->rowCount());
@@ -943,6 +1164,57 @@ QTMListView::QTMListView (const command& cmd,
  */
 void
 QTMListView::selectionChanged (const QItemSelection& c, const QItemSelection& p) {
+BEGIN_SLOT
   QListView::selectionChanged (c, p);
   emit selectionChanged (c);
+END_SLOT
 }
+
+/******************************************************************************
+ * QTMTreeView
+ ******************************************************************************/
+
+QTMTreeView::QTMTreeView (command cmd, tree data, const tree& roles, QWidget* p)
+: QTreeView (p), _t (data), _cmd (cmd) {
+  setModel (QTMTreeModel::instance (_t, roles));
+  setUniformRowHeights (true);  // assuming we display only text.
+  setHeaderHidden (true);       // for now...
+  QObject::connect (this, SIGNAL (pressed (const QModelIndex&)),
+                    this,   SLOT (callOnChange (const QModelIndex&)));
+}
+
+void
+QTMTreeView::currentChanged (const QModelIndex& curr, const QModelIndex& prev) {
+  (void) prev;
+  if (selectedIndexes().contains(curr))
+    callOnChange (curr, false);
+}
+
+void
+QTMTreeView::callOnChange (const QModelIndex& index, bool mouse) {
+BEGIN_SLOT
+  object arguments = mouse ? list_object ((int)QApplication::mouseButtons())
+                           : list_object (-1);
+    
+    // docs state the index is valid, no need to check
+  QVariant d = tmModel()->data (index, QTMTreeModel::CommandRole);
+    // If there's no CommandRole, we return the subtree by default
+  if (!d.isValid() || !d.canConvert (QVariant::String))
+    arguments = cons (tmModel()->item_from_index (index), arguments);
+  else
+    arguments = cons (from_qstring (d.toString()), arguments);
+  int cnt = QTMTreeModel::TMUserRole;
+  d = tmModel()->data (index, cnt);
+  while (d.isValid() && d.canConvert (QVariant::String)) {
+    arguments = cons (from_qstring (d.toString()), arguments);
+    d = tmModel()->data (index, ++cnt);
+  }
+  _cmd (arguments);
+END_SLOT
+}
+
+inline QTMTreeModel*
+QTMTreeView::tmModel() const {
+  return static_cast<QTMTreeModel*> (model());
+}
+
