@@ -245,12 +245,15 @@ class QTMLineEdit : public QLineEdit {
 
   bool completing;
   string     type; // type of input field
+  string     name; // optional name of input field
+  string   serial; // optional serial number of input field
   string       ww; // width of the parsed widget
   command     cmd;
   int    last_key;
 public:
   QTMLineEdit (QWidget *parent, string _type, string _ww,
                int style=0, command _cmd= command ());
+  void set_type (string t);
   bool continuous ();
   virtual QSize	sizeHint () const ;
   virtual bool event (QEvent* ev);
@@ -318,6 +321,7 @@ class QTMFieldWidgetHelper : public QObject {
   
 public:
   QTMFieldWidgetHelper (qt_widget _wid, QComboBox* parent);
+  QTMFieldWidgetHelper (qt_widget _wid, QLineEdit* parent);
 
 public slots:
   void commit (const QString& qst);
@@ -447,9 +451,12 @@ class QTMTreeView : public QTreeView {
 public:
   QTMTreeView (command cmd, tree data, const tree& roles, QWidget* parent=0);
   QTMTreeModel* tmModel() const;
-
+  
+protected:
+  virtual void currentChanged (const QModelIndex& cur, const QModelIndex& pre);
+  
 private slots:
-  void callOnClick (const QModelIndex& index);
+  void callOnChange (const QModelIndex& index, bool mouse=true);
 };
 
 
