@@ -47,9 +47,9 @@ qt_widget_rep::~qt_widget_rep() {
     debug_widgets << "~qt_widget_rep: deleted a " << type_as_string() << LF;
 
   // DON'T DO THIS! (several qt_widget_rep may have the same underlying QWidget)
-  // UPD: really? when? The problem is rather order of destruction. If we
-  // completely bypassed QObject's hierarchy deleting should be ok.
-  //delete qwid;
+  // UPD: really? when? And it wouldn't matter since we are using a guarded
+  // pointer
+//  delete qwid;
 }
 
 void
@@ -509,8 +509,8 @@ widget input_text_widget (command call_back, string type, array<string> def,
 widget color_picker_widget (command call_back, bool bg, array<tree> proposals) {
   return tm_new<qt_color_picker_widget_rep> (call_back, bg, proposals);
 }
-widget file_chooser_widget (command cmd, string type, bool save) {
-  return tm_new<qt_chooser_widget_rep> (cmd, type, save);
+widget file_chooser_widget (command cmd, string type, string prompt) {
+  return tm_new<qt_chooser_widget_rep> (cmd, type, prompt);
 }
 widget printer_widget (command cmd, url ps_pdf_file) {
   return tm_new<qt_printer_widget_rep> (cmd, ps_pdf_file);
