@@ -14,6 +14,7 @@
 
 #include "widget.hpp"
 #include "message.hpp"
+#include <QPointer>
 
 class QWidget;
 class QLayoutItem;
@@ -60,9 +61,10 @@ class qt_widget;
  * as_qwidget() returns a regular QWidget, for example to be embedded in a
    standalone window.
  
- * get_qmenu() wraps its as_qaction() in a QMenu and returns it. This method
-   also promotes the object where it was invoked to owner of the QMenu. Again:
-   calling wid->get_qmenu() creates the QMenu and leaves ownership of it to wid.
+ get_qmenu() returns the QMenu owned by the QAction returned by as_qaction().
+ This method also promotes the object upon which it was invoked to owner of the
+ QMenu. Again: calling wid->get_qmenu() creates the QMenu and leaves ownership
+ of it to wid.
  
  In the first three cases a new instance of the QObject is created if neede
  and ownership is transferred to the caller. One reason why the underlying
@@ -85,7 +87,8 @@ protected:
   array<widget> children;
 public:
   long                id;
-  QWidget*          qwid;
+  QPointer<QWidget> qwid;
+
   /*! A list of all supported widget types.
    FIXME: This enum breaks the basic inheritance rules, since we have to 
    update the base class each time we implement a new subclass. It's also some
