@@ -40,6 +40,8 @@
   (tm-errput tm_errput (void string))
   (win32-display win32_display (void string))
   (cpp-error cpp_error (void))
+  (supports-native-pdf? supports_native_pdf (bool))
+  (supports-ghostscript? supports_ghostscript (bool))
   (rescue-mode? in_rescue_mode (bool))
   (scheme-dialect scheme_dialect (string))
   (get-texmacs-path get_texmacs_path (url))
@@ -49,23 +51,29 @@
   (font-exists-in-tt? tt_font_exists (bool string))
   (eval-system eval_system (string string))
   (var-eval-system var_eval_system (string string))
+  (evaluate-system evaluate_system
+    (array_string array_string array_int array_string array_int))
   (get-locale-language get_locale_language (string))
   (get-locale-charset get_locale_charset (string))
   (locale-to-language locale_to_language (string string))
   (language-to-locale language_to_locale (string string))
   (texmacs-time texmacs_time (int))
+  (pretty-time pretty_time (string int))
   (texmacs-memory mem_used (int))
   (bench-print bench_print (void string))
   (bench-print-all bench_print (void))
   (system-wait system_wait (void string string))
   (set-latex-command set_latex_command (void string))
   (set-bibtex-command set_bibtex_command (void string))
+  (number-latex-errors number_latex_errors (int url))
+  (number-latex-pages number_latex_pages (int url))
   (math-symbol-group math_symbol_group (string string))
   (math-group-members math_group_members (array_string string))
   (math-symbol-type math_symbol_type (string string))
   (object->command as_command (command object))
   (exec-delayed exec_delayed (void object))
   (exec-delayed-pause exec_delayed_pause (void object))
+  (protected-call protected_call (void object))
   (notify-preferences-booted notify_preferences_booted (void))
   (cpp-has-preference? has_user_preference (bool string))
   (cpp-get-preference get_user_preference (string string string))
@@ -83,11 +91,15 @@
   (tree-translate-from-to tree_translate (tree content string string))
   (force-load-translations force_load_dictionary (void string string))
   (color named_color (int string))
+  (get-hex-color get_hex_color (string string))
+  (named-color->xcolormap named_color_to_xcolormap (string string))
   (new-author new_author (double))
   (set-author set_author (void double))
   (get-author get_author (double))
   (debug-set debug_set (void string bool))
   (debug-get debug_get (bool string))
+  (get-debug-messages get_debug_messages (tree string int))
+  (clear-debug-messages clear_debug_messages (void))
   (cout-buffer cout_buffer (void))
   (cout-unbuffer cout_unbuffer (string))
   (mark-new new_marker (double))
@@ -96,6 +108,7 @@
   (set-new-fonts set_new_fonts (void bool))
   (new-fonts? get_new_fonts (bool))
   (tmtm-eqnumber->nonumber eqnumber_to_nonumber (tree tree))
+  (busy-versioning? is_busy_versioning (bool))
 
   ;; routines for the font database
   (tt-exists? tt_font_exists (bool string))
@@ -118,6 +131,8 @@
   (font-database-search font_database_search (array_string string string))
   (font-database-characteristics
    font_database_characteristics (array_string string string))
+  (font-database-substitutions
+   font_database_substitutions (scheme_tree string))
   (font-family->master family_to_master (string string))
   (font-master->families master_to_families (array_string string))
   (font-master-features master_features (array_string string))
@@ -144,6 +159,7 @@
   (search-font-families search_font_families (array_string array_string))
   (search-font-styles search_font_styles (array_string string array_string))
   (logical-font-patch patch_font (array_string array_string array_string))
+  (logical-font-substitute apply_substitutions (array_string array_string))
   (font-family-main main_family (string string))
 
   ;; routines for images
@@ -192,15 +208,19 @@
   (tree-child-long-name get_child_long_name (string content int))
   (tree-child-type get_child_type (string content int))
   (tree-child-env get_env_child (tree content int string content))
+  (tree-descendant-env get_env_descendant (tree content path content))
   (tree-load-inclusion load_inclusion (tree url))
   (tree-as-string tree_as_string (string content))
   (tree-extents tree_extents (tree content))
   (tree-empty? is_empty (bool content))
   (tree-is-buffer? admits_edit_observer (bool tree))
   (tree-search-sections search_sections (array_tree tree))
+  (tree-search-tree search (array_path content content path int))
   (tag-minimal-arity minimal_arity (int tree_label))
   (tag-maximal-arity maximal_arity (int tree_label))
   (tag-possible-arity? correct_arity (bool tree_label int))
+  (set-access-mode set_access_mode (int int))
+  (get-access-mode get_access_mode (int))
 
   (tree-assign tree_assign (tree tree content))
   (tree-var-insert tree_insert (tree tree int content))
@@ -236,6 +256,7 @@
   (math-status-reset math_status_reset (void))
 
   ;; paths
+  (path-strip strip (path path path))
   (path-inf? path_inf (bool path path))
   (path-inf-eq? path_inf_eq (bool path path))
   (path-less? path_less (bool path path))
@@ -258,6 +279,67 @@
   (path-previous-argument previous_argument (path content path))
   (path-previous-section previous_section (path content path))
 
+  ;; modifications on trees
+  (make-modification make_modification (modification string path content))
+  (modification-assign mod_assign (modification path content))
+  (modification-insert mod_insert (modification path int content))
+  (modification-remove mod_remove (modification path int int))
+  (modification-split mod_split (modification path int int))
+  (modification-join mod_join (modification path int))
+  (modification-assign-node mod_assign_node (modification path tree_label))
+  (modification-insert-node mod_insert_node (modification path int content))
+  (modification-remove-node mod_remove_node (modification path int))
+  (modification-set-cursor mod_set_cursor (modification path int content))
+  (modification-kind get_type (string modification))
+  (modification-path get_path (path modification))
+  (modification-tree get_tree (tree modification))
+  (modification-root root (path modification))
+  (modification-index index (int modification))
+  (modification-argument argument (int modification))
+  (modification-label L (tree_label modification))
+
+  (modification-copy copy (modification modification))
+  (modification-applicable? is_applicable (bool content modification))
+  (modification-apply var_clean_apply (tree content modification))
+  (modification-inplace-apply var_apply (tree tree modification))
+  (modification-invert invert (modification modification content))
+  (modification-commute? commute (bool modification modification))
+  (modification-can-pull? can_pull (bool modification modification))
+  (modification-pull pull (modification modification modification))
+  (modification-co-pull co_pull (modification modification modification))
+
+  ;; patches
+  (patch-pair patch (patch modification modification))
+  (patch-compound patch (patch array_patch))
+  (patch-branch branch_patch (patch array_patch))
+  (patch-birth patch (patch double bool))
+  (patch-author patch (patch double patch))
+  (patch-pair? is_modification (bool patch))
+  (patch-compound? is_compound (bool patch))
+  (patch-branch? is_branch (bool patch))
+  (patch-birth? is_birth (bool patch))
+  (patch-author? is_author (bool patch))
+  (patch-arity N (int patch))
+  (patch-ref access (patch patch int))
+  (patch-direct get_modification (modification patch))
+  (patch-inverse get_inverse (modification patch))
+  (patch-get-birth get_birth (bool patch))
+  (patch-get-author get_author (double patch))
+
+  (patch-copy copy (patch patch))
+  (patch-applicable? is_applicable (bool patch content))
+  (patch-apply var_clean_apply (tree content patch))
+  (patch-inplace-apply var_apply (tree tree patch))
+  (patch-compactify compactify (patch patch))
+  (patch-cursor-hint cursor_hint (path patch content))
+  (patch-invert invert (patch patch content))
+  (patch-commute? commute (bool patch patch))
+  (patch-can-pull? can_pull (bool patch patch))
+  (patch-pull pull (patch patch patch))
+  (patch-co-pull co_pull (patch patch patch))
+  (patch-remove-set-cursor remove_set_cursor (patch patch))
+  (patch-modifies? does_modify (bool patch))
+
   ;; links
   (tree->ids get_ids (list_string tree))
   (id->trees get_trees (list_tree string))
@@ -283,6 +365,7 @@
   (string-count-occurrences count_occurrences (int string string))
   (string-search-forwards search_forwards (int string int string))
   (string-search-backwards search_backwards (int string int string))
+  (string-overlapping overlapping (int string string))
   (string-replace replace (string string string string))
   (string-alpha? is_alpha (bool string))
   (string-locase-alpha? is_locase_alpha (bool string))
@@ -300,6 +383,10 @@
   (string-quote scm_quote (string string))
   (string-unquote scm_unquote (string string))
   (string-convert convert (string string string string))
+  (encode-base64 encode_base64 (string string))
+  (decode-base64 decode_base64 (string string))
+  (sourcecode->cork sourcecode_to_cork (string string))
+  (cork->sourcecode cork_to_sourcecode (string string))
   (utf8->cork utf8_to_cork (string string))
   (cork->utf8 cork_to_utf8 (string string))
   (utf8->t2a utf8_to_t2a (string string))
@@ -315,6 +402,12 @@
   (integer->hexadecimal as_hexadecimal (string int))
   (integer->padded-hexadecimal as_hexadecimal (string int int))
   (hexadecimal->integer from_hexadecimal (int string))
+  (cpp-string-tokenize tokenize (array_string string string))
+  (cpp-string-recompose recompose (string array_string string))
+  
+  ; Program bracket matching
+  (find-left-bracket find_left_bracket (path path string string))
+  (find-right-bracket find_right_bracket (path path string string))
 
   ;; routines for strings in the TeXmacs encoding
   (string->tmstring tm_encode (string string))
@@ -326,6 +419,13 @@
   (list->tmstring tm_recompose (string array_string))
   (string-next tm_char_next (int string int))
   (string-previous tm_char_previous (int string int))
+  (tmstring-split tm_string_split (array_string string))
+
+  (tmstring-translit uni_translit (string string))
+  (tmstring-locase-first uni_locase_first (string string))
+  (tmstring-upcase-first uni_upcase_first (string string))
+  (tmstring-locase-all uni_locase_all (string string))
+  (tmstring-upcase-all uni_upcase_all (string string))
 
   ;; Packrat grammar and parsing tools
   (packrat-define packrat_define (void string string tree))
@@ -350,15 +450,33 @@
   (parse-latex parse_latex (tree string))
   (parse-latex-document parse_latex_document (tree string))
   (latex->texmacs latex_to_tree (tree tree))
-  (cpp-latex-document->texmacs latex_document_to_tree (tree string bool bool))
+  (cpp-latex-document->texmacs latex_document_to_tree (tree string bool))
   (latex-class-document->texmacs latex_class_document_to_tree (tree string))
+  (tracked-latex->texmacs tracked_latex_to_texmacs (tree string bool))
+  (conservative-texmacs->latex conservative_texmacs_to_latex
+                               (string content object))
+  (tracked-texmacs->latex tracked_texmacs_to_latex (string content object))
+  (conservative-latex->texmacs conservative_latex_to_texmacs (tree string bool))
+  (get-line-number get_line_number (int string int))
+  (get-column-number get_column_number (int string int))
+  (try-latex-export try_latex_export (tree content object url url))
   (parse-xml parse_xml (scheme_tree string))
   (parse-html parse_html (scheme_tree string))
   (parse-bib parse_bib (tree string))
+  (conservative-bib-import conservative_bib_import
+                           (tree string content string))
+  (conservative-bib-export conservative_bib_export
+                           (string content string content))
   (upgrade-tmml tmml_upgrade (tree scheme_tree))
   (upgrade-mathml upgrade_mathml (tree content))
   (vernac->texmacs vernac_to_tree (tree string))
   (vernac-document->texmacs vernac_document_to_tree (tree string))
+  (compute-keys-string compute_keys (array_string string string))
+  (compute-keys-tree compute_keys (array_string content string))
+  (compute-keys-url compute_keys (array_string url))
+  (compute-index-string compute_index (scheme_tree string string))
+  (compute-index-tree compute_index (scheme_tree content string))
+  (compute-index-url compute_index (scheme_tree url))
 
   ;; routines for urls
   (url->url url (url url))
@@ -381,10 +499,13 @@
   (url-or url_or (url url url))
   (url-none? is_none (bool url))
   (url-rooted? is_rooted (bool url))
+  (url-rooted-protocol? is_rooted (bool url string))
   (url-rooted-web? is_rooted_web (bool url))
   (url-rooted-tmfs? is_rooted_tmfs (bool url))
+  (url-rooted-tmfs-protocol? is_rooted_tmfs (bool url string))
   (url-root get_root (string url))
   (url-unroot unroot (url url))
+  (url-atomic? is_atomic (bool url))
   (url-concat? is_concat (bool url))
   (url-or? is_or (bool url))
   (url-ref url_ref (url url int))
@@ -422,14 +543,18 @@
   (url-cache-invalidate web_cache_invalidate (void url))
   (string-save string_save (void string url))
   (string-load string_load (string url))
+  (string-append-to-file string_append_to_file (void string url))
   (system-move move (void url url))
   (system-copy copy (void url url))
   (system-remove remove (void url))
   (system-mkdir mkdir (void url))
+  (system-rmdir rmdir (void url))
   (system-search-score search_score (int url array_string))
   (system-1 system (void string url))
   (system-2 system (void string url url))
+  (system-url->string sys_concretize (string url))
   (url-grep grep (url string url))
+  (url-search-upwards search_file_upwards (url url string array_string))
 
   ;; Persistent data
   (persistent-set persistent_set (void url string string))
@@ -438,14 +563,31 @@
   (persistent-get persistent_get (string url string))
   (persistent-file-name persistent_file_name (url url string))
 
+  ;; native TeXmacs databases
+  (tmdb-keep-history keep_history (void url bool))
+  (tmdb-set-field set_field (void url string string array_string double))
+  (tmdb-get-field get_field (array_string url string string double))
+  (tmdb-remove-field remove_field (void url string string double))
+  (tmdb-get-attributes get_attributes (array_string url string double))
+  (tmdb-set-entry set_entry (void url string scheme_tree double))
+  (tmdb-get-entry get_entry (scheme_tree url string double))
+  (tmdb-remove-entry remove_entry (void url string double))
+  (tmdb-query query (array_string url scheme_tree double int))
+  (tmdb-inspect-history inspect_history (void url string))
+  (tmdb-get-completions get_completions (array_string url string))
+  (tmdb-get-name-completions get_name_completions (array_string url string))
+
   ;; SQL interface
+  (supports-sql? sqlite3_present (bool))
   (sql-exec sql_exec (scheme_tree url string))
+  (sql-quote sql_quote (string string))
 
   ;; TeXmacs servers and clients
   (server-start server_start (void))
   (server-stop server_stop (void))
   (server-read server_read (string int))
   (server-write server_write (void int string))
+  (server-started? server_started (bool))
   (client-start client_start (int string))
   (client-stop client_stop (void int))
   (client-read client_read (string int))
@@ -486,6 +628,7 @@
   (widget-choices choice_widget (widget command array_string array_string))
   (widget-filtered-choice choice_widget
     (widget command array_string string string))
+  (widget-tree-view tree_view_widget (widget command tree tree))
   (widget-xpm xpm_widget (widget url))
   (widget-box box_widget (widget scheme_tree string int bool bool))
   (widget-glue glue_widget (widget bool bool int int))
@@ -497,7 +640,7 @@
   (widget-icon-tabs icon_tabs_widget (widget array_url array_widget
                                              array_widget))
   (widget-scrollable user_canvas_widget (widget widget int))
-  (widget-resize resize_widget (widget widget int string string
+  (widget-resize resize_widget (widget widget int string string string string
                                        string string string string))
   (widget-hsplit hsplit_widget (widget widget widget))
   (widget-vsplit vsplit_widget (widget widget widget))
@@ -508,6 +651,7 @@
   (widget-refreshable refreshable_widget (widget object string))
   (object->promise-widget as_promise_widget (promise_widget object))
   (tree-bounding-rectangle get_bounding_rectangle (array_int tree))
+  (widget-size get_widget_size (array_int widget))
   (show-balloon show_help_balloon (void widget int int))
   (get-style-menu get_style_menu (object))
   (hidden-package? hidden_package (bool string))
@@ -537,6 +681,7 @@
   (buffer-pretend-modified pretend_buffer_modified (void url))
   (buffer-pretend-saved pretend_buffer_saved (void url))
   (buffer-pretend-autosaved pretend_buffer_autosaved (void url))
+  (buffer-attach-notifier attach_buffer_notifier (void url))
   (buffer-has-name? buffer_has_name (bool url))
   (buffer-aux? is_aux_buffer (bool url))
   (buffer-import buffer_import (bool url url string))
@@ -576,8 +721,8 @@
   (open-window-geometry open_window (url content))
   (clone-window clone_window (void))
   (buffer-close kill_buffer (void url))
-  (kill-window kill_window (void))
-  (kill-window-and-buffer kill_window_and_buffer (void))
+  (kill-window kill_window (void url))
+  (kill-current-window-and-buffer kill_current_window_and_buffer (void))
 
   (project-attach project_attach (void string))
   (project-detach project_attach (void))
@@ -597,7 +742,9 @@
   (alt-window-set-position window_set_position (void int int int))
 
   ;; routines for BibTeX
+  (bibtex-run bibtex_run (tree string string url array_string))
   (bib-add-period bib_add_period (scheme_tree scheme_tree))
+  (bib-locase-first bib_locase_first (scheme_tree scheme_tree))
   (bib-upcase-first bib_upcase_first (scheme_tree scheme_tree))
   (bib-locase bib_locase (scheme_tree scheme_tree))
   (bib-upcase bib_upcase (scheme_tree scheme_tree))
