@@ -39,25 +39,25 @@
 
   ;; general modification routines
   (inside? inside (bool tree_label))
-  (insert insert_tree (void content))
-  (insert-go-to var_insert_tree (void content path))
+  (cpp-insert insert_tree (void content))
+  (cpp-insert-go-to var_insert_tree (void content path))
   (insert-raw-go-to insert_tree (void content path))
   (insert-raw-return insert_return (void))
   (remove-text remove_text (void bool))
   (remove-structure remove_structure (void bool))
   (remove-structure-upwards remove_structure_upwards (void))
 
-  (make make_compound (void tree_label))
-  (make-arity make_compound (void tree_label int))
+  (cpp-make make_compound (void tree_label))
+  (cpp-make-arity make_compound (void tree_label int))
   (activate activate (void))
   (insert-argument insert_argument (void bool))
   (remove-argument remove_argument (void bool))
   (insert-argument-at insert_argument (void path bool))
   (remove-argument-at remove_argument (void path bool))
-  (make-with make_with (void string string))
+  (cpp-make-with make_with (void string string))
   (make-mod-active make_mod_active (void tree_label))
   (make-style-with make_style_with (void string string))
-  (make-hybrid make_hybrid (void))
+  (cpp-make-hybrid make_hybrid (void))
   (activate-latex activate_latex (void))
   (activate-hybrid activate_hybrid (void bool))
   (activate-symbol activate_symbol (void))
@@ -84,6 +84,10 @@
   (init-has? defined_in_init (bool string))
   (get-page-width get_page_width (int))
   (get-page-height get_page_height (int))
+  (get-attachment get_att (tree string))
+  (set-attachment set_att (void string content))
+  (reset-attachment reset_att (void string))
+  (list-attachments list_atts (array_string))
 
   ;; modify text
   (make-htab make_htab (void string))
@@ -105,19 +109,19 @@
   (length-divide divide_lengths (double string string))
 
   ;; modify mathematics
-  (make-rigid make_rigid (void))
-  (make-lprime make_lprime (void string))
-  (make-rprime make_rprime (void string))
-  (make-below make_below (void))
-  (make-above make_above (void))
-  (make-script make_script (void bool bool))
-  (make-fraction make_fraction (void))
-  (make-sqrt make_sqrt (void))
-  (make-wide make_wide (void string))
-  (make-wide-under make_wide_under (void string))
-  (make-var-sqrt make_var_sqrt (void))
-  (make-neg make_neg (void))
-  (make-tree make_tree (void))
+  (cpp-make-rigid make_rigid (void))
+  (cpp-make-lprime make_lprime (void string))
+  (cpp-make-rprime make_rprime (void string))
+  (cpp-make-below make_below (void))
+  (cpp-make-above make_above (void))
+  (cpp-make-script make_script (void bool bool))
+  (cpp-make-fraction make_fraction (void))
+  (cpp-make-sqrt make_sqrt (void))
+  (cpp-make-wide make_wide (void string))
+  (cpp-make-wide-under make_wide_under (void string))
+  (cpp-make-var-sqrt make_var_sqrt (void))
+  (cpp-make-neg make_neg (void))
+  (cpp-make-tree make_tree (void))
 
   ;; modify tables
   (make-subtable make_subtable (void))
@@ -129,12 +133,16 @@
   (table-remove-column table_remove_column (void bool))
   (table-nr-rows table_nr_rows (int))
   (table-nr-columns table_nr_columns (int))
+  (table-get-extents table_get_extents (array_int))
   (table-set-extents table_set_extents (void int int))
   (table-which-row table_which_row (int))
   (table-which-column table_which_column (int))
+  (table-which-cells table_which_cells (array_int))
+  
   (table-cell-path table_search_cell (path int int))
   (table-go-to table_go_to (void int int))
   (table-set-format table_set_format (void string content))
+  (table-get-format-all table_get_format (tree))
   (table-get-format table_get_format (string string))
   (table-del-format table_del_format (void string))
   (table-row-decoration table_row_decoration (void bool))
@@ -181,6 +189,7 @@
   (go-start-paragraph go_start_paragraph (void))
   (go-end-paragraph go_end_paragraph (void))
   (go-to-label go_to_label (void string))
+  (cursor-accessible? cursor_is_accessible (bool))
   (cursor-show-if-hidden show_cursor_if_hidden (void))
 
   ;; selections
@@ -203,11 +212,11 @@
   (selection-path selection_get_path (path))
   (selection-set selection_set_paths (void path path))
   (selection-set-range-set selection_set_range_set (void array_path))
-  (clipboard-copy selection_copy (void string))
-  (clipboard-cut selection_cut (void string))
+  (cpp-clipboard-copy selection_copy (void string))
+  (cpp-clipboard-cut selection_cut (void string))
   (clipboard-cut-at cut (void path))
   (clipboard-cut-between cut (void path path))
-  (clipboard-paste selection_paste (void string))
+  (cpp-clipboard-paste selection_paste (void string))
   (selection-move selection_move (void))
   (clipboard-clear selection_clear (void string))
   (selection-cancel selection_cancel (void))
@@ -238,6 +247,7 @@
   (redo-possibilities redo_possibilities (int))
   (redo redo (void int))
   (show-history show_history (void))
+  (archive-state archive_state (void))
   (start-editing start_editing (void))
   (end-editing end_editing (void))
   (cancel-editing cancel_editing (void))
