@@ -37,7 +37,11 @@
 #endif // __MINGW32__
 
 #if defined(GUILE_D) || defined(GUILE_C) 
+#ifdef GUILE_BIN_guile18
+#include <libguile18.h>
+#else
 #include <libguile.h>
+#endif
 #else
 #include <guile/gh.h>
 #endif
