@@ -13,32 +13,36 @@
 
 #include "promise.hpp"
 #include "tree.hpp"
+#include "drd_mode.hpp"
 #include "tree_search.hpp"
+#include "modification.hpp"
+#include "patch.hpp"
 
 #include "boxes.hpp"
 #include "editor.hpp"
-#include "analyze.hpp"
+#include "universal.hpp"
 #include "convert.hpp"
 #include "file.hpp"
 #include "iterator.hpp"
 #include "Freetype/tt_tools.hpp"
+#include "Database/database.hpp"
 #include "Sqlite3/sqlite3.hpp"
 #include "Updater/tm_updater.hpp"
 
 tmscm 
-blackboxP (tmscm  t) {
+blackboxP (tmscm t) {
   bool b= tmscm_is_blackbox (t);
   return bool_to_tmscm (b);
 }
 
 #if 0
-template<class T> tmscm  box_to_tmscm (T o) {
+template<class T> tmscm box_to_tmscm (T o) {
   return blackbox_to_tmscm (close_box<T> (o)); }
-template<class T> T tmscm_to_box (tmscm  obj) { 
+template<class T> T tmscm_to_box (tmscm obj) { 
   return open_box<T>(tmscm_to_blackbox (obj));  }
-template<class T> tmscm  cmp_box (tmscm  o1, tmscm  o2) { 
+template<class T> tmscm cmp_box (tmscm o1, tmscm o2) { 
   return bool_to_tmscm (tmscm_to_box<T> (o1) == tmscm_to_box<T> (o2)); }
-template<class T> tmscm  boxP (tmscm  t) {
+template<class T> tmscm boxP (tmscm t) {
   bool b= tmscm_is_blackbox (t) && 
           (type_box (blackboxvalue(t)) == type_helper<T>::id);
   return bool_to_tmscm (b);
@@ -125,6 +129,11 @@ supports_ghostscript () {
 #endif
 }
 
+bool
+is_busy_versioning () {
+  return busy_versioning;
+}
+
 /******************************************************************************
 * Redirections
 ******************************************************************************/
@@ -154,6 +163,10 @@ TMSCM_ASSERT (tmscm_is_int (i), i, arg, rout);
 //TMSCM_ASSERT (SCM_REALP (i), i, arg, rout);
 #define TMSCM_ASSERT_URL(u,arg,rout) \
 TMSCM_ASSERT (tmscm_is_url (u) || tmscm_is_string (u), u, arg, rout)
+#define TMSCM_ASSERT_MODIFICATION(m,arg,rout) \
+TMSCM_ASSERT (tmscm_is_modification (m), m, arg, rout)
+#define TMSCM_ASSERT_PATCH(p,arg,rout) \
+TMSCM_ASSERT (tmscm_is_patch (p), p, arg, rout)
 #define TMSCM_ASSERT_BLACKBOX(t,arg,rout) \
 TMSCM_ASSERT (tmscm_is_blackbox (t), t, arg, rout)
 #define TMSCM_ASSERT_SYMBOL(s,arg,rout) \
@@ -176,7 +189,7 @@ tree_label_to_tmscm (tree_label l) {
 }
 
 tree_label
-tmscm_to_tree_label (tmscm  p) {
+tmscm_to_tree_label (tmscm p) {
   string s= tmscm_to_symbol (p);
   return make_tree_label (s);
 }
@@ -189,7 +202,7 @@ tmscm_to_tree_label (tmscm  p) {
 
 
 bool
-tmscm_is_tree (tmscm  u) {
+tmscm_is_tree (tmscm u) {
   return (tmscm_is_blackbox (u) && 
          (type_box (tmscm_to_blackbox(u)) == type_helper<tree>::id));
 }
@@ -200,12 +213,12 @@ tree_to_tmscm (tree o) {
 }
 
 tree
-tmscm_to_tree (tmscm  obj) {
+tmscm_to_tree (tmscm obj) {
   return open_box<tree>(tmscm_to_blackbox (obj));
 }
 
 tmscm 
-treeP (tmscm  t) {
+treeP (tmscm t) {
   bool b= tmscm_is_blackbox (t) && 
           (type_box (tmscm_to_blackbox(t)) == type_helper<tree>::id);
   return bool_to_tmscm (b);
@@ -387,11 +400,12 @@ scheme_tree_to_tmscm (scheme_tree t) {
       return string_to_tmscm (scm_unquote (s));
     //if ((N(s)>=2) && (s[0]=='\42') && (s[N(s)-1]=='\42'))
     //return string_to_tmscm (s (1, N(s)-1));
+    if (N(s) >= 1 && s[0] == '\'') return symbol_to_tmscm (s (1, N(s)));
     return symbol_to_tmscm (s);
   }
   else {
     int i;
-    tmscm  p= tmscm_null ();
+    tmscm p= tmscm_null ();
     for (i=N(t)-1; i>=0; i--)
       p= tmscm_cons (scheme_tree_to_tmscm (t[i]), p);
     return p;
@@ -399,7 +413,7 @@ scheme_tree_to_tmscm (scheme_tree t) {
 }
 
 scheme_tree
-tmscm_to_scheme_tree (tmscm  p) {
+tmscm_to_scheme_tree (tmscm p) {
   if (tmscm_is_list (p)) {
     tree t (TUPLE);
     while (!tmscm_is_null (p)) {
@@ -422,7 +436,7 @@ tmscm_to_scheme_tree (tmscm  p) {
 ******************************************************************************/
 
 bool
-tmscm_is_content (tmscm  p) {
+tmscm_is_content (tmscm p) {
   if (tmscm_is_string (p) || tmscm_is_tree (p)) return true;
   else if (!tmscm_is_pair (p) || !tmscm_is_symbol (tmscm_car (p))) return false;
   else {
@@ -438,7 +452,7 @@ tmscm_is_content (tmscm  p) {
 #define content_to_tmscm tree_to_tmscm
 
 tree
-tmscm_to_content (tmscm  p) {
+tmscm_to_content (tmscm p) {
   if (tmscm_is_string (p)) return tmscm_to_string (p);
   if (tmscm_is_tree (p)) return tmscm_to_tree (p);
   if (tmscm_is_pair (p)) {
@@ -455,7 +469,7 @@ tmscm_to_content (tmscm  p) {
 }
 
 tmscm 
-contentP (tmscm  t) {
+contentP (tmscm t) {
   bool b= tmscm_is_content (t);
   return bool_to_tmscm (b);
 }
@@ -465,7 +479,7 @@ contentP (tmscm  t) {
 ******************************************************************************/
 
 bool
-tmscm_is_path (tmscm  p) {
+tmscm_is_path (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else return tmscm_is_int (tmscm_car (p)) && tmscm_is_path (tmscm_cdr (p));
 }
@@ -480,7 +494,7 @@ path_to_tmscm (path p) {
 }
 
 path
-tmscm_to_path (tmscm  p) {
+tmscm_to_path (tmscm p) {
   if (tmscm_is_null (p)) return path ();
   else return path ((int) tmscm_to_int (tmscm_car (p)), 
                           tmscm_to_path (tmscm_cdr (p)));
@@ -496,7 +510,7 @@ TMSCM_ASSERT (tmscm_is_observer (o), o, arg, rout)
 
 
 bool
-tmscm_is_observer (tmscm  o) {
+tmscm_is_observer (tmscm o) {
   return (tmscm_is_blackbox (o) &&
          (type_box (tmscm_to_blackbox(o)) == type_helper<observer>::id));
 }
@@ -507,12 +521,12 @@ observer_to_tmscm (observer o) {
 }
 
 static observer
-tmscm_to_observer (tmscm  obj) {
+tmscm_to_observer (tmscm obj) {
   return open_box<observer>(tmscm_to_blackbox (obj));
 }
 
 tmscm 
-observerP (tmscm  t) {
+observerP (tmscm t) {
   bool b= tmscm_is_blackbox (t) && 
   (type_box (tmscm_to_blackbox(t)) == type_helper<observer>::id);
   return bool_to_tmscm (b);
@@ -527,7 +541,7 @@ observerP (tmscm  t) {
 TMSCM_ASSERT (tmscm_is_widget (o), o, arg, rout)
 
 bool
-tmscm_is_widget (tmscm  u) {
+tmscm_is_widget (tmscm u) {
   return (tmscm_is_blackbox (u) &&
          (type_box (tmscm_to_blackbox(u)) == type_helper<widget>::id));
 }
@@ -539,7 +553,7 @@ widget_to_tmscm (widget o) {
 }
 
 widget
-tmscm_to_widget (tmscm  o) {
+tmscm_to_widget (tmscm o) {
   return open_box<widget> (tmscm_to_blackbox (o));
 }
 
@@ -551,7 +565,7 @@ tmscm_to_widget (tmscm  o) {
 TMSCM_ASSERT (tmscm_is_command (o), o, arg, rout)
 
 bool
-tmscm_is_command (tmscm  u) {
+tmscm_is_command (tmscm u) {
   return (tmscm_is_blackbox (u) && 
       (type_box (tmscm_to_blackbox(u)) == type_helper<command>::id));
 }
@@ -562,7 +576,7 @@ command_to_tmscm (command o) {
 }
 
 static command
-tmscm_to_command (tmscm  o) {
+tmscm_to_command (tmscm o) {
   return open_box<command> (tmscm_to_blackbox (o));
 }
 
@@ -577,20 +591,18 @@ typedef promise<widget> promise_widget;
 TMSCM_ASSERT (tmscm_is_promise_widget (o), o, arg, rout)
 
 bool
-tmscm_is_promise_widget (tmscm  u) {
+tmscm_is_promise_widget (tmscm u) {
   return (tmscm_is_blackbox (u) && 
          (type_box (tmscm_to_blackbox(u)) == type_helper<promise_widget>::id));
 }
 
-
-
 static tmscm 
 promise_widget_to_tmscm (promise_widget o) {
   return blackbox_to_tmscm (close_box<promise_widget> (o));
 }
 
 static promise_widget
-tmscm_to_promise_widget (tmscm  o) {
+tmscm_to_promise_widget (tmscm o) {
   return open_box<promise_widget> (tmscm_to_blackbox (o));
 }
 
@@ -599,7 +611,7 @@ tmscm_to_promise_widget (tmscm  o) {
 ******************************************************************************/
 
 bool
-tmscm_is_url (tmscm  u) {
+tmscm_is_url (tmscm u) {
   return (tmscm_is_blackbox (u)
               && (type_box (tmscm_to_blackbox(u)) == type_helper<url>::id))
          || (tmscm_is_string(u));
@@ -611,7 +623,7 @@ url_to_tmscm (url u) {
 }
 
 url
-tmscm_to_url (tmscm  obj) {
+tmscm_to_url (tmscm obj) {
   if (tmscm_is_string (obj))
 #ifdef __MINGW32__
     return url_system (tmscm_to_string (obj));
@@ -622,7 +634,7 @@ tmscm_to_url (tmscm  obj) {
 }
 
 tmscm 
-urlP (tmscm  t) {
+urlP (tmscm t) {
   bool b= tmscm_is_url (t);
   return bool_to_tmscm (b);
 }
@@ -632,8 +644,89 @@ url url_or (url u1, url u2) { return u1 | u2; }
 void string_save (string s, url u) { (void) save_string (u, s); }
 string string_load (url u) {
   string s; (void) load_string (u, s, false); return s; }
+void string_append_to_file (string s, url u) { (void) append_string (u, s); }
 url url_ref (url u, int i) { return u[i]; }
 
+/******************************************************************************
+* Modification
+******************************************************************************/
+
+bool
+tmscm_is_modification (tmscm m) {
+  return (tmscm_is_blackbox (m) &&
+	  (type_box (tmscm_to_blackbox(m)) == type_helper<modification>::id))
+    || (tmscm_is_string (m));
+}
+
+tmscm 
+modification_to_tmscm (modification m) {
+  return blackbox_to_tmscm (close_box<modification> (m));
+}
+
+modification
+tmscm_to_modification (tmscm obj) {
+  return open_box<modification> (tmscm_to_blackbox (obj));
+}
+
+tmscm 
+modificationP (tmscm t) {
+  bool b= tmscm_is_modification (t);
+  return bool_to_tmscm (b);
+}
+
+tree
+var_apply (tree& t, modification m) {
+  apply (t, copy (m));
+  return t;
+}
+
+tree
+var_clean_apply (tree& t, modification m) {
+  return clean_apply (t, copy (m));
+}
+
+/******************************************************************************
+* Patch
+******************************************************************************/
+
+bool
+tmscm_is_patch (tmscm p) {
+  return (tmscm_is_blackbox (p) &&
+	  (type_box (tmscm_to_blackbox(p)) == type_helper<patch>::id))
+    || (tmscm_is_string (p));
+}
+
+tmscm 
+patch_to_tmscm (patch p) {
+  return blackbox_to_tmscm (close_box<patch> (p));
+}
+
+patch
+tmscm_to_patch (tmscm obj) {
+  return open_box<patch> (tmscm_to_blackbox (obj));
+}
+
+tmscm 
+patchP (tmscm t) {
+  bool b= tmscm_is_patch (t);
+  return bool_to_tmscm (b);
+}
+
+patch
+branch_patch (array<patch> a) {
+  return patch (true, a);
+}
+
+tree
+var_clean_apply (tree t, patch p) {
+  return clean_apply (copy (p), t);
+}
+
+tree
+var_apply (tree& t, patch p) {
+  apply (copy (p), t);
+  return t;
+}
 
 /******************************************************************************
 * Table types
@@ -642,11 +735,11 @@ url url_ref (url u, int i) { return u[i]; }
 typedef hashmap<string,string> table_string_string;
 
 static bool
-tmscm_is_table_string_string (tmscm  p) {
+tmscm_is_table_string_string (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else if (!tmscm_is_pair (p)) return false;
   else {
-    tmscm  f= tmscm_car (p);
+    tmscm f= tmscm_car (p);
     return tmscm_is_pair (f) &&
     tmscm_is_string (tmscm_car (f)) &&
     tmscm_is_string (tmscm_cdr (f)) &&
@@ -659,21 +752,21 @@ TMSCM_ASSERT (tmscm_is_table_string_string (p), p, arg, rout)
 
 tmscm 
 table_string_string_to_tmscm (hashmap<string,string> t) {
-  tmscm  p= tmscm_null ();
+  tmscm p= tmscm_null ();
   iterator<string> it= iterate (t);
   while (it->busy ()) {
     string s= it->next ();
-    tmscm  n= tmscm_cons (string_to_tmscm (s), string_to_tmscm (t[s]));
+    tmscm n= tmscm_cons (string_to_tmscm (s), string_to_tmscm (t[s]));
     p= tmscm_cons (n, p);
   }
   return p;
 }
 
 hashmap<string,string>
-tmscm_to_table_string_string (tmscm  p) {
+tmscm_to_table_string_string (tmscm p) {
   hashmap<string,string> t;
   while (!tmscm_is_null (p)) {
-    tmscm  n= tmscm_car (p);
+    tmscm n= tmscm_car (p);
     t (tmscm_to_string (tmscm_car (n)))= tmscm_to_string (tmscm_cdr (n));
     p= tmscm_cdr (p);
   }
@@ -694,6 +787,7 @@ typedef array<int> array_int;
 typedef array<string> array_string;
 typedef array<tree> array_tree;
 typedef array<url> array_url;
+typedef array<patch> array_patch;
 typedef array<path> array_path;
 typedef array<widget> array_widget;
 typedef array<double> array_double;
@@ -701,7 +795,7 @@ typedef array<array<double> > array_array_double;
 typedef array<array<array<double> > > array_array_array_double;
 
 static bool
-tmscm_is_array_int (tmscm  p) {
+tmscm_is_array_int (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else return tmscm_is_pair (p) &&
     tmscm_is_int (tmscm_car (p)) &&
@@ -714,13 +808,13 @@ TMSCM_ASSERT (tmscm_is_array_int (p), p, arg, rout)
 /* static */ tmscm 
 array_int_to_tmscm (array<int> a) {
   int i, n= N(a);
-  tmscm  p= tmscm_null ();
+  tmscm p= tmscm_null ();
   for (i=n-1; i>=0; i--) p= tmscm_cons (int_to_tmscm (a[i]), p);
   return p;
 }
 
 /* static */ array<int>
-tmscm_to_array_int (tmscm  p) {
+tmscm_to_array_int (tmscm p) {
   array<int> a;
   while (!tmscm_is_null (p)) {
     a << ((int) tmscm_to_int (tmscm_car (p)));
@@ -730,7 +824,7 @@ tmscm_to_array_int (tmscm  p) {
 }
 
 static bool
-tmscm_is_array_string (tmscm  p) {
+tmscm_is_array_string (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else return tmscm_is_pair (p) && 
     tmscm_is_string (tmscm_car (p)) &&
@@ -739,7 +833,7 @@ tmscm_is_array_string (tmscm  p) {
 
 
 static bool
-tmscm_is_array_double (tmscm  p) {
+tmscm_is_array_double (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else return tmscm_is_pair (p) &&
     tmscm_is_double (tmscm_car (p)) &&
@@ -752,13 +846,13 @@ TMSCM_ASSERT (tmscm_is_array_double (p), p, arg, rout)
 /* static */ tmscm 
 array_double_to_tmscm (array<double> a) {
   int i, n= N(a);
-  tmscm  p= tmscm_null();
+  tmscm p= tmscm_null();
   for (i=n-1; i>=0; i--) p= tmscm_cons (double_to_tmscm (a[i]), p);
   return p;
 }
 
 /* static */ array<double>
-tmscm_to_array_double (tmscm  p) {
+tmscm_to_array_double (tmscm p) {
   array<double> a;
   while (!tmscm_is_null (p)) {
     a << ((double) tmscm_to_double (tmscm_car (p)));
@@ -768,7 +862,7 @@ tmscm_to_array_double (tmscm  p) {
 }
 
 static bool
-tmscm_is_array_array_double (tmscm   p) {
+tmscm_is_array_array_double (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else return tmscm_is_pair (p) &&
     tmscm_is_array_double (tmscm_car (p)) &&
@@ -781,13 +875,13 @@ TMSCM_ASSERT (tmscm_is_array_array_double (p), p, arg, rout)
 /* static */ tmscm 
 array_array_double_to_tmscm (array<array_double> a) {
   int i, n= N(a);
-  tmscm  p= tmscm_null ();
+  tmscm p= tmscm_null ();
   for (i=n-1; i>=0; i--) p= tmscm_cons (array_double_to_tmscm (a[i]), p);
   return p;
 }
 
 /* static */ array<array_double>
-tmscm_to_array_array_double (tmscm  p) {
+tmscm_to_array_array_double (tmscm p) {
   array<array_double> a;
   while (!tmscm_is_null (p)) {
     a << ((array_double) tmscm_to_array_double (tmscm_car (p)));
@@ -797,7 +891,7 @@ tmscm_to_array_array_double (tmscm  p) {
 }
 
 static bool
-tmscm_is_array_array_array_double (tmscm  p) {
+tmscm_is_array_array_array_double (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else return tmscm_is_pair (p) &&
     tmscm_is_array_array_double (tmscm_car (p)) &&
@@ -810,13 +904,13 @@ TMSCM_ASSERT (tmscm_is_array_array_array_double (p), p, arg, rout)
 /* static */ tmscm 
 array_array_array_double_to_tmscm (array<array_array_double> a) {
   int i, n= N(a);
-  tmscm  p= tmscm_null ();
+  tmscm p= tmscm_null ();
   for (i=n-1; i>=0; i--) p= tmscm_cons (array_array_double_to_tmscm (a[i]), p);
   return p;
 }
 
 /* static */ array<array_array_double>
-tmscm_to_array_array_array_double (tmscm  p) {
+tmscm_to_array_array_array_double (tmscm p) {
   array<array_array_double> a;
   while (!tmscm_is_null (p)) {
     a << ((array_array_double) tmscm_to_array_array_double (tmscm_car (p)));
@@ -836,13 +930,13 @@ TMSCM_ASSERT (tmscm_is_array_string (p), p, arg, rout)
 /* static */ tmscm 
 array_string_to_tmscm (array<string> a) {
   int i, n= N(a);
-  tmscm  p= tmscm_null ();
+  tmscm p= tmscm_null ();
   for (i=n-1; i>=0; i--) p= tmscm_cons (string_to_tmscm (a[i]), p);
   return p;
 }
 
 /* static */ array<string>
-tmscm_to_array_string (tmscm  p) {
+tmscm_to_array_string (tmscm p) {
   array<string> a;
   while (!tmscm_is_null (p)) {
     a << tmscm_to_string (tmscm_car (p));
@@ -852,7 +946,7 @@ tmscm_to_array_string (tmscm  p) {
 }
 
 static bool
-tmscm_is_array_tree (tmscm  p) {
+tmscm_is_array_tree (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else return tmscm_is_pair (p) && 
     tmscm_is_tree (tmscm_car (p)) &&
@@ -865,13 +959,13 @@ TMSCM_ASSERT (tmscm_is_array_tree (p), p, arg, rout)
 /* static */ tmscm 
 array_tree_to_tmscm (array<tree> a) {
   int i, n= N(a);
-  tmscm  p= tmscm_null ();
+  tmscm p= tmscm_null ();
   for (i=n-1; i>=0; i--) p= tmscm_cons (tree_to_tmscm (a[i]), p);
   return p;
 }
 
 /* static */ array<tree>
-tmscm_to_array_tree (tmscm  p) {
+tmscm_to_array_tree (tmscm p) {
   array<tree> a;
   while (!tmscm_is_null (p)) {
     a << tmscm_to_tree (tmscm_car (p));
@@ -881,7 +975,7 @@ tmscm_to_array_tree (tmscm  p) {
 }
 
 static bool
-tmscm_is_array_widget (tmscm  p) {
+tmscm_is_array_widget (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else return tmscm_is_pair (p) &&
     tmscm_is_widget (tmscm_car (p)) &&
@@ -894,13 +988,13 @@ TMSCM_ASSERT (tmscm_is_array_widget (p), p, arg, rout)
 /* static */ tmscm 
 array_widget_to_tmscm (array<widget> a) {
   int i, n= N(a);
-  tmscm  p= tmscm_null ();
+  tmscm p= tmscm_null ();
   for (i=n-1; i>=0; i--) p= tmscm_cons (widget_to_tmscm (a[i]), p);
   return p;
 }
 
 /* static */ array<widget>
-tmscm_to_array_widget (tmscm  p) {
+tmscm_to_array_widget (tmscm p) {
   array<widget> a;
   while (!tmscm_is_null (p)) {
     a << tmscm_to_widget (tmscm_car (p));
@@ -910,7 +1004,7 @@ tmscm_to_array_widget (tmscm  p) {
 }
 
 static bool
-tmscm_is_array_url (tmscm  p) {
+tmscm_is_array_url (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else return tmscm_is_pair (p) &&
     tmscm_is_url (tmscm_car (p)) &&
@@ -924,13 +1018,13 @@ TMSCM_ASSERT (tmscm_is_array_url (p), p, arg, rout)
 /* static */ tmscm 
 array_url_to_tmscm (array<url> a) {
   int i, n= N(a);
-  tmscm  p= tmscm_null ();
+  tmscm p= tmscm_null ();
   for (i=n-1; i>=0; i--) p= tmscm_cons (url_to_tmscm (a[i]), p);
   return p;
 }
 
 /* static */ array<url>
-tmscm_to_array_url (tmscm  p) {
+tmscm_to_array_url (tmscm p) {
   array<url> a;
   while (!tmscm_is_null (p)) {
     a << tmscm_to_url (tmscm_car (p));
@@ -940,7 +1034,37 @@ tmscm_to_array_url (tmscm  p) {
 }
 
 static bool
-tmscm_is_array_path (tmscm  p) {
+tmscm_is_array_patch (tmscm p) {
+  if (tmscm_is_null (p)) return true;
+  else return tmscm_is_pair (p) &&
+    tmscm_is_patch (tmscm_car (p)) &&
+    tmscm_is_array_patch (tmscm_cdr (p));
+}
+
+
+#define TMSCM_ASSERT_ARRAY_PATCH(p,arg,rout) \
+TMSCM_ASSERT (tmscm_is_array_patch (p), p, arg, rout)
+
+/* static */ tmscm 
+array_patch_to_tmscm (array<patch> a) {
+  int i, n= N(a);
+  tmscm p= tmscm_null ();
+  for (i=n-1; i>=0; i--) p= tmscm_cons (patch_to_tmscm (a[i]), p);
+  return p;
+}
+
+/* static */ array<patch>
+tmscm_to_array_patch (tmscm p) {
+  array<patch> a;
+  while (!tmscm_is_null (p)) {
+    a << tmscm_to_patch (tmscm_car (p));
+    p= tmscm_cdr (p);
+  }
+  return a;
+}
+
+static bool
+tmscm_is_array_path (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else return tmscm_is_pair (p) &&
     tmscm_is_path (tmscm_car (p)) &&
@@ -953,13 +1077,13 @@ TMSCM_ASSERT (tmscm_is_array_path (p), p, arg, rout)
 /* static */ tmscm 
 array_path_to_tmscm (array<path> a) {
   int i, n= N(a);
-  tmscm  p= tmscm_null ();
+  tmscm p= tmscm_null ();
   for (i=n-1; i>=0; i--) p= tmscm_cons (path_to_tmscm (a[i]), p);
   return p;
 }
 
 /* static */ array<path>
-tmscm_to_array_path (tmscm  p) {
+tmscm_to_array_path (tmscm p) {
   array<path> a;
   while (!tmscm_is_null (p)) {
     a << tmscm_to_path (tmscm_car (p));
@@ -975,7 +1099,7 @@ tmscm_to_array_path (tmscm  p) {
 typedef list<string> list_string;
 
 bool
-tmscm_is_list_string (tmscm  p) {
+tmscm_is_list_string (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else return tmscm_is_pair (p) &&
     tmscm_is_string (tmscm_car (p)) &&
@@ -993,7 +1117,7 @@ list_string_to_tmscm (list_string l) {
 }
 
 list_string
-tmscm_to_list_string (tmscm  p) {
+tmscm_to_list_string (tmscm p) {
   if (tmscm_is_null (p)) return list_string ();
   return list_string (tmscm_to_string (tmscm_car (p)),
             tmscm_to_list_string (tmscm_cdr (p)));
@@ -1002,7 +1126,7 @@ tmscm_to_list_string (tmscm  p) {
 typedef list<tree> list_tree;
 
 bool
-tmscm_is_list_tree (tmscm  p) {
+tmscm_is_list_tree (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else return tmscm_is_pair (p) &&
     tmscm_is_tree (tmscm_car (p)) &&
@@ -1020,7 +1144,7 @@ list_tree_to_tmscm (list_tree l) {
 }
 
 list_tree
-tmscm_to_list_tree (tmscm  p) {
+tmscm_to_list_tree (tmscm p) {
   if (tmscm_is_null (p)) return list_tree ();
   return list_tree (tmscm_to_tree (tmscm_car (p)),
             tmscm_to_list_tree (tmscm_cdr (p)));
@@ -1069,11 +1193,13 @@ tmscm_to_list_tree (tmscm  p) {
 
 void
 initialize_glue () {
-  tmscm_install_procedure ("tree?",  treeP, 1, 0, 0);
-  tmscm_install_procedure ("tm?",  contentP, 1, 0, 0);
-  tmscm_install_procedure ("observer?",  observerP, 1, 0, 0);
-  tmscm_install_procedure ("url?",  urlP, 1, 0, 0);
-  tmscm_install_procedure ("blackbox?",  blackboxP, 1, 0, 0);
+  tmscm_install_procedure ("tree?", treeP, 1, 0, 0);
+  tmscm_install_procedure ("tm?", contentP, 1, 0, 0);
+  tmscm_install_procedure ("observer?", observerP, 1, 0, 0);
+  tmscm_install_procedure ("url?", urlP, 1, 0, 0);
+  tmscm_install_procedure ("modification?", modificationP, 1, 0, 0);
+  tmscm_install_procedure ("patch?", patchP, 1, 0, 0);
+  tmscm_install_procedure ("blackbox?", blackboxP, 1, 0, 0);
   
   initialize_glue_basic ();
   initialize_glue_editor ();
