@@ -31,6 +31,8 @@
 #include <X11/Xlib.h>
 #else
 #include <sys/stat.h>
+#include <sys/file.h>
+#include <unistd.h>
 #endif
 #include <sys/types.h>
 #include <string.h>  // strerror
@@ -73,6 +75,14 @@ load_string (url u, string& s, bool fatal) {
     FILE* fin= fopen (_name, "rb");
 #else
     FILE* fin= fopen (_name, "r");
+    int fd= -1;
+    if (fin != NULL) {
+      fd= fileno (fin);
+      if (flock (fd, LOCK_SH) == -1) {
+        fclose (fin);
+        fin= NULL;
+      }
+    }
 #endif
     if (fin == NULL) {
       err= true;
@@ -89,6 +99,10 @@ load_string (url u, string& s, bool fatal) {
       }
       if (err) {
         std_warning << "Seek failed for " << as_string (u) << "\n";
+#if defined (OS_WIN32) || defined (__MINGW__) || defined (__MINGW32__)
+#else
+        flock (fd, LOCK_UN);
+#endif
         fclose (fin);
       }
     }
@@ -97,6 +111,10 @@ load_string (url u, string& s, bool fatal) {
       s->resize (size);
       int read= fread (&(s[0]), 1, size, fin);
       if (read < size) s->resize (read);
+#if defined (OS_WIN32) || defined (__MINGW__) || defined (__MINGW32__)
+#else
+      flock (fd, LOCK_UN);
+#endif
       fclose (fin);
     }
     bench_cumul ("load file");
@@ -138,7 +156,18 @@ save_string (url u, string s, bool fatal) {
 #elif defined (__MINGW__) || defined (__MINGW32__)
       FILE* fout= fopen (_name, "wb");
 #else
-      FILE* fout= fopen (_name, "w");
+      FILE* fout= fopen (_name, "r+");
+      bool rw= (fout != NULL);
+      if (!rw) fout= fopen (_name, "w");
+      int fd= -1;
+      if (fout != NULL) {
+        fd= fileno (fout);
+        if (flock (fd, LOCK_EX) == -1) {
+          fclose (fout);
+          fout= NULL;
+        }
+        else if (rw) ftruncate (fd, 0);
+      }
 #endif
       if (fout == NULL) {
         err= true;
@@ -149,6 +178,10 @@ save_string (url u, string s, bool fatal) {
         int i, n= N(s);
         for (i=0; i<n; i++)
           fputc (s[i], fout);
+#if defined (OS_WIN32) || defined (__MINGW__) || defined (__MINGW32__)
+#else
+        flock (fd, LOCK_UN);
+#endif
         fclose (fout);
       }
     }
@@ -170,6 +203,61 @@ save_string (url u, string s, bool fatal) {
   return err;
 }
 
+bool
+append_string (url u, string s, bool fatal) {
+  if (is_rooted_tmfs (u)) FAILED ("file not appendable");
+
+  // cout << "Save " << u << LF;
+  url r= u;
+  if (!is_rooted_name (r)) r= resolve (r, "");
+  bool err= !is_rooted_name (r);
+  if (!err) {
+    string name= concretize (r);
+    {
+      c_string _name (name);
+#if defined (OS_WIN32)
+      FILE* fout= _fopen (_name, "ab");
+#elif defined (__MINGW__) || defined (__MINGW32__)
+      FILE* fout= fopen (_name, "ab");
+#else
+      FILE* fout= fopen (_name, "a");
+      int fd= -1;
+      if (fout != NULL) {
+        fd= fileno (fout);
+        if (flock (fd, LOCK_EX) == -1) {
+          fclose (fout);
+          fout= NULL;
+        }
+      }
+#endif
+      if (fout == NULL) {
+        err= true;
+        std_warning << "Append error for " << name << ", "
+                    << strerror(errno) << "\n";
+      }
+      if (!err) {
+        int i, n= N(s);
+        for (i=0; i<n; i++)
+          fputc (s[i], fout);
+#if defined (OS_WIN32) || defined (__MINGW__) || defined (__MINGW32__)
+#else
+        flock (fd, LOCK_UN);
+#endif
+        fclose (fout);
+      }
+    }
+    // Cache file contents
+    declare_out_of_date (url_parent (r));
+    // End caching
+  }
+
+  if (err && fatal) {
+    failed_error << "File name= " << as_string (u) << "\n";
+    FAILED ("file not appendable");
+  }
+  return err;
+}
+
 /******************************************************************************
 * Getting attributes of a file
 ******************************************************************************/
@@ -369,14 +457,14 @@ is_newer (url which, url than) {
 url
 url_temp (string suffix) {
 #ifdef __MINGW32__
-  int rnd= raw_time ();
+  unsigned int rnd= raw_time ();
 #else
   static bool initialized= false;
   if (!initialized) {
     srandom ((int) raw_time ());
     initialized= true;
   }
-  int rnd= random ();
+  unsigned int rnd= random ();
 #endif
   string name= "tmp_" * as_string (rnd) * suffix;
   url u= url_temp_dir () * url (name);
@@ -508,6 +596,14 @@ remove (url u) {
   remove_sub (expand (complete (u)));
 }
 
+void
+append_to (url what, url to) {
+  string what_s;
+  if (load_string (what, what_s, false) ||
+      append_string (to, what_s, false))
+    std_warning << "Append failed for " << to << LF;
+}
+
 void
 rmdir (url u) {
   remove_sub (expand (complete (u, "dr")));
@@ -516,8 +612,7 @@ rmdir (url u) {
 void
 mkdir (url u) {
 #if defined (HAVE_SYS_TYPES_H) && defined (HAVE_SYS_STAT_H)
-  if (exists (u)) return;
-  {
+  if (!exists (u)) {
     c_string _u (concretize (u));
 #if defined(__MINGW__) || defined(__MINGW32__)
     (void) ::mkdir (_u);
@@ -743,3 +838,106 @@ search_score (url u, array<string> a) {
   }
   return score;
 }
+
+/******************************************************************************
+* Finding recursive non hidden subdirectories of a given directory
+******************************************************************************/
+
+static void
+search_sub_dirs (url& all, url root) {
+  if (is_none (root));
+  else if (is_or (root)) {
+    search_sub_dirs (all, root[2]);
+    search_sub_dirs (all, root[1]);
+  }
+  else if (is_directory (root)) {
+    bool err= false;
+    array<string> a= read_directory (root, err);
+    if (!err) {
+      for (int i=N(a)-1; i>=0; i--)
+        if (N(a[i])>0 && a[i][0] != '.')
+          search_sub_dirs (all, root * a[i]);
+    }
+    all= root | all;
+  }
+}
+
+url
+search_sub_dirs (url root) {
+  url all= url_none ();
+  //cout << "Search in " << root << " -> " << expand (complete (root, "dr")) << LF;
+  search_sub_dirs (all, expand (complete (root, "dr")));
+  return all;
+}
+
+/******************************************************************************
+* Searching files in a directory tree with caching
+******************************************************************************/
+
+array<string> no_strings;
+hashmap<tree,int> dir_stamp (0);
+hashmap<tree,bool> dir_is_dir (false);
+hashmap<tree,array<string> > dir_contents (no_strings);
+
+array<string>
+var_read_directory (url u) {
+  array<string> d;
+  if (is_rooted (u, "default") || is_rooted (u, "file")) {
+    bool error_flag= false;
+    array<string> a= read_directory (u, error_flag);
+    for (int i=0; i<N(a); i++)
+      if (!starts (a[i], "."))
+        d << a[i];
+  }
+  return d;
+}
+
+url
+search_file_in (url u, string name) {
+  // cout << "Search in " << u << ", " << name << LF;
+  if (!dir_stamp->contains (u->t) ||
+      texmacs_time () - dir_stamp [u->t] > 10000) {
+    dir_is_dir->reset (u->t);
+    dir_contents->reset (u->t);
+  }
+  dir_stamp (u->t)= texmacs_time ();
+
+  if (!dir_is_dir->contains (u->t))
+    dir_is_dir (u->t)= is_directory (u);
+  if (!dir_is_dir [u->t]) {
+    if (as_string (tail (u)) == name) return u;
+    return url_none ();
+  }
+
+  if (!dir_contents->contains (u->t)) {
+    array<string> d= var_read_directory (u);
+    dir_contents (u->t)= d;
+  }
+
+  array<string> d= dir_contents [u->t];
+  for (int i=0; i<N(d); i++) {
+    url f= search_file_in (u * d[i], name);
+    if (!is_none (f)) return f;
+  }
+  return url_none ();
+}
+
+bool
+find_stop (url u, array<string> stops) {
+  if (head (u) == u) return false;
+  for (int i=0; i<N(stops); i++)
+    if (as_string (tail (u)) == stops[i]) return true;
+  return find_stop (head (u), stops);
+}
+
+url
+search_file_upwards (url u, string name, array<string> stops) {
+  // cout << "Search upwards " << u << ", " << name << LF;
+  url f= search_file_in (u, name);
+  if (!is_none (f)) return f;
+  if (head (u) == u) return url_none ();
+  if (!find_stop (head (u), stops)) return url_none ();
+  for (int i=0; i<N(stops); i++)
+    if (as_string (tail (u)) == stops[i]) return url_none ();
+  return search_file_upwards (head (u), name, stops);
+}
