@@ -17,6 +17,7 @@
 
 bool load_string (url file_name, string& s, bool fatal);
 bool save_string (url file_name, string s, bool fatal=false);
+bool append_string (url u, string s, bool fatal= false);
 
 bool is_of_type (url name, string filter);
 bool is_regular (url name);
@@ -53,6 +54,7 @@ inline string eval_system (string which, url u1, url u2) {
 void move (url from, url to);
 void copy (url from, url to);
 void remove (url what);
+void append_to (url what, url to);
 void mkdir (url dir);
 void rmdir (url what);
 void change_mode (url u, int mode);
@@ -60,8 +62,11 @@ void ps2pdf (url u1, url u2);
 
 int search_score (url u, array<string> a);
 
+url search_sub_dirs (url root);
 array<string> file_completions (url search, url dir);
 
 url grep (string what, url u);
+url search_file_in (url u, string name);
+url search_file_upwards (url u, string name, array<string> stops);
 
 #endif // defined FILE_H
