@@ -14,9 +14,12 @@
 
 extern text_property_rep tp_normal_rep;
 extern text_property_rep tp_hyph_rep;
+extern text_property_rep tp_thin_space_rep;
 extern text_property_rep tp_space_rep;
 extern text_property_rep tp_dspace_rep;
-extern text_property_rep tp_blank_rep;
+extern text_property_rep tp_nb_thin_space_rep;
+extern text_property_rep tp_nb_space_rep;
+extern text_property_rep tp_nb_dspace_rep;
 extern text_property_rep tp_period_rep;
 extern text_property_rep tp_operator_rep;
 extern text_property_rep tp_shortop_rep;
@@ -79,4 +82,13 @@ struct scilab_language_rep: language_rep {
   string get_color (tree t, int start, int end);
 };
 
+struct python_language_rep: language_rep {
+  hashmap<string,string> colored;
+  python_language_rep (string name);
+  text_property advance (tree t, int& pos);
+  array<int> get_hyphens (string s);
+  void hyphenate (string s, int after, string& left, string& right);
+  string get_color (tree t, int start, int end);
+};
+
 #endif // defined IMPL_LANGUAGE_H
