@@ -19,14 +19,20 @@ text_property_rep tp_normal_rep
   (TP_NORMAL);
 text_property_rep tp_hyph_rep
   (TP_HYPH, SPC_NONE, SPC_NONE, 0, 0);
+text_property_rep tp_thin_space_rep
+  (TP_THIN_SPACE, SPC_NONE, SPC_THIN_SPACE, 0, 0);
 text_property_rep tp_space_rep
-  (TP_SPACE, SPC_NONE, SPC_SPACE, 0,0);
+  (TP_SPACE, SPC_NONE, SPC_SPACE, 0, 0);
 text_property_rep tp_dspace_rep
-  (TP_DSPACE, SPC_NONE, SPC_DSPACE, 0,0);
+  (TP_DSPACE, SPC_NONE, SPC_DSPACE, 0, 0);
+text_property_rep tp_nb_thin_space_rep
+  (TP_NB_THIN_SPACE, SPC_NONE, SPC_THIN_SPACE, 0, HYPH_INVALID);
+text_property_rep tp_nb_space_rep
+  (TP_NB_SPACE, SPC_NONE, SPC_SPACE, 0, HYPH_INVALID);
+text_property_rep tp_nb_dspace_rep
+  (TP_NB_DSPACE, SPC_NONE, SPC_DSPACE, 0, HYPH_INVALID);
 text_property_rep tp_period_rep
   (TP_PERIOD, SPC_NONE, SPC_PERIOD, 0, 0);
-text_property_rep tp_blank_rep
-  (TP_BLANK, SPC_NONE, SPC_SPACE, 0, HYPH_INVALID);
 text_property_rep tp_half_rep
   (TP_OPERATOR, SPC_NONE, SPC_HALF, 0, HYPH_INVALID);
 text_property_rep tp_operator_rep
@@ -204,6 +210,7 @@ initialize_color_encodings () {
   language_rep::color_encoding ("constant_module")= 15;
   language_rep::color_encoding ("constant_number")= 16;
   language_rep::color_encoding ("constant_string")= 17;
+  language_rep::color_encoding ("constant_char")= 18;
   language_rep::color_encoding ("variable")= 20;
   language_rep::color_encoding ("variable_identifier")= 21;
   language_rep::color_encoding ("variable_function")= 22;
@@ -222,7 +229,7 @@ initialize_color_encodings () {
   language_rep::color_encoding ("operator_field")= 42;
   language_rep::color_encoding ("operator_special")= 43;
   language_rep::color_encoding ("keyword")= 50;
-  language_rep::color_encoding ("keyword_conditionnal")= 51;
+  language_rep::color_encoding ("keyword_conditional")= 51;
   language_rep::color_encoding ("keyword_control")= 52;
 }
 
@@ -241,8 +248,9 @@ initialize_color_decodings (string lan_name) {
   lan->color_decoding (13)= get_preference (pfx * "constant_type", "#4040c0");
   lan->color_decoding (14)= get_preference (pfx * "constant_category", "#4040c0");
   lan->color_decoding (15)= get_preference (pfx * "constant_module", "#4040c0");
-  lan->color_decoding (16)= get_preference (pfx * "constant_number", "#4040c0");
+  lan->color_decoding (16)= get_preference (pfx * "constant_number", "#3030b0");
   lan->color_decoding (17)= get_preference (pfx * "constant_string", "dark grey");
+  lan->color_decoding (18)= get_preference (pfx * "constant_char", "#333333");
   lan->color_decoding (20)= get_preference (pfx * "variable", "#606060");
   lan->color_decoding (21)= get_preference (pfx * "variable_identifier", "#204080");
   lan->color_decoding (22)= get_preference (pfx * "variable_function", "#606060");
@@ -261,7 +269,7 @@ initialize_color_decodings (string lan_name) {
   lan->color_decoding (42)= get_preference (pfx * "operator_field", "#888888");
   lan->color_decoding (43)= get_preference (pfx * "operator_special", "orange");
   lan->color_decoding (50)= get_preference (pfx * "keyword", "#309090");
-  lan->color_decoding (51)= get_preference (pfx * "keyword_conditionnal", "#309090");
+  lan->color_decoding (51)= get_preference (pfx * "keyword_conditional", "#309090");
   lan->color_decoding (52)= get_preference (pfx * "keyword_control", "#000080");
 }
 
