@@ -146,6 +146,10 @@ packrat_parser_rep::serialize (tree t, path p) {
     case NEW_DPAGE:
       break;
 
+    case WIDE:
+    case NEG:
+      serialize (t[0], p * 0);
+      break;
     case SYNTAX:
       serialize (t[1], path (-1));
       break;
@@ -178,6 +182,7 @@ packrat_parser_rep::serialize (tree t, path p) {
     case LINK:
     case URL:
     case SCRIPT:
+    case OBSERVER:
     case FIND_ACCESSIBLE:
       break;
     case HLINK:
@@ -189,6 +194,7 @@ packrat_parser_rep::serialize (tree t, path p) {
     case LABEL:
     case REFERENCE:
     case PAGEREF:
+    case GET_ATTACHMENT:
     case WRITE:
     case TOC_NOTIFY:
       break;
