@@ -54,7 +54,7 @@ text_language_rep::advance (tree t, int& pos) {
     // while ((pos<N(s)) && (s[pos]==' ')) pos++;
     if ((pos == N(s)) || (!is_punctuation (s[pos])))
       return &tp_space_rep;
-    return &tp_blank_rep;
+    return &tp_nb_space_rep;
   }
   
   if (is_punctuation (s[pos])) {
@@ -109,6 +109,104 @@ text_language_rep::hyphenate (
   std_hyphenate (s, after, left, right, penalty[after]);
 }
 
+/******************************************************************************
+* French typography
+******************************************************************************/
+
+struct french_language_rep: language_rep {
+  hashmap<string,string> patterns;
+  hashmap<string,string> hyphenations;
+
+  french_language_rep (string lan_name, string hyph_name);
+  text_property advance (tree t, int& pos);
+  array<int> get_hyphens (string s);
+  void hyphenate (string s, int after, string& left, string& right);
+};
+
+french_language_rep::french_language_rep (string lan_name, string hyph_name):
+  language_rep (lan_name), patterns ("?"), hyphenations ("?") {
+    load_hyphen_tables (hyph_name, patterns, hyphenations, true); }
+
+inline bool
+is_french_punctuation (register char c) {
+  return is_punctuation (c) || (c=='\23') || (c=='\24');
+}
+
+text_property
+french_language_rep::advance (tree t, int& pos) {
+  string s= t->label;
+  if (pos == N(s)) return &tp_normal_rep;
+
+  if (s[pos]==' ') {
+    pos++;
+    if (pos>1 && s[pos-2] == '\23')
+      return &tp_nb_thin_space_rep;
+    // while ((pos<N(s)) && (s[pos]==' ')) pos++;
+    if ((pos == N(s)) || (!is_french_punctuation (s[pos])))
+      return &tp_space_rep;
+    if (s[pos] == '\23')
+      return &tp_space_rep;
+    if (s[pos] == ':' || s[pos] == ';' ||
+        s[pos] == '!' || s[pos] == '?' || s[pos] == '\24')
+      return &tp_nb_thin_space_rep;
+    return &tp_nb_space_rep;
+  }
+  
+  if (is_french_punctuation (s[pos])) {
+    while ((pos<N(s)) && is_french_punctuation (s[pos])) pos++;
+    if ((pos==N(s)) || (s[pos]!=' ')) return &tp_normal_rep;
+    switch (s[pos-1]) {
+    case '\23':
+      return &tp_nb_thin_space_rep;
+    case '\24':
+    case ',': case ':': case ';': case '`': case '\'':
+      return &tp_space_rep;
+    case '.': case '!': case '?':
+      return &tp_period_rep;
+    }
+    return &tp_space_rep;
+  }
+
+  if (s[pos]=='-') {
+    pos++;
+    while ((pos<N(s)) && (s[pos]=='-')) pos++;
+    return &tp_hyph_rep;
+  }
+
+  if (is_iso_alpha (s[pos])) {
+    while ((pos<N(s)) && is_iso_alpha (s[pos])) pos++;
+    return &tp_normal_rep;
+  }
+
+  if (is_numeric (s[pos])) { // can not be a '.'
+    while ((pos<N(s)) && is_numeric (s[pos])) pos++;
+    while (s[pos-1]=='.') pos--;
+    return &tp_normal_rep;
+  }
+
+  if (s[pos]=='<') {
+    while ((pos<N(s)) && (s[pos]!='>')) pos++;
+    if (pos<N(s)) pos++;
+    return &tp_normal_rep;
+  }
+
+  pos++;
+  return &tp_normal_rep;
+}
+
+array<int>
+french_language_rep::get_hyphens (string s) {
+  return ::get_hyphens (s, patterns, hyphenations);
+}
+
+void
+french_language_rep::hyphenate (
+  string s, int after, string& left, string& right)
+{
+  array<int> penalty= get_hyphens (s);
+  std_hyphenate (s, after, left, right, penalty[after]);
+}
+
 /******************************************************************************
 * Eastern text languages / using UCS entities
 ******************************************************************************/
@@ -140,7 +238,7 @@ ucs_text_language_rep::advance (tree t, int& pos) {
     // while ((pos<N(s)) && (s[pos]==' ')) pos++;
     if ((pos == N(s)) || (!is_punctuation (s[pos])))
       return &tp_space_rep;
-    return &tp_blank_rep;
+    return &tp_nb_space_rep;
   }
   
   if (is_punctuation (s[pos])) {
@@ -319,6 +417,7 @@ windows_locale_to_language (string s) {
     return "taiwanese"; // for windows xp
   if (s == "Chinese (Traditional)_Taiwan.950")
     return "taiwanese"; // for windows 7
+  if (s == "Croatian_Croatia.1250") return "croatian";
   if (s == "Czech_Czech Republic.1250") return "czech";
   if (s == "Danish_Denmark.1252") return "danish";
   if (s == "Dutch_Netherlands.1252") return "dutch";
@@ -327,6 +426,7 @@ windows_locale_to_language (string s) {
   if (s == "Finnish_Finland.1252") return "finnish";
   if (s == "French_France.1252") return "french";
   if (s == "German_Germany.1252") return "german";
+  if (s == "Greek_Greece.1253") return "greek";
   if (s == "Hungarian_Hungary.1250") return "hungarian";
   if (s == "Italian_Italy.1252") return "italian";
   if (s == "Japanese_Japan.932") return "japanese";
@@ -350,6 +450,7 @@ locale_to_language (string s) {
   if (N(s) > 2) s= s (0, 2);
   if (s == "bg") return "bulgarian";
   if (s == "zh") return "chinese";
+  if (s == "hr") return "croatian";
   if (s == "cs") return "czech";
   if (s == "da") return "danish";
   if (s == "nl") return "dutch";
@@ -357,6 +458,7 @@ locale_to_language (string s) {
   if (s == "fi") return "finnish";
   if (s == "fr") return "french";
   if (s == "de") return "german";
+  if (s == "gr") return "greek";
   if (s == "hu") return "hungarian";
   if (s == "it") return "italian";
   if (s == "ja") return "japanese";
@@ -378,6 +480,7 @@ language_to_locale (string s) {
   if (s == "british")    return "en_GB";
   if (s == "bulgarian")  return "bg_BG";
   if (s == "chinese")    return "zh_CN";
+  if (s == "croatian")   return "hr_HR";
   if (s == "czech")      return "cs_CZ";
   if (s == "danish")     return "da_DK";
   if (s == "dutch")      return "nl_NL";
@@ -385,6 +488,7 @@ language_to_locale (string s) {
   if (s == "finnish")    return "fi_FI";
   if (s == "french")     return "fr_FR";
   if (s == "german")     return "de_DE";
+  if (s == "greek")      return "gr_GR";
   if (s == "hungarian")  return "hu_HU";
   if (s == "italian")    return "it_IT";
   if (s == "japanese")   return "ja_JP";
@@ -405,7 +509,9 @@ string
 language_to_local_ISO_charset (string s) {
   if (s == "bulgarian")  return "ISO-8859-5";
   if (s == "chinese")    return "";
+  if (s == "croatian")   return "ISO-8859-2";
   if (s == "czech")      return "ISO-8859-2";
+  if (s == "greek")      return "ISO-8859-7";
   if (s == "hungarian")  return "ISO-8859-2";
   if (s == "japanese")   return "";
   if (s == "korean")     return "";
@@ -453,6 +559,11 @@ string
 get_date (string lan, string fm) {
   return qt_get_date(lan, fm);
 }
+
+string
+pretty_time (int t) {
+  return qt_pretty_time (t);
+}
 #else
 
 static bool
@@ -511,6 +622,11 @@ get_date (string lan, string fm) {
   set_env (lvar, old);
   return date;
 }
+
+string
+pretty_time (int t) {
+  return var_eval_system ("date -r " * as_string (t));
+}
 #endif
 
 /******************************************************************************
@@ -529,6 +645,11 @@ make_text_language (string s, string h) {
   return tm_new<text_language_rep> (s, h);
 }
 
+static language
+make_french_language (string s, string h) {
+  return tm_new<french_language_rep> (s, h);
+}
+
 static language
 make_oriental_language (string s) {
   return tm_new<oriental_language_rep> (s);
@@ -541,13 +662,15 @@ text_language (string s) {
   if (s == "british")    return make_text_language (s, "ukenglish");
   if (s == "bulgarian")  return make_ucs_text_language (s, "bulgarian");
   if (s == "chinese")    return make_oriental_language (s);
+  if (s == "croatian")   return make_text_language (s, "croatian");
   if (s == "czech")      return make_text_language (s, "czech");
   if (s == "danish")     return make_text_language (s, "danish");
   if (s == "dutch")      return make_text_language (s, "dutch");
   if (s == "english")    return make_text_language (s, "us");
   if (s == "finnish")    return make_text_language (s, "finnish");
-  if (s == "french")     return make_text_language (s, "french");
+  if (s == "french")     return make_french_language (s, "french");
   if (s == "german")     return make_text_language (s, "german");
+  if (s == "greek")      return make_text_language (s, "greek");
   if (s == "hungarian")  return make_text_language (s, "hungarian");
   if (s == "italian")    return make_text_language (s, "italian");
   if (s == "japanese")   return make_oriental_language (s);
@@ -562,7 +685,7 @@ text_language (string s) {
   if (s == "taiwanese")  return make_oriental_language (s);
   if (s == "ukrainian")  return make_ucs_text_language (s, "ukrainian");
   if (s == "verbatim")   return tm_new<verb_language_rep> ("verbatim");
-  cerr << "\nThe language was " << s << "\n";
+  failed_error << "The language was " << s << "\n";
   FAILED ("unknown language");
   return tm_new<verb_language_rep> ("verbatim");
 }
