@@ -13,6 +13,7 @@
 #define SYS_UTILS_H
 #include "string.hpp"
 #include "url.hpp"
+#include "array.hpp"
 
 extern int script_status; // 0: never accept, 1: prompt, 2: always accept
 
@@ -27,4 +28,8 @@ string get_stacktrace (unsigned int max_frames= 127);
 url get_texmacs_path ();
 url get_texmacs_home_path ();
 
+array<string> evaluate_system (array<string> arg,
+			       array<int> fd_in, array<string> in,
+			       array<int> fd_out);
+
 #endif // defined SYS_UTILS_H
