@@ -27,6 +27,17 @@ string propose_title (string old_title, url u, tree doc);
 * Check for changes in the buffer
 ******************************************************************************/
 
+void
+tm_buffer_rep::attach_notifier () {
+  if (notify) return;
+  string id= as_string (buf->name, URL_UNIX);
+  tree& st (subtree (the_et, rp));
+  call ("buffer-initialize", id, st, buf->name);
+  lns= link_repository (true);
+  lns->insert_locus (id, st, "buffer-notify");
+  notify= true;
+}
+
 bool
 tm_buffer_rep::needs_to_be_saved () {
   if (buf->read_only) return false;
@@ -392,6 +403,13 @@ pretend_buffer_autosaved (url name) {
     view_to_editor (vs[i]) -> notify_save (false);
 }
 
+void
+attach_buffer_notifier (url name) {
+  tm_buffer buf= concrete_buffer (name);
+  if (is_nil (buf)) return;
+  buf->attach_notifier ();
+}
+
 /******************************************************************************
 * Loading
 ******************************************************************************/
@@ -401,6 +419,7 @@ attach_subformat (tree t, url u, string fm) {
   if (fm != "verbatim" && fm != "scheme" && fm != "cpp") return t;
   string s= suffix (u);
   if (s == "scm") fm= "scheme";
+  if (s == "py")  fm= "python";
   if (s == "cpp" || s == "hpp" || s == "cc" || s == "hh") fm= "cpp";
   if (s == "mmx" || s == "mmh") fm= "mathemagix";
   if (s == "sce" || s == "sci") fm= "scilab";
@@ -476,6 +495,14 @@ load_style_tree (string package) {
 
 bool
 export_tree (tree doc, url u, string fm) {
+  // NOTE: hook for encryption
+  tree init= extract (doc, "initial");
+  if (fm == "texmacs")
+    for (int i=0; i<N(init); i++)
+      if (is_func (init[i], ASSOCIATE, 2) && init[i][0] == "encryption")
+	doc= as_tree (call ("tree-export-encrypted",
+			    object (u), object (doc)));
+  // END hook
   if (fm == "generic") fm= "verbatim";
   string s= tree_to_generic (doc, fm * "-document");
   if (s == "* error: unknown format *") return true;
@@ -517,6 +544,13 @@ buffer_export (url name, url dest, string fm) {
   return export_tree (doc, dest, fm);
 }
 
+tree
+latex_expand (tree doc, url name) {
+  tm_view vw= concrete_view (get_recent_view (name));
+  tree body= vw->ed->exec_latex (extract (doc, "body"));
+  return change_doc_attr (doc, "body", body);
+}
+
 tree
 latex_expand (tree doc) {
   tm_view vw= concrete_view (url (as_string (extract (doc, "view"))));
