@@ -53,6 +53,8 @@ extern tree the_et;
 extern bool texmacs_started;
 
 bool disable_error_recovery= false;
+bool start_server_flag= false;
+void server_start ();
 
 tm_ostream& cout= tm_ostream::cout;
 tm_ostream& cerr= tm_ostream::cerr;
@@ -273,13 +275,15 @@ TeXmacs_main (int argc, char** argv) {
 	i++;
 	if (i<argc) my_init_cmds= (my_init_cmds * " ") * argv[i];
       }
+      else if (s == "-server") start_server_flag= true;
       else if (s == "-log-file") i++;
       else if ((s == "-Oc") || (s == "-no-char-clipping")) char_clip= false;
       else if ((s == "+Oc") || (s == "-char-clipping")) char_clip= true;
       else if ((s == "-S") || (s == "-setup") ||
 	       (s == "-delete-cache") || (s == "-delete-font-cache") ||
 	       (s == "-delete-style-cache") || (s == "-delete-file-cache") ||
-	       (s == "-delete-doc-cache") || (s == "-delete-plugin-cache"));
+	       (s == "-delete-doc-cache") || (s == "-delete-plugin-cache") ||
+	       (s == "-delete-server-data") || (s == "-delete-databases"));
       else if (starts (s, "-psn"));
       else {
 	cout << "\n";
@@ -381,6 +385,7 @@ TeXmacs_main (int argc, char** argv) {
   if (DEBUG_STD) debug_boot << "Starting event loop...\n";
   texmacs_started= true;
   if (!disable_error_recovery) signal (SIGSEGV, clean_exit_on_segfault);
+  if (start_server_flag) server_start ();
   gui_start_loop ();
 
   if (DEBUG_STD) debug_boot << "Stopping server...\n";
@@ -419,6 +424,19 @@ boot_hacks () {
   //getrlimit (RLIMIT_NOFILE, &lims);
   //printf ("cur: %i\n", lims.rlim_cur);
   //printf ("max: %i\n", lims.rlim_max);
+#if defined(MAC_OS_X_VERSION_10_10)
+  mac_fix_yosemite_bug();
+#endif
+
+#ifdef QTTEXMACS
+#if defined(MAC_OS_X_VERSION_10_9) || defined(MAC_OS_X_VERSION_10_10)
+#if QT_VERSION <= QT_VERSION_CHECK(4,8,5)
+  // Work around Qt bug: https://bugreports.qt-project.org/browse/QTBUG-32789
+  QFont::insertSubstitution (".Lucida Grande UI", "Lucida Grande");
+#endif
+#endif
+#endif
+
 #endif
 }
 
@@ -471,8 +489,13 @@ immediate_options (int argc, char** argv) {
       remove (url ("$TEXMACS_HOME_PATH/system/cache/dir_cache.scm"));
       remove (url ("$TEXMACS_HOME_PATH/system/cache/stat_cache.scm"));
     }
-    else if (s == "-delete-plugin-cache") {
+    else if (s == "-delete-plugin-cache")
       remove (url ("$TEXMACS_HOME_PATH/system/cache/plugin_cache.scm"));
+    else if (s == "-delete-server-data")
+      system ("rm -rf", url ("$TEXMACS_HOME_PATH/server"));
+    else if (s == "-delete-databases") {
+      system ("rm -rf", url ("$TEXMACS_HOME_PATH/system/database"));
+      system ("rm -rf", url ("$TEXMACS_HOME_PATH/users"));
     }
     else if (s == "-log-file" && i + 1 < argc) {
       i++;
@@ -506,10 +529,6 @@ main (int argc, char** argv) {
 #endif
 #ifdef QTTEXMACS
   // initialize the Qt application infrastructure
-#if defined(MAC_OS_X_VERSION_10_9) && (QT_VERSION <= QT_VERSION_CHECK(4,8,5))
-    // Work around Qt bug: https://bugreports.qt-project.org/browse/QTBUG-32789
-  QFont::insertSubstitution (".Lucida Grande UI", "Lucida Grande");
-#endif
   new QTMApplication (argc, argv);
 #endif
   TeXmacs_init_paths (argc, argv);
