@@ -200,7 +200,14 @@ public:
 void
 close_embedded_command_rep::apply () {
   //cout << "Destroy " << vw->buf->buf->name << "\n";
-  window_focus (abstract_window (vw->ed->mvw->win));
+  ASSERT (!is_nil(vw->ed), "embedded command acting on deleted editor");
+  url foc= abstract_window (vw->ed->mvw->win);
+  if (is_none (foc)) {
+    array<url> a= windows_list ();
+    ASSERT (N(a) != 0, "no remaining windows");
+    foc= a[0];
+  }
+  window_focus (foc);
   //cout << "Changed focus\n";
   tm_window win= vw->win;
   ASSERT (N (buffer_to_views (vw->buf->buf->name)) == 1,
@@ -256,7 +263,9 @@ texmacs_input_widget (tree doc, tree style, url wname) {
   doc= enrich_embedded_document (doc, style);
   url       base = get_master_buffer (get_current_buffer ());
   tm_view   curvw= concrete_view (get_current_view ());
-  url       name = embedded_name (wname); create_buffer (name, doc);
+  url       name = embedded_name (wname);
+  if (contains (name, get_all_buffers ())) set_buffer_tree (name, doc);
+  else create_buffer (name, doc);
   tm_view   vw   = concrete_view (get_passive_view (name));
   tm_window win  = tm_new<tm_window_rep> (doc, command ());
   set_master_buffer (name, base);
@@ -279,6 +288,11 @@ tm_window_rep::set_window_name (string s) {
   }
 }
 
+void
+tm_window_rep::set_modified (bool flag) {
+  ::set_modified (wid, flag);
+}
+
 void
 tm_window_rep::set_window_url (url u) {
   if (!is_none (u)) set_file (wid, as_string (u));
@@ -303,6 +317,12 @@ tm_window_rep::refresh () {
 * Menus
 ******************************************************************************/
 
+#ifdef QTTEXMACS
+bool menu_caching= false;
+#else
+bool menu_caching= true;
+#endif
+
 bool
 tm_window_rep::get_menu_widget (int which, string menu, widget& w) {
   object xmenu= call ("menu-expand", eval ("'" * menu));
@@ -319,7 +339,7 @@ tm_window_rep::get_menu_widget (int which, string menu, widget& w) {
   //cout << "Compute " << menu << "\n";
   object umenu= eval ("'" * menu);
   w= make_menu_widget (umenu);
-  menu_cache (xmenu)= w;
+  if (menu_caching) menu_cache (xmenu)= w;
   return true;
 }
 
@@ -352,6 +372,15 @@ tm_window_rep::side_tools (int which, string tools) {
   }
 }
 
+void
+tm_window_rep::bottom_tools (int which, string tools) {
+  eval ("(lazy-initialize-force)");
+  widget w;
+  if (get_menu_widget (20 + which, tools, w)) {
+    if (which == 0) set_bottom_tools (wid, w);
+  }
+}
+
 void
 tm_window_rep::set_header_flag (bool flag) {
   set_header_visibility (wid, flag);
@@ -370,6 +399,11 @@ tm_window_rep::set_side_tools_flag (int which, bool flag) {
   if (which == 0) set_side_tools_visibility (wid, flag);
 }
 
+void
+tm_window_rep::set_bottom_tools_flag (int which, bool flag) {
+  if (which == 0) set_bottom_tools_visibility (wid, flag);
+}
+
 bool
 tm_window_rep::get_header_flag () {
   return get_header_visibility (wid);
@@ -390,6 +424,12 @@ tm_window_rep::get_side_tools_flag (int which) {
   else return false;
 }
 
+bool
+tm_window_rep::get_bottom_tools_flag (int which) {
+  if (which == 0) return get_bottom_tools_visibility (wid);
+  else return false;
+}
+
 /******************************************************************************
 * The canvas
 ******************************************************************************/
