@@ -28,6 +28,11 @@ SI
 italic_correction (box L, box R) {
   double slope_L= L->right_slope ();
   double slope_R= R->left_slope ();
+  //cout << L << ", " << slope_L << " | " << R << ", " << slope_R << LF;
+  if (L->get_type () == BIG_OP_BOX)
+    if (1.5 * slope_L >= slope_R && slope_L > 0)
+      if (R->get_type () != BIG_OP_BOX)
+        return L->right_correction () + R->left_correction ();
   if (slope_L == slope_R) return 0;
   if (slope_L * slope_R == 0.0)
     return L->right_correction () + R->left_correction ();
@@ -308,15 +313,40 @@ tree_box_rep::find_child (SI x, SI y, SI delta, bool force) {
 * Computation of wide accent
 ******************************************************************************/
 
-void
+bool
 compute_wide_accent (path ip, box b, string s,
                      font fn, pencil pen, bool request_wide, bool above,
                      box& wideb, SI& sep) {
   bool wide= (b->w() >= (fn->wfn)) || request_wide;
   if (ends (s, "dot>") || (s == "<acute>") ||
       (s == "<grave>") || (s == "<abovering>")) wide= false;
+  bool stix= starts (fn->res_name, "stix-");
+  if (wide && stix) {
+    if (s == "^") s= "<hat>";
+    if (s == "~") s= "<tilde>";
+    if (s == "<hat>" || s == "<tilde>" || s == "<check>" ||
+        ends (s, "brace>") || ends (s, "brace*>")) {
+      font wfn= rubber_font (fn);
+      SI width= b->x2- b->x1 - fn->wfn/2;
+      wideb= wide_stix_box (decorate_middle (ip),
+                            "<rubber-" * s (1, N(s)-1) * ">",
+                            wfn, pen, width);
+      if (wideb->w() >= width) {
+        if (b->right_slope () != 0)
+          wideb= shift_box (decorate_middle (ip), wideb,
+                            (SI) (-0.5 * b->right_slope () * fn->yx), 0);
+        sep= above? -fn->yx: fn->sep;
+        if (above) {
+          if (s == "<overbrace>" || s == "<squnderbrace*>") sep= 2 * fn->sep;
+          if (s == "<poverbrace>") sep= 3 * fn->sep;
+        }
+        return wide;
+      }
+    }
+  }
   if (wide) {
     SI w= fn->wline;
+    if (stix) w= (SI) (1.189 * w);
     pencil wpen= pen->set_width (w);
     if ((s == "^") || (s == "<hat>"))
       wideb= wide_hat_box   (decorate_middle (ip), b->x1, b->x2, wpen);
@@ -328,9 +358,9 @@ compute_wide_accent (path ip, box b, string s,
       wideb= wide_vect_box  (decorate_middle (ip), b->x1, b->x2, wpen);
     else if (s == "<check>")
       wideb= wide_check_box (decorate_middle (ip), b->x1, b->x2, wpen);
-    else if (s == "<breve>")
+    else if (s == "<breve>" || s == "<punderbrace>" || s == "<punderbrace*>")
       wideb= wide_breve_box (decorate_middle (ip), b->x1, b->x2, wpen);
-    else if (s == "<invbreve>")
+    else if (s == "<invbreve>" || s == "<poverbrace>" || s == "<poverbrace*>")
       wideb= wide_invbreve_box(decorate_middle (ip), b->x1, b->x2, wpen);
     else if (s == "<squnderbrace>" || s == "<squnderbrace*>")
       wideb= wide_squbr_box (decorate_middle (ip), b->x1, b->x2, wpen);
@@ -340,14 +370,18 @@ compute_wide_accent (path ip, box b, string s,
                           "<rubber-" * s (1, N(s)-1) * ">",
                           fn, pen, b->x2- b->x1);
     sep= fn->sep;
+    if (stix) sep= (SI) (1.5 * sep);
   }
   else {
     wideb= text_box (decorate_middle (ip), 0, s, fn, pen);
-    if (fn->type == FONT_TYPE_UNICODE && b->right_slope () != 0)
+    if (fn->type == FONT_TYPE_UNICODE && b->right_slope () != 0) {
+      double factor= (stix? 0.2: 0.5);
       wideb= shift_box (decorate_middle (ip), wideb,
-                        (SI) (-0.5 * b->right_slope () * fn->yx), 0);
+                        (SI) (-factor * b->right_slope () * fn->yx), 0);
+    }
     sep= above? -fn->yx: fn->sep;
   }
+  return wide;
 }
 
 /******************************************************************************
@@ -360,6 +394,7 @@ struct wide_box_rep: public composite_box_rep {
   font   fn;
   pencil pen;
   bool   request_wide;
+  bool   wide;
   bool   above;
   SI     sep;
   SI     dw, dh, dd;
@@ -410,6 +445,7 @@ struct wide_box_rep: public composite_box_rep {
     if (above)
       return min (ref->sup_hi_lim (level) + dh, box_rep::sup_hi_lim (level));
     return ref->sup_hi_lim (level); }
+  void get_bracket_extents (SI& lo, SI& hi);
 };
 
 wide_box_rep::wide_box_rep (
@@ -419,7 +455,7 @@ wide_box_rep::wide_box_rep (
     request_wide (request_wide2), above (above2)
 {
   box hi;
-  compute_wide_accent (ip, ref, s, fn, pen, request_wide, above, hi, sep);
+  wide= compute_wide_accent (ip, ref, s, fn, pen, request_wide, above, hi, sep);
   SI X, Y, dx;
   SI hw= max (ref->w(), hi->w()) >> 1;
   SI m = (ref->x1 + ref->x2) >> 1;
@@ -473,6 +509,15 @@ wide_box_rep::right_slope () {
   return ref->right_slope ();
 }
 
+void
+wide_box_rep::get_bracket_extents (SI& lo, SI& hi) {
+  if (wide) box_rep::get_bracket_extents (lo, hi);
+  else {
+    lo= ref->y1;
+    hi= ref->y2;
+  }
+}
+
 /******************************************************************************
 * box construction routines
 ******************************************************************************/
