@@ -308,6 +308,7 @@ struct side_box_rep: public composite_box_rep {
     return nr_right==0? bs[0]->rsub_correction (): right_correction (); }
   SI rsup_correction () {
     return nr_right==0? bs[0]->rsup_correction (): right_correction (); }
+  void get_bracket_extents (SI& lo, SI& hi);
 };
 
 side_box_rep::side_box_rep (
@@ -452,7 +453,8 @@ side_box_rep::adjust_kerning (int mode, double factor) {
 int
 side_box_rep::find_child (SI x, SI y, SI delta, bool force) {
   if (outside (x, delta, x1, x2)) {
-    int side= box_rep::find_box_path (x, y, delta, force)->item;
+    bool found;
+    int side= box_rep::find_box_path (x, y, delta, force, found)->item;
     if (bs[0]->accessible () || force) {
       if ((side == 0) && (nr_left == 0)) return 0;
       if ((side == 1) && (nr_right == 0)) return 0;
@@ -605,6 +607,20 @@ side_box_rep::find_selection (path lbp, path rbp) {
   return composite_box_rep::find_selection (lbp, rbp);
 }
 
+
+void
+side_box_rep::get_bracket_extents (SI& lo, SI& hi) {
+  int i;
+  SI ex= fn->yx;
+  SI dd= ex / 4;
+  lo= sy1 (0);
+  hi= sy2 (0);
+  for (i=1; i<N(bs); i++) {
+    lo= min (lo, sy1 (i) + dd);
+    hi= max (hi, sy2 (i) - dd);
+  }
+}
+
 /******************************************************************************
 * box construction routines
 ******************************************************************************/
