@@ -107,7 +107,7 @@ build_locus (edit_env env, tree t, list<string>& ids, string& col, string &ref,
 	ids= list<string> (id, ids);
 	visited= visited || has_been_visited ("id:" * id);
       }
-      if (is_compound (arg, "link") && N(arg) >= 2) {
+      else if (is_compound (arg, "link") && N(arg) >= 2) {
 	if (is_func (arg[1], ATTR)) arg= copy (arg);
 	else arg= arg (0, 1) * tree (LINK, tree (ATTR)) * arg (1, N(arg));
 	arg[1] << tree ("secure")
@@ -124,6 +124,17 @@ build_locus (edit_env env, tree t, list<string>& ids, string& col, string &ref,
 	  }
 	}
       }
+      else if (is_compound (arg, "observer", 2)) {
+	string id= as_string (arg[0]);
+	string cb= cork_to_utf8 (as_string (arg[1]));
+	if (accessible) {
+          if (env->secure ||
+              as_bool (eval ("(secure? '(" * cb * " #f #f #f))")))
+            env->link_env->insert_locus (id, body, cb);
+        }
+	ids= list<string> (id, ids);
+	visited= visited || has_been_visited ("id:" * id);
+      }
     }
   }
 
@@ -229,8 +240,8 @@ concater_rep::typeset_write (tree t, path ip) {
 void
 concater_rep::typeset_toc_notify (tree t, path ip) {
   if (N(t) != 2) { typeset_error (t, ip); return; }
-  string kind = tree_to_verbatim (env->exec (t[0]));
-  string title= tree_to_verbatim (env->exec (t[1]));
+  string kind = tree_to_verbatim (env->exec (t[0]), false, "cork");
+  string title= tree_to_verbatim (env->exec (t[1]), false, "cork");
   title= replace (title, "T_EX_MACS", "TeXmacs");
   title= replace (title, "L^AT_EX", "LaTeX");
   title= replace (title, "T_EX", "TeX");
