@@ -49,6 +49,9 @@ concater_rep::typeset_colored_substring
   switch (spc_type) { \
   case SPC_NONE: \
     break; \
+  case SPC_THIN_SPACE: \
+    print ((6 * spc) / 10); \
+    break; \
   case SPC_SPACE: \
     print (spc); \
     break; \
@@ -68,6 +71,7 @@ concater_rep::typeset_colored_substring
     print (space (spc->min>>1, spc->def>>1, spc->max)); \
     break; \
   case SPC_BIGOP: \
+    print (spc); \
     break; \
   case SPC_CJK_NORMAL: \
     print (space (-(spc->min>>5), 0, spc->max>>5)); \
@@ -81,6 +85,9 @@ concater_rep::typeset_colored_substring
   switch (spc_type) { \
   case SPC_NONE: \
     break; \
+  case SPC_THIN_SPACE: \
+    print ((6 * spc) / 10); \
+    break; \
   case SPC_SPACE: \
     print (spc); \
     break; \
@@ -263,16 +270,13 @@ concater_rep::typeset_rigid (tree t, path ip) {
 }
 
 void
-concater_rep::typeset_syntax (tree t, path ip) {
-  if (N(t) != 2) { typeset_error (t, ip); return; }
-  box b= typeset_as_concat (env, t[0], descend (ip, 0));
-  b= move_box (ip, b, 0, 0, true);
-  if (is_atomic (t[1]) && tm_string_length (t[1]->label) == 1) {
+concater_rep::print_semantic (box b, tree sem) {
+  if (is_atomic (sem) && tm_string_length (sem->label) == 1) {
     space  spc= env->fn->spc;
     space  extra= env->fn->extra;
     bool   condensed= env->math_condensed;
     int    pos= 0;
-    string s= t[1]->label;
+    string s= sem->label;
     text_property tp= env->lan->advance (s, pos);
     int k= N(a);
     while (k > 0 && a[k-1]->op_type == OP_SKIP) k--;
@@ -293,7 +297,16 @@ concater_rep::typeset_syntax (tree t, path ip) {
       else PRINT_SPACE (tp->spc_after)
     }
   }
-  else print (b);
+  else print (b);  
+}
+
+void
+concater_rep::typeset_syntax (tree t, path ip) {
+  if (N(t) != 2) { typeset_error (t, ip); return; }
+  box b= typeset_as_concat (env, t[0], descend (ip, 0));
+  b= move_box (ip, b, 0, 0, true);
+  tree sem= env->exec (t[1]);
+  print_semantic (b, sem);
 }
 
 /******************************************************************************
