@@ -90,10 +90,16 @@ concater_rep::flag_ok (string s, path ip, color col) {
   a= a/6;
   color light= rgb_color (r, g, b, a);
   int info= env->info_level;
-  if (info == INFO_MINIMAL || info == INFO_SHORT) {
+  if (info == INFO_MINIMAL || info == INFO_SHORT || info == INFO_SHORT_PAPER) {
     box infob= info_box (dip, h, pencil (col, env->fn->wline), light);
-    box specb= specific_box (ip, infob, "screen", env->fn);
-    print (specb);
+    if (info == INFO_SHORT_PAPER) {
+      box b= resize_box (ip, infob, 0, 0, 0, env->fn->yx);
+      print (b);
+    }
+    else {
+      box specb= specific_box (ip, infob, "screen", env->fn);
+      print (specb);
+    }
   }
   else if (info == INFO_DETAILED || info == INFO_PAPER) {
     int sz= script (env->fn_size, env->index_level+2);
@@ -162,6 +168,7 @@ concater_rep::typeset (tree t, path ip) {
 
   if (!is_accessible (ip)) {
     path ip2= obtain_ip (t);
+    //if (ip2 != ip) cout << t << ", " << ip << " -> " << ip2 << "\n";
     if (ip2 != path (DETACHED))
       ip= ip2;
   }
@@ -494,9 +501,12 @@ concater_rep::typeset (tree t, path ip) {
   case EXTERN:
     typeset_rewrite (t, ip);
     break;
-  case INCLUDE:
+  case VAR_INCLUDE:
     typeset_include (t, ip);
     break;
+  case INCLUDE:
+    typeset_compound (t, ip);
+    break;
   case USE_PACKAGE:
   case USE_MODULE:
     typeset_executable (t, ip);
@@ -533,6 +543,7 @@ concater_rep::typeset (tree t, path ip) {
   case TRANSLATE:
   case CHANGE_CASE:
   case FIND_FILE:
+  case FIND_FILE_UPWARDS:
   case IS_TUPLE:
   case LOOK_UP:
   case EQUAL:
@@ -541,6 +552,7 @@ concater_rep::typeset (tree t, path ip) {
   case LESSEQ:
   case GREATER:
   case GREATEREQ:
+  case BLEND:
   case BOX_INFO:
   case FRAME_DIRECT:
   case FRAME_INVERSE:
@@ -612,6 +624,7 @@ concater_rep::typeset (tree t, path ip) {
   case LINK:
   case URL:
   case SCRIPT:
+  case OBSERVER:
   case FIND_ACCESSIBLE:
     typeset_inactive (t, ip);
     break;
@@ -630,6 +643,9 @@ concater_rep::typeset (tree t, path ip) {
   case PAGEREF:
     typeset_compound (t, ip);
     break;
+  case GET_ATTACHMENT:
+    typeset_executable (t, ip);
+    break;
   case WRITE:
     typeset_write (t, ip);
     break;
