@@ -51,6 +51,7 @@ initialize_default_env () {
   env (THE_LABEL)        = "?";         // value of the next label
   env (THE_TAGS)         = tree(TUPLE); // current tags
   env (THE_MODULES)      = tree(TUPLE); // necessary modules and plug-ins
+  env (WARN_MISSING)     = "true";      // warn about missing references
   env (GLOBAL_TITLE)     = "";          // global document title
   env (GLOBAL_AUTHOR)    = "";          // global document author
   env (GLOBAL_SUBJECT)   = "";          // global document subject
@@ -124,6 +125,7 @@ initialize_default_env () {
   env (PAGE_SCREEN_MARGIN) = "true";    // special margins for screen editing?
   env (PAGE_BREAKING)    = "professional";  // quality of page breaking
   env (PAGE_FLEXIBILITY) = "1";         // flexibility factor of stretch
+  env (PAGE_FIRST)       = "1";         // number of first page
   env (PAGE_NR)          = "0";         // the page number
   env (PAGE_THE_PAGE)    = the_page;    // the page number as text
   env (PAGE_WIDTH)       = "auto";      // physical width of pages
@@ -174,6 +176,7 @@ initialize_default_env () {
   env (TABLE_BBORDER)    = "0ln";       // bottom table border width
   env (TABLE_TBORDER)    = "0ln";       // top table border width
   env (TABLE_HYPHEN)     = "n";         // vertical hyphenation
+  env (TABLE_BLOCK)      = "no";        // consider table as block content
   env (TABLE_MIN_ROWS)   = "";          // suggested minimal number of rows
   env (TABLE_MIN_COLS)   = "";          // suggested minimal number of columns
   env (TABLE_MAX_ROWS)   = "";          // suggested maximal number of rows
@@ -191,10 +194,10 @@ initialize_default_env () {
   env (CELL_VMODE)       = "auto";      // height determination mode
   env (CELL_HALIGN)      = "l";         // horizontal alignment
   env (CELL_VALIGN)      = "B";         // vertical alignment
-  env (CELL_LSEP)        = "0fn";       // left cell padding
-  env (CELL_RSEP)        = "0fn";       // right cell padding
-  env (CELL_BSEP)        = "0fn";       // bottom cell padding
-  env (CELL_TSEP)        = "0fn";       // top cell padding
+  env (CELL_LSEP)        = "1spc";      // left cell padding
+  env (CELL_RSEP)        = "1spc";      // right cell padding
+  env (CELL_BSEP)        = "1sep";      // bottom cell padding
+  env (CELL_TSEP)        = "1sep";      // top cell padding
   env (CELL_LBORDER)     = "0ln";       // left cell border width
   env (CELL_RBORDER)     = "0ln";       // right cell border width
   env (CELL_BBORDER)     = "0ln";       // bottom cell border width
@@ -322,6 +325,8 @@ initialize_default_env () {
 			   tree (LOCUS, copy (ref_id), ln3, reftxt));
   env ("pageref")= tree (MACRO, "Id",
 			 tree (LOCUS, copy (ref_id), copy (ln3), preftxt));
+  env ("include")= tree (MACRO, "name",
+			 tree (VAR_INCLUDE, tree (ARG, "name")));
 
   /* further standard macros */
   env ("error")=
