@@ -85,19 +85,23 @@ edit_env_rep::rewrite (tree t) {
       if (N(t)>=5) end  = as_int (exec (t[4]));
       int i, n= max (0, end-start);
       tree r (make_tree_label (t[1]->label), n);
-      for (i=0; i<n; i++)
-	r[i]= tree (make_tree_label (t[0]->label),
-		    tree (ARG, copy (t[2]), as_string (start+i)),
-		    as_string (start+i));
-
+      if (t[0]->label == "identity")
+	for (i=0; i<n; i++)
+	  r[i]= tree (ARG, copy (t[2]), as_string (start+i));
+      else
+	for (i=0; i<n; i++)
+	  r[i]= tree (make_tree_label (t[0]->label),
+		      tree (ARG, copy (t[2]), as_string (start+i)),
+		      as_string (start+i));
       macro_arg= old_var;
       macro_src= old_src;
       return r;
     }
-  case INCLUDE:
+  case VAR_INCLUDE:
     {
       if (N(t) == 0) return tree (ERROR, "invalid include");
-      url file_name= url_system (exec_string (t[0]));
+      url file_name= url_unix (exec_string (t[0]));
+      //cout << "file_name= " << as_tree (file_name) << LF;
       return load_inclusion (relative (base_file_name, file_name));
     }
   case REWRITE_INACTIVE:
@@ -257,7 +261,7 @@ edit_env_rep::exec (tree t) {
     return exec_for_each (t);
   case EXTERN:
     return exec_rewrite (t);
-  case INCLUDE:
+  case VAR_INCLUDE:
     return exec_rewrite (t);
   case USE_PACKAGE:
     return exec_use_package (t);
@@ -315,6 +319,8 @@ edit_env_rep::exec (tree t) {
     return exec_change_case (t);
   case FIND_FILE:
     return exec_find_file (t);
+  case FIND_FILE_UPWARDS:
+    return exec_find_file_upwards (t);
   case IS_TUPLE:
     return exec_is_tuple (t);
   case LOOK_UP:
@@ -331,6 +337,8 @@ edit_env_rep::exec (tree t) {
     return exec_greater (t);
   case GREATEREQ:
     return exec_greatereq (t);
+  case BLEND:
+    return exec_blend (t);
 
   case CM_LENGTH:
     return exec_cm_length ();
@@ -418,6 +426,8 @@ edit_env_rep::exec (tree t) {
     return exec_hard_id (t);
   case SCRIPT:
     return exec_script (t);
+  case FIND_ACCESSIBLE:
+    return exec_find_accessible (t);
   case HLINK:
   case ACTION:
     return exec_compound (t);
@@ -425,6 +435,8 @@ edit_env_rep::exec (tree t) {
     return exec_set_binding (t);
   case GET_BINDING:
     return exec_get_binding (t);
+  case GET_ATTACHMENT:
+    return exec_get_attachment (t);
 
   case PATTERN:
     return exec_pattern (t);
@@ -500,7 +512,10 @@ edit_env_rep::exec_assign (tree t) {
   tree r= exec (t[0]);
   if (is_compound (r)) return tree (ERROR, "bad assign");
   assign (r->label, copy (t[1]));
-  return tree (ASSIGN, r, tree (QUOTE, read (r->label)));
+  tree v= read (r->label);
+  if (is_atomic (v) || is_func (v, MACRO));
+  else v= tree (QUOTE, v);
+  return tree (ASSIGN, r, v);
 }
 
 tree
@@ -949,8 +964,9 @@ edit_env_rep::exec_use_package (tree t) {
     //cout << "Package " << as_string (t[i]) << "\n";
     url name= url_none ();
     url styp= "$TEXMACS_STYLE_PATH";
-    //if (is_rooted_web (base_file_name)) styp= styp | head (base_file_name);
-    //else styp= ::expand (head (base_file_name) * url_ancestor ()) | styp;
+    if (is_rooted (base_file_name, "default"))
+      styp= styp | ::expand (head (base_file_name) * url_ancestor ());
+    else styp= styp | head (base_file_name);
     if (ends (as_string (t[i]), ".ts")) name= as_string (t[i]);
     else name= styp * (as_string (t[i]) * string (".ts"));
     name= resolve (name);
@@ -979,19 +995,17 @@ edit_env_rep::exec_use_module (tree t) {
 
 tree
 edit_env_rep::exec_or (tree t) {
-  if (N(t)<2) return tree (ERROR, "bad or");
+  if (N(t) < 2) return tree (ERROR, "bad or");
   for (int i=0; i<N(t); i++) {
     tree ti= exec (t[i]);
-    if (is_compound (ti)) return tree (ERROR, "bad or");
-    if (! is_bool (ti->label)) return tree (ERROR, "bad or");
-    if (as_bool (ti->label)) return as_string_bool (true);
+    if (ti != "false") return ti;
   }
   return as_string_bool (false);
 }
 
 tree
 edit_env_rep::exec_xor (tree t) {
-  if (N(t)!=2) return tree (ERROR, "bad xor");
+  if (N(t) != 2) return tree (ERROR, "bad xor");
   tree t1= exec (t[0]);
   tree t2= exec (t[1]);
   if (is_compound (t1) || is_compound (t2)) return tree (ERROR, "bad xor");
@@ -1002,23 +1016,20 @@ edit_env_rep::exec_xor (tree t) {
 
 tree
 edit_env_rep::exec_and (tree t) {
-  if (N(t)<2) return tree (ERROR, "bad and");
-  for (int i=0; i<N(t); i++) {
+  if (N(t) < 2) return tree (ERROR, "bad and");
+  for (int i=0; i<N(t)-1; i++) {
     tree ti= exec (t[i]);
-    if (is_compound (ti)) return tree (ERROR, "bad and");
-    if (! is_bool (ti->label)) return tree (ERROR, "bad and");
-    if (! as_bool (ti->label)) return as_string_bool (false);
+    if (ti == "false") return ti;
   }
-  return as_string_bool (true);
+  return exec (t[N(t)-1]);
 }
 
 tree
 edit_env_rep::exec_not (tree t) {
-  if (N(t)!=1) return tree (ERROR, "bad not");
-  tree tt= exec(t[0]);
-  if (is_compound (tt)) return tree (ERROR, "bad not");
-  if (! is_bool (tt->label)) return tree (ERROR, "bad not");
-  return as_string_bool (! as_bool (tt->label));
+  if (N(t) != 1) return tree (ERROR, "bad not");
+  tree tt= exec (t[0]);
+  if (tt == "false") return "true";
+  else return "false";
 }
 
 tree
@@ -1368,8 +1379,7 @@ edit_env_rep::exec_translate (tree t) {
   tree t1= exec (t[0]);
   tree t2= exec (t[1]);
   tree t3= exec (t[2]);
-  if (is_compound (t1) || is_compound (t2) || is_compound (t3))
-    return tree (ERROR, "bad translate");
+  if (is_compound (t1) || is_compound (t2) || is_compound (t3)) return t1;
   return translate (t1->label, t2->label, t3->label);
 }
 
@@ -1427,18 +1437,46 @@ edit_env_rep::exec_find_file (tree t) {
   for (i=0; i<(n-1); i++) {
     url u= resolve (url (r[i]->label, r[n-1]->label));
     if (!is_none (u)) {
+      url d= delta (base_file_name, u);
+      if (!is_rooted (d) && !(is_concat (d) && is_parent (d[1])))
+        return as_string (d);
       if (is_rooted (u, "default")) u= reroot (u, "file");
       return as_string (u);
     }
   }
   url u= resolve (base_file_name * url_parent () * r[n-1]->label);
   if (!is_none (u)) {
+    url d= delta (base_file_name, u);
+    if (!is_rooted (d) && !(is_concat (d) && is_parent (d[1])))
+      return as_string (d);
     if (is_rooted (u, "default")) u= reroot (u, "file");
     return as_string (u);
   }
   return "false";
 }
 
+tree
+edit_env_rep::exec_find_file_upwards (tree t) {
+  if (N(t) < 1) return tree (ERROR, "bad find file upwards");
+  tree name= exec (t[0]);
+  array<string> roots;
+  for (int i=1; i<N(t); i++) {
+    tree root= exec (t[i]);
+    if (!is_atomic (name) || !is_atomic (root))
+      return tree (ERROR, "bad find file upwards");
+    roots << root->label;
+  }
+  url u= search_file_upwards (base_file_name, name->label, roots);
+  if (!is_none (u)) {
+    url d= delta (base_file_name, u);
+    if (!is_rooted (d))
+      return as_string (d);
+    //if (is_rooted (u, "default")) u= reroot (u, "file");
+    return as_string (u);
+  }
+  return "false";
+}
+
 tree
 edit_env_rep::exec_is_tuple (tree t) {
   if (N(t)!=1) return tree (ERROR, "bad tuple query");
@@ -1542,6 +1580,23 @@ edit_env_rep::exec_greatereq (tree t) {
   return tree (ERROR, "bad greater or equal");
 }
 
+tree
+edit_env_rep::exec_blend (tree t) {
+  if (N(t)!=2) return tree (ERROR, "bad blend");
+  tree t1= exec (t[0]);
+  tree t2= exec (t[1]);
+  if (is_compound (t1) || is_compound (t2))
+    return tree (ERROR, "bad blend");
+  string s1= t1->label;
+  string s2= t2->label;
+  if (is_color_name (s1) && (is_color_name (s2))) {
+    color c1= named_color (s1);
+    color c2= named_color (s2);
+    return get_hex_color (blend_colors (c1, c2));
+  }
+  return tree (ERROR, "bad blend");
+}
+
 tree
 edit_env_rep::exec_hard_id (tree t) {
   pointer ptr= (pointer) this;
@@ -1564,9 +1619,19 @@ edit_env_rep::exec_hard_id (tree t) {
 
 tree
 edit_env_rep::exec_script (tree t) {
-  if (N(t) != 1 && N(t) != 2) return tree (ERROR, "bad script");
-  if (N(t) == 1) return tree (SCRIPT, exec (t[0]));
-  else return tree (SCRIPT, exec (t[0]), expand (t[1], true));
+  int i, n= N(t);
+  if (n < 1) return tree (ERROR, "bad script");
+  tree r (t, n);
+  r[0]= exec (t[0]);
+  for (i=1; i<n; i++)
+    r[i]= exec (t[i]);
+  return r;
+}
+
+tree
+edit_env_rep::exec_find_accessible (tree t) {
+  if (N(t) < 1) return tree (ERROR, "bad find-accessible");
+  return expand (t[0], true);
 }
 
 tree
@@ -1624,8 +1689,9 @@ edit_env_rep::exec_set_binding (tree t) {
       string old_s= tree_as_string (old_value[0]);
       string new_s= tree_as_string (value);
       if (new_s != old_s && !starts (key, "auto-")) {
-	if (new_s == "") system_warning ("Redefined", key);
-	else system_warning ("Redefined " * key * " as", new_s);
+        redefined << tree (TUPLE, key, new_s);
+	//if (new_s == "") typeset_warning << "Redefined " << key << LF;
+	//else typeset_warning << "Redefined " << key << " as " << new_s << LF;
       }
     }
   }
@@ -1642,11 +1708,22 @@ edit_env_rep::exec_get_binding (tree t) {
   if (is_func (value, TUPLE) && (N(value) >= 2)) value= value[type];
   else if (type == 1) value= tree (UNINIT);
   if (complete && value == tree (UNINIT))
-    system_warning ("Undefined reference", key);
+    if (get_bool (WARN_MISSING)) {
+      missing (key)= tree (GET_BINDING, key);
+      //typeset_warning << "Undefined reference " << key << LF;
+    }
   //cout << t << ": " << key << " -> " << value << "\n";
   return value;
 }
 
+tree
+edit_env_rep::exec_get_attachment (tree t) {
+  if (N(t) != 1) return tree (ERROR, "bad get attachment");
+  string key= exec_string (t[0]);
+  tree value= local_att->contains (key)? local_att [key]: global_att [key];
+  return value;
+}
+
 tree
 edit_env_rep::exec_pattern (tree t) {
   if (N(t)<1) return tree (ERROR, "bad pattern");
@@ -1689,8 +1766,8 @@ edit_env_rep::exec_pattern (tree t) {
     w= as_string (imw);
     h= as_string (imh);
   }
-  else if ((!is_int (w) && !is_percentage (w)) ||
-	   (!is_int (h) && !is_percentage (h)))
+  else if ((!is_int (w) && !is_percentage (w) && !is_percentage (w, "@")) ||
+	   (!is_int (h) && !is_percentage (h) && !is_percentage (h, "@")))
     return "white";
   tree r (PATTERN, as_string (image), w, h);
   if (N(t) == 4) r << exec (t[3]);
@@ -2038,7 +2115,7 @@ edit_env_rep::exec_until (tree t, path p, string var, int level) {
     (void) exec (t);
     return false;
   case EXTERN:
-  case INCLUDE:
+  case VAR_INCLUDE:
     return exec_until_rewrite (t, p, var, level);
   case USE_PACKAGE:
   case USE_MODULE:
@@ -2059,6 +2136,7 @@ edit_env_rep::exec_until (tree t, path p, string var, int level) {
   case _DATE:
   case TRANSLATE:
   case FIND_FILE:
+  case FIND_FILE_UPWARDS:
   case IS_TUPLE:
   case LOOK_UP:
   case EQUAL:
@@ -2067,6 +2145,7 @@ edit_env_rep::exec_until (tree t, path p, string var, int level) {
   case LESSEQ:
   case GREATER:
   case GREATEREQ:
+  case BLEND:
     (void) exec (t);
     return false;
   case STYLE_WITH:
@@ -2395,7 +2474,7 @@ edit_env_rep::depends (tree t, string s, int level) {
 	   is_func (t, EVAL_ARGS))
     {
       // FIXME: this does not handle more complex dependencies,
-      // like those encountered after rewritings (INCLUDE, EXTERN, etc.)
+      // like those encountered after rewritings (VAR_INCLUDE, EXTERN, etc.)
       tree v= (L(t) == MAP_ARGS? t[2]: t[0]);
       if (is_compound (v)) return false;
       if (!macro_arg->item->contains (v->label)) return false;
