@@ -12,6 +12,8 @@
 #include "env.hpp"
 #include "page_type.hpp"
 #include "typesetter.hpp"
+#include "Boxes/construct.hpp"
+#include "analyze.hpp"
 
 /******************************************************************************
 * Retrieving the page size
@@ -81,6 +83,7 @@ initialize_default_var_type () {
   var_type (PAGE_TYPE)          = Env_Fixed;
   var_type (PAGE_BREAKING)      = Env_Fixed;
   var_type (PAGE_FLEXIBILITY)   = Env_Fixed;
+  var_type (PAGE_FIRST)         = Env_Fixed;
   var_type (PAGE_WIDTH)         = Env_Page_Extents;
   var_type (PAGE_HEIGHT)        = Env_Page_Extents;
   var_type (PAGE_WIDTH_MARGIN)  = Env_Page;
@@ -258,6 +261,92 @@ edit_env_rep::get_page_pars (SI& w, SI& h, SI& width, SI& height,
   */
 }
 
+/******************************************************************************
+* Retrieving ornament parameters
+******************************************************************************/
+
+static tree
+tuplify (tree t) {
+  array<string> a= tokenize (t->label, ",");
+  tree r (TUPLE, N(a));
+  for (int i=0; i<N(a); i++)
+    r[i]= a[i];
+  return r;
+}
+
+ornament_parameters
+edit_env_rep::get_ornament_parameters () {
+  tree  shape = read (ORNAMENT_SHAPE);
+  tree  tst   = read (ORNAMENT_TITLE_STYLE);
+  tree  bg    = read (ORNAMENT_COLOR);
+  tree  xc    = read (ORNAMENT_EXTRA_COLOR);
+  tree  sunny = read (ORNAMENT_SUNNY_COLOR);
+  tree  shadow= read (ORNAMENT_SHADOW_COLOR);
+  int   a     = alpha;
+  tree  w     = read (ORNAMENT_BORDER);
+  tree  ext   = read (ORNAMENT_SWELL);
+  tree  xpad  = read (ORNAMENT_HPADDING);
+  tree  ypad  = read (ORNAMENT_VPADDING);
+
+  array<brush> border;
+  if (is_func (sunny, TUPLE)) {
+    for (int i=0; i<N(sunny); i++)
+      border << brush (sunny[i], a);
+  }
+  else {
+    border << brush (sunny, a);
+    border << brush (shadow, a);
+  }
+  if (N(border) == 1) border << border[0];
+  if (N(border) == 2) border << border[1] << border[0];
+
+  SI lw, bw, rw, tw;
+  if (is_atomic (w) && occurs (",", w->label))
+    w= tuplify (w);
+  if (is_func (w, TUPLE, 4)) {
+    lw= ((as_length (w[0]) >> 1) << 1);
+    bw= ((as_length (w[1]) >> 1) << 1);
+    rw= ((as_length (w[2]) >> 1) << 1);
+    tw= ((as_length (w[3]) >> 1) << 1);
+  }
+  else lw= bw= rw= tw= ((as_length (w) >> 1) << 1);
+
+  double lx, bx, rx, tx;
+  if (is_atomic (ext) && occurs (",", ext->label))
+    ext= tuplify (ext);
+  if (is_func (ext, TUPLE, 4)) {
+    lx= as_double (ext[0]);
+    bx= as_double (ext[1]);
+    rx= as_double (ext[2]);
+    tx= as_double (ext[3]);
+  }
+  else lx= bx= rx= tx= as_double (ext);
+
+  SI lpad, rpad;
+  if (is_atomic (xpad) && occurs (",", xpad->label))
+    xpad= tuplify (xpad);
+  if (is_func (xpad, TUPLE, 2)) {
+    lpad= as_length (xpad[0]);
+    rpad= as_length (xpad[1]);
+  }
+  else lpad= rpad= as_length (xpad);
+
+  SI bpad, tpad;
+  if (is_atomic (ypad) && occurs (",", ypad->label))
+    ypad= tuplify (ypad);
+  if (is_func (ypad, TUPLE, 2)) {
+    bpad= as_length (ypad[0]);
+    tpad= as_length (ypad[1]);
+  }
+  else bpad= tpad= as_length (ypad);
+
+  return ornament_parameters (shape, tst,
+                              lw, bw, rw, tw,
+			      lx, bx, rx, tx,
+			      lpad, bpad, rpad, tpad,
+                              brush (bg, a), brush (xc, a), border);
+}
+
 /******************************************************************************
 * Updating the environment from the variables
 ******************************************************************************/
@@ -356,6 +445,7 @@ edit_env_rep::update_info_level () {
   else if (s == "short") info_level= INFO_SHORT;
   else if (s == "detailed") info_level= INFO_DETAILED;
   else if (s == "paper") info_level= INFO_PAPER;
+  else if (s == "short-paper") info_level= INFO_SHORT_PAPER;
   else info_level= INFO_MINIMAL;
 }
 
@@ -643,6 +733,9 @@ edit_env_rep::update (string s) {
     update_language ();
     update_font ();
     break;
+  case Env_Info_Level:
+    update_info_level ();
+    break;
   case Env_Font:
     update_font ();
     break;
