@@ -30,6 +30,7 @@
 #define SCROLL_BOX    4
 #define TEXT_BOX      5
 #define SHORTER_BOX   6
+#define BIG_OP_BOX    7
 
 /******************************************************************************
 * The cursor class
@@ -182,7 +183,7 @@ public:
   virtual path      find_rip ();
   virtual path      find_left_box_path ();
   virtual path      find_right_box_path ();
-  virtual path      find_box_path (SI x, SI y, SI delta, bool force);
+  virtual path      find_box_path (SI x, SI y, SI d, bool force, bool& found);
   virtual cursor    find_cursor (path bp);
   virtual selection find_selection (path lbp, path rbp);
   virtual path      find_tree_path (path bp);
@@ -207,6 +208,7 @@ public:
   virtual SI        sup_lo_lim  (int level);
   virtual SI        sup_lo_base (int level);
   virtual SI        sup_hi_lim  (int level);
+  virtual void      get_bracket_extents (SI& lo, SI& hi);
 
   /*************************** for graphical boxes ***************************/
 
