@@ -107,6 +107,7 @@
 #define INFO_SHORT         2
 #define INFO_DETAILED      3
 #define INFO_PAPER         4
+#define INFO_SHORT_PAPER   5
 
 /******************************************************************************
 * The edit environment
@@ -134,6 +135,8 @@ public:
   hashmap<string,tree>&        global_ref;
   hashmap<string,tree>&        local_aux;
   hashmap<string,tree>&        global_aux;
+  hashmap<string,tree>&        local_att;
+  hashmap<string,tree>&        global_att;
   bool                         complete;    // typeset complete document ?
   bool                         read_only;   // write-protected ?
   hashmap<string,tree>         missing;     // missing refs
@@ -147,6 +150,7 @@ public:
   double       magn;
   double       mgfy;
   double       flexibility;
+  int          first_page;
   int          mode;
   int          mode_op;
   language     lan;
@@ -260,6 +264,7 @@ private:
   tree exec_change_case (tree t, tree nc, bool exec_flag, bool first);
   tree exec_change_case (tree t);
   tree exec_find_file (tree t);
+  tree exec_find_file_upwards (tree t);
   tree exec_is_tuple (tree t);
   tree exec_lookup (tree t);
   tree exec_equal (tree t);
@@ -268,6 +273,7 @@ private:
   tree exec_lesseq (tree t);
   tree exec_greater (tree t);
   tree exec_greatereq (tree t);
+  tree exec_blend (tree t);
 
   tree exec_cm_length ();
   tree exec_mm_length ();
@@ -308,6 +314,7 @@ private:
   tree exec_find_accessible (tree t);
   tree exec_set_binding (tree t);
   tree exec_get_binding (tree t);
+  tree exec_get_attachment (tree t);
 
   tree exec_pattern (tree t);
 
@@ -347,7 +354,9 @@ public:
 		hashmap<string,tree>& local_ref,
 		hashmap<string,tree>& global_ref,
 		hashmap<string,tree>& local_aux,
-		hashmap<string,tree>& global_aux);
+		hashmap<string,tree>& global_aux,
+		hashmap<string,tree>& local_att,
+		hashmap<string,tree>& global_att);
   void   style_init_env ();
 
   /* execution of trees and setting environment variables */
@@ -477,7 +486,9 @@ class edit_env {
 	    hashmap<string,tree>& local_ref,
 	    hashmap<string,tree>& global_ref,
 	    hashmap<string,tree>& local_aux,
-	    hashmap<string,tree>& global_aux);
+	    hashmap<string,tree>& global_aux,
+	    hashmap<string,tree>& local_att,
+	    hashmap<string,tree>& global_att);
 };
 CONCRETE_NULL_CODE(edit_env);
 
